﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Střelci
{
	/// <summary>
	/// Představuje algoritmus minimax.
	/// </summary>
	public static class Minimax
	{
		private struct Node
		{
			public Move Move;
			public int Value;

			public Node(Move move, int value)
			{
				Move = move;
				Value = value;
			}

			public Node Negate()
			{
				return new Node(Move, -Value);
			}
		}

		private struct CheckMateInfo
		{
			public Move Move;
			public int Depth;

			public CheckMateInfo(Move move, int depth)
			{
				Move = move;
				Depth = depth;
			}
		}

		/// <summary>
		/// Najde nejlepší tah pro hráče v dané partii.
		/// </summary>
		/// <param name="game">Partie, ve které se má tah odehrát.</param>
		/// <param name="playerColor">Barva hráče, který má táhnout.</param>
		/// <param name="depth">Počet půltahů, které se mají promýšlet dopředu.</param>
		public static Move FindBest(Game game, PieceColor playerColor, int depth)
		{
			int nejlepsi = -int.MaxValue;
			int index = -1;
			Stack<MoveInfo> historie = new Stack<MoveInfo>();
			List<int> maty = new List<int>();
			List<CheckMateInfo> vsechny_maty = new List<CheckMateInfo>();

			// nechame si vygenerovat tahy hrace
			Move[] tahy = Generator.GeneratePossibleMoves(game.Chessboard, game,
				playerColor, Judge.ValidMovesFilter);

			// pomocny soudce
			Judge soudce = new Judge(game);

			// projdeme je
			for(int i = 0; i < tahy.Length; i++)
			{
				maty.Clear();

				// vygenerujeme sachovnici po tahu
				Chessboard po_tahu = Judge.ChessboardAfterMove(game, tahy[i]);

				// spocitame jeho vyhodnost
				int hodnoceni = -FindExtreme3(po_tahu, game, Player.ContraryColor(playerColor),
					depth-1, -int.MaxValue, int.MaxValue, historie, soudce, maty, playerColor);

				// pokud nasel mat jednim tahem, skoncili jsme
				if(maty.Contains(depth - 1))
					return tahy[i];
				else
				{
					// jinak si je dame do seznamu matu
					foreach(int hloubka in maty)
						vsechny_maty.Add(new CheckMateInfo(tahy[i], hloubka));
				}

				// hledame nejlepsi
				if(hodnoceni > nejlepsi)
				{
					nejlepsi = hodnoceni;
					index = i;
				}
			}

			// projdem seznam matu a hledame nejlepsi
			int mat_nejlepsi = -1;
			Move mat_tah = null;
			foreach(CheckMateInfo cmi in vsechny_maty)
			{
				if(cmi.Depth > mat_nejlepsi)
				{
					mat_nejlepsi = cmi.Depth;
					mat_tah = cmi.Move;
				}
			}
			// pokud jsme nejaky mat nasli, vratime tah, kterym k nemu dospejeme
			if(mat_tah != null)
				return mat_tah;

			// vratime vysledek
			if(index == -1)
				return null;
			else
				return tahy[index];

			//// najdeme extremni
			////return FindExtreme(game.Chessboard, game, playerColor, true, depth, out hodnoceni);
			//return FindExtreme2(game.Chessboard, game, playerColor, depth,
			//    new Node(null, -int.MaxValue), new Node(null, int.MaxValue), out hodnoceni);
		}

		/// <summary>
		/// Najde extrémní tah.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které se tah provede.</param>
		/// <param name="game">Partie, ve které se tah odehraje.</param>
		/// <param name="playerColor">Barva hráče, pro kterého se hledá nejlepší tah.</param>
		/// <param name="max">Určuje, zda na této úrovni hledat nejlepší tah pro hráče či oponenta.</param>
		/// <param name="depth">Hloubka stromu tahů, do které se má zajít.</param>
		/// <param name="extremeRating">Obdrží hodnocení vráceného tahu.</param>
		private static Move FindExtreme(Chessboard chessboard, Game game, PieceColor playerColor,
			bool max, int depth, out int extremeRating)
		{
			Move[] tahy;
			int index = -1; // index extremniho tahu
			int nejlepsi = int.MinValue;
			PieceColor barva = max ? playerColor : Player.ContraryColor(playerColor);

			// najdeme vsechny mozne tahy hrace
			tahy = Generator.GeneratePossibleMoves(chessboard, game, barva, Judge.ValidMovesFilter);

			// nemuze-li tahnout, vratime null
			if(tahy.Length == 0)
			{
				extremeRating = 0;
				return null;
			}

			// projdeme mozne tahy
			for(int i = 0; i < tahy.Length; i++)
			{
				// vygenerujeme sachovnici po tahu
				Chessboard po_tahu = Judge.ChessboardAfterMove(game, tahy[i]);
				// spocitame jeji hodnoceni
				int hodnoceni = Rating.Rate(po_tahu, game, barva);

				// pokud jsme na vyssi urovni,
				if(depth > 1)
				{
					// spocitame nejlepsi tah protihrace
					int extrem;
					Move tah = FindExtreme(po_tahu, game, playerColor, !max, depth - 1, out extrem);
					// pokud takovy existuje, vezemem ho v uvahu
					if(tah != null)
						hodnoceni += max ? -extrem : extrem;
				}

				// hledame nejlepsi tah
				if(hodnoceni > nejlepsi)
				{
					nejlepsi = hodnoceni;
					index = i;
				}
			}

			// nasli jsme nejlepsi
			extremeRating = nejlepsi;

			// vratime nejlepsi tah
			return tahy[index];
		}

		private static Move FindExtreme2(Chessboard chessboard, Game game, PieceColor playerColor,
			int depth, Node alfa, Node beta, out int extremeRating)
		{
			Move[] tahy;
			int index = 0; // index extremniho tahu

			Debug.Assert(depth > 0, "Hloubka musí být kladné číslo.");

			// zacinat budem s minimalni vhodnosti
			// neni tu MinValue, protoze tuto hodnotu potrebujeme negovat
			extremeRating = -int.MaxValue;

			// najdeme vsechny mozne tahy hrace
			tahy = Generator.GeneratePossibleMoves(chessboard, game, playerColor, Judge.ValidMovesFilter);

			// nemuze-li tahnout, vratime null
			if(tahy.Length == 0)
				return null;

			// projdeme mozne tahy
			for(int i = 0; i < tahy.Length; i++)
			{
				int hodnoceni;
				// vygenerujeme sachovnici po tahu
				Chessboard po_tahu = Judge.ChessboardAfterMove(game, tahy[i]);

				// pokud jsme na konci stromu,
				if(depth == 1)
				{
					// ohodnotime sachovnici po tahu
					hodnoceni = Rating.Rate(po_tahu, game, playerColor);
				}
				else
				{
					// jinak hledame nejlepsi podvetev
					FindExtreme2(po_tahu, game, Player.ContraryColor(playerColor), depth - 1, 
						beta.Negate(), alfa.Negate(), out hodnoceni);
					hodnoceni = -hodnoceni;
				}

				// alfa-beta orezavani
				if(hodnoceni > alfa.Value)
				{
					if(hodnoceni >= beta.Value)
					{
						extremeRating = beta.Value;
						return beta.Move;
					}
					alfa.Value = extremeRating = hodnoceni;
					alfa.Move = tahy[i];
					index = i;
				}

				//// hledame nejlepsi tah
				//if(hodnoceni > extremeRating)
				//{
				//    extremeRating = hodnoceni;
				//    index = i;
				//}
			}

			// extrem bude alfa
			//extremeRating = alfa;

			// vratime nejlepsi tah
			return tahy[index];
		}

		private static int FindExtreme3(Chessboard chessboard, Game game, PieceColor playerColor,
			int depth, int alfa, int beta, Stack<MoveInfo> history, Judge judge, List<int> checkmates,
			PieceColor targetColor)
		{
			Debug.Assert(depth >= 0, "Hloubka musí být kladné číslo.");
			Debug.Assert(beta > alfa, "Neplatný interval alfa-beta.");

			// pokud jsme na konci stromu, vratime jen ohodnoceni
			if(depth == 0)
				return Rating.Rate(chessboard, game, playerColor);

			// najdeme vsechny mozne tahy dalsiho hrace
			Move[] tahy = Generator.GeneratePossibleMoves(chessboard, game,
				playerColor, Judge.ValidMovesFilter);

			// nemame-li nic, je to mat nebo pat
			if(tahy.Length == 0)
			{
				bool mat = Judge.IsCheck(chessboard, game, playerColor);
				// je-li mat protihrace,
				if(mat && playerColor != targetColor)
				{
					// zapamatujem si hloubku, ve kt jsme ho nasli
					checkmates.Add(depth);
				}
				return mat ? Rating.LoseRating : 0;
			}

			// projdeme je
			for(int i = 0; i < tahy.Length; i++)
			{
				// provedeme tah
				history.Push(judge.DoMoveAndDescribe(tahy[i]));

				// pocitame hodnotu
				int hodnoceni = -FindExtreme3(chessboard, game, Player.ContraryColor(playerColor),
					depth - 1, -beta, -alfa, history, judge, checkmates, targetColor);

				// vratime tah
				judge.RevertMove(history.Pop());

				// alfa-beta orezavani
				if(hodnoceni > alfa)
				{
					alfa = hodnoceni;
					if(hodnoceni >= beta)
						return beta;
				}
			}

			return alfa;
		}
	}
}
