﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Střelci
{
	/// <summary>
	/// Ohodnocuje stav šachovnice.
	/// </summary>
	public static class Rating
	{
		public const int PerFigureRating = 50;
		public const int PerCaptureCoeficient = 1;
		public const int OnlyKingRating = -1000;
		
		// V koncovce jsou hodnoty pescu daleko vyssi
		public const int PawnRating = 250;
		public const int BishopRating = 300;
		public const int QueenRating = 3000;

		public const int CheckRating = -190;
		public const int LoseRating = -1000000;
		public const int StaleMateRating = -10000;

		public const int PawnClosingToConvertRatingBase = 3;

		// Ohodnoceni pozice krale na sachovnici
		public static readonly int[,] KingPositionRating = 
			{
			{	 0,  10,  20,  30,  30,  20,  10,   0 },
			{	10,  20,  30,  40,  40,  30,  20,  10 },
			{	20,  30,  40,  50,  50,  40,  30,  20 },
			{	30,  40,  50,  60,  60,  50,  40,  30 },
			{	30,  40,  50,  60,  60,  50,  40,  30 },
			{	20,  30,  40,  50,  50,  40,  30,  20 },
			{	10,  20,  30,  40,  40,  30,  20,  10 },
			{	 0,  10,  20,  30,  30,  20,  10,   0 }
			};

		/// <summary>
		/// Provede ohodnocení stavu na šachovnici vzhledem k zadanému hráči.
		/// </summary>
		/// <param name="chessboard">Šachovnice, která se bude hodnotit.</param>
		/// <param name="game">Partie, ve které se bude hodnotit.</param>
		/// <param name="playerColor">Barva hráče, pro kterého se bude hodnotit.</param>
		/// <returns>Vrací celé číslo.</returns>
		public static int Rate(Chessboard chessboard, Game game, PieceColor playerColor)
		{
			// celkove hodnoceni je hodnoceni z naseho pohledu minus hodnoceni z pohledu oponenta
			int hrac = RateFromPlayerView(chessboard, game, playerColor);
			int protihrac = RateFromPlayerView(chessboard, game, Player.ContraryColor(playerColor));
			// maty se neodecitaji
			if(hrac == LoseRating && protihrac == LoseRating)
				return 0;
			else if(hrac == LoseRating)
				return LoseRating;
			else if(protihrac == LoseRating)
				return -LoseRating;
			else
				return hrac - protihrac;
		}

		/// <summary>
		/// Vrátí hodnocení figurky.
		/// </summary>
		private static int RatePiece(Piece piece)
		{
			int h;

			// vybereme, zhodnotime
			if(piece is Pawn)
				h = Rating.PawnRating;
			else if(piece is Bishop)
				h = Rating.BishopRating;
			else if(piece is Queen)
				h = Rating.QueenRating;
			else
				h = 0;

			return h;
		}

		/// <summary>
		/// Provede ohodnocení stavu na šachovnici z pohledu zadaného hráče.
		/// </summary>
		private static int RateFromPlayerView(Chessboard chessboard, Game game, PieceColor playerColor)
		{
			int hodnota = 0;
			int pocet = 0;

			// projdeme figurky
			foreach(Piece figurka in chessboard)
			{
				// pokud je nase,
				if(figurka.Color == playerColor)
				{
					// hodnotime
					hodnota += RatePiece(figurka);
					pocet++;

					// pokud je to pesec,
					if(figurka is Pawn)
					{
						// zjistime jeho cilovou lajnu
						int cil = game.Judge.GetPawnDestinationRow((Pawn)figurka);
						// zapocitame jeho vzdalenost od cilove lajny
						hodnota += (int)Math.Pow(Rating.PawnClosingToConvertRatingBase,
							(Chessboard.Size - Math.Abs(cil - figurka.Position.Y)));
					}
					// pokud je to kral,
					else if(figurka is King)
					{
						// zapocitame vhodnost jeho pozice
						hodnota += KingPositionRating[figurka.Position.Y, figurka.Position.X]/2;
					}
				}
			}

			// pricteme hodnoceni za figury
			hodnota += Rating.PerFigureRating * pocet;

			// zjistime, co muzeme zajmout
			Move[] zajmuti = Generator.GeneratePossibleMoves(chessboard, game, playerColor, Judge.ValidCapturesFilter);
			// projdeme zajmuti
			foreach(Move tah in zajmuti)
			{
				// ohodnotime kazdou moznost zajmuti
				hodnota += RatePiece(chessboard.GetPieceOnTile(tah.To)) * Rating.PerCaptureCoeficient;
			}

			// zbyl pouze kral
			if(pocet == 1)
				hodnota += Rating.OnlyKingRating;

			// zjistime, zda mame sach
			bool sach = Judge.IsCheck(chessboard, game, playerColor);
			// a pat
			bool pat = Judge.IsStalemate(chessboard, game, playerColor);

			// mame-li sach,
			if(sach)
			{
				// je to spatne
				hodnota += Rating.CheckRating;
				// mame-li mat, je to v haji
				if(pat)
					// Zde nemůžeme hodnocení pouze přičíst, protože jinak by se PC
					// pouze hnal za nejlepším skóre, místo za nejrychlejším matem.
					return Rating.LoseRating;
			}
			else if(pat)
			{
				// jinak mame-li pouze pat, je to spatne
				hodnota += Rating.StaleMateRating;
			}

			return hodnota;
		}
	}
}
