using System;
using System.Diagnostics;
using System.Windows.Forms;

namespace Stelci
{
	partial class ChessForm
	{
		/// <summary>
		/// Zahj editaci novho rozvren achovnice.
		/// </summary>
		protected void DesignNewLayout()
		{
			// pozastavime
			if(Game != null && Game.Running)
				Paused = true;

			// stornujeme prehravani
			CancelReplay();

			// dotaz na ulozeni
			if(!AskForSaving())
				return;

			// zrusime hru
			_CurrentGame = null;

			// pokud tu byla brana, zrusime ji
			if(_Gate != null)
			{
				_Gate.Dispose();
				_Gate = null;
			}

			HighlightedPlayer = SelectedPlayer.None;

			editorToolStrip.Visible = true;

			historyListView.Items.Clear();

			// vytvorime novou branu
			Chessboard sachovnice = new Chessboard();
			EditorGate gate = new EditorGate(sachovnice, new Painter2D(sachovnice, boardPictureBox));
			gate.Painter.HighQualityRendering = Properties.Settings.Default.HighQualityDrawing;
			gate.JudgementChanged += new EventHandler(gate_JudgementChanged);

			_Gate = gate;

			// zvolime nastroje, aby sedely s vychozim nastavenim brany
			pawnToolStripSplitButton.PerformClick();
			whiteToolStripButton.PerformClick();
			whiteOnMoveToolStripMenuItem.PerformClick();

			// novy dokument
			_Document = new FileDocument("nov rozmstn", true);

			RefreshDesignedChessboard();
			RefreshDrawOptions();
			RefreshGameControls();
			RefreshMoveControls();
			RefreshReplay();
			RefreshTitle();
		}

		void gate_JudgementChanged(object sender, EventArgs e)
		{
			RefreshDesignedChessboard();
		}

		/// <summary>
		/// Aktualizuje ovldac prvky editovan achovnice.
		/// </summary>
		protected void RefreshDesignedChessboard()
		{
			EditorGate brana = ((EditorGate)Gate);

			// aktualizujeme popisek
			if(brana.IsValid)
				RefreshJudgeComment();
			else
				RefreshJudgeComment(brana.JudgeComment);

			runToolStripButton.Enabled = brana.IsValid;
		}

		/// <summary>
		/// Zru vbr vech nstroj.
		/// </summary>
		protected void ClearToolsCheck()
		{
			eraserToolStripButton.Checked = pawnToolStripSplitButton.Checked =
				bishopToolStripButton.Checked = queenToolStripButton.Checked =
				kingToolStripButton.Checked = false;
		}

		private void editorToolToolStripButton_Click(object sender, EventArgs e)
		{
			Debug.Assert(Gate is EditorGate, "Nejsme v reimu editace.");

			EditorGate brana = ((EditorGate)Gate);

			ClearToolsCheck();
			// vybereme nastroj
			((ToolStripButton)sender).Checked = true;

			if(sender == pawnToolStripSplitButton)
				brana.Tool = EditorTool.Pawn;
			else if(sender == bishopToolStripButton)
				brana.Tool = EditorTool.Bishop;
			else if(sender == queenToolStripButton)
				brana.Tool = EditorTool.Queen;
			else if(sender == kingToolStripButton)
				brana.Tool = EditorTool.King;
			else
				brana.Tool = EditorTool.Eraser;
		}

		private void editorColorToolStripButton_Click(object sender, EventArgs e)
		{
			Debug.Assert(Gate is EditorGate, "Nejsme v reimu editace.");

			EditorGate brana = ((EditorGate)Gate);

			whiteToolStripButton.Checked = blackToolStripButton.Checked = false;
			// vybereme barvu
			((ToolStripButton)sender).Checked = true;

			if(sender == whiteToolStripButton)
				brana.Color = PieceColor.White;
			else
				brana.Color = PieceColor.Black;
		}

		private void colorOnMoveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Debug.Assert(Gate is EditorGate, "Nejsme v reimu editace.");

			EditorGate brana = ((EditorGate)Gate);

			// vybereme barvu
			whiteOnMoveToolStripMenuItem.Checked = blackOnMoveToolStripMenuItem.Checked = false;
			((ToolStripMenuItem)sender).Checked = true;

			if(sender == whiteOnMoveToolStripMenuItem)
				brana.PlayerOnMoveColor = PieceColor.White;
			else
				brana.PlayerOnMoveColor = PieceColor.Black;
		}

		private void runToolStripButton_Click(object sender, EventArgs e)
		{
			Debug.Assert(Gate is EditorGate, "Nejsme v reimu editace.");

			RunLayout();
		}

		/// <summary>
		/// Spust hru z editovanho rozmstn.
		/// </summary>
		protected void RunLayout()
		{
			Debug.Assert(Gate is EditorGate, "Nejsme v reimu editace.");

			EditorGate brana = ((EditorGate)Gate);

			Debug.Assert(brana.IsValid, "Rozmstn nen platn.");

			RefreshStatus("Probh spoutn partie...", true);

			// vycistime historii
			historyListView.Items.Clear();

			try
			{
				// vytvorime hrace
				Player p1 = new Player(PlayerType.Human, PieceColor.White);
				Player p2 = new Player(PlayerType.Human, PieceColor.Black);

				// vytvorime novou hru
				Game partie = new Game(brana.Chessboard, p1, p2, p1.Color == brana.PlayerOnMoveColor ? p1 : p2);

				// vytvorime dokument
				_Document = new FileDocument("partie z rozmstn");

				// zahajime hru
				BeginGame(partie);
			}
			finally
			{
				RefreshStatus();
			}
		}
	}
}