﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace Střelci
{
	/// <summary>
	/// Představuje hlavní formulář s partií.
	/// </summary>
	partial class ChessForm : Form
	{
		private Gate _Gate;
		private Game _CurrentGame;
		//protected NewGameForm.NewGameInfo _GameInfo;
		private SelectedPlayer _HighlightedPlayer = SelectedPlayer.None;

		protected FileDocument _Document;

		/// <summary>
		/// Počet milisekund, kolik je zvýrazněn tah.
		/// </summary>
		public const int MoveHighlightDuration = 1000;

		/// <summary>
		/// Visačka položky tahu, který může být proveden pomocí Provést znovu.
		/// </summary>
		private const string FutureMoveTag = "future";

		/// <summary>
		/// Zvolený hráč.
		/// </summary>
		public enum SelectedPlayer
		{
			None,
			FirstPlayer,
			SecondPlayer
		}

		/// <summary>
		/// Zvýrazněný hráč.
		/// </summary>
		public SelectedPlayer HighlightedPlayer
		{
			get { return _HighlightedPlayer; }
			set
			{
				_HighlightedPlayer = value;
				// prekreslime
				gameControlsPanel.Invalidate();
			}
		}

		/// <summary>
		/// Vrátí hru, která právě v okně probíhá.
		/// </summary>
		public Game Game
		{
			get { return _CurrentGame; }
		}

		/// <summary>
		/// Vrátí bránu, která zprostředkovává partii.
		/// </summary>
		protected Gate Gate
		{
			get { return _Gate; }
		}

		/// <summary>
		/// Vrátí/nastaví hodnotu určující, zda je hra pozastavena.
		/// </summary>
		public bool Paused
		{
			get
			{
				lock(Game)
				{
					return Game.Paused;
				}
			}
			set
			{
				Debug.Assert(Game != null, "Nemáme partii.");

				lock(Game)
				{

					if(Game == null)
						return;

					Debug.Assert(Game.Running, "Partie již skončila.");

					// pokud hra skoncila, konec
					if(!Game.Running)
						return;

					Game.Paused = value;

					pauseToolStripButton.Checked = value;

					// pokud je na tahu pocitac,
					if(Game.PlayerOnMove.Type == PlayerType.Computer)
					{
						// pri pauznuti ho vypneme
						if(value)
						{
							StopComputerPlayerThinking();
						}
						// jinak ho zapneme
						else
						{
							StartComputerPlayerThinking();
						}
					}
				}
			}
		}

		/// <summary>
		/// Vytvoří formulář s partií.
		/// </summary>
		public ChessForm()
		{
			InitializeComponent();

			// Oprava poradi panelu
			if(toolStripContainer.TopToolStripPanel.Controls.IndexOf(gameToolStrip) > 1)
			{
				toolStripContainer.TopToolStripPanel.Controls.Clear();
				toolStripContainer.TopToolStripPanel.Join(editorToolStrip);
				toolStripContainer.TopToolStripPanel.Join(computerPlayersToolStrip);
				toolStripContainer.TopToolStripPanel.Join(gameToolStrip);
				toolStripContainer.TopToolStripPanel.Join(mainMenuStrip);
				computerPlayersToolStrip.Visible = false;
				editorToolStrip.Visible = false;
			}

			// pridame do imagelistu cernou a bilou
			historyImageList.ImageSize = Properties.Resources.bílá.Size;
			historyImageList.Images.Add("white", Properties.Resources.bílá);
			historyImageList.Images.Add("black", Properties.Resources.černá);
			historyListView.SmallImageList = historyImageList;

			// pripojime se na info o zmene nastaveni
			Properties.Settings.Default.PropertyChanged += new PropertyChangedEventHandler(Settings_PropertyChanged);

			// umistime informaci o nespustene partii
			noGameLabel.Location = gameControlsPanel.Location;
			noGameLabel.Size = gameControlsPanel.Size;

			// napoveda
			helpProvider.HelpNamespace = Program.GetHelpFileName();
#if DEBUG
			// zobrazime menu pro ladeni
			debugFormToolStripSeparator.Visible = true;
			debugFormToolStripMenuItem.Visible = true;
			počítadlaToolStripMenuItem.Visible = true;

			// zobrazime vyuziti pameti
			memoryToolStripStatusLabel.Visible = true;
			monitoringTimer.Enabled = true;
#endif

			RefreshStatus();
			RefreshTitle();
			RefreshGameControls();

			// prizpusobime se nastaveni
			RefreshSettings();
		}

		void Settings_PropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			// obnovime se
			RefreshSettings();
		}

		/// <summary>
		/// Obnoví prvky okna podle uživatelského nastavení.
		/// </summary>
		protected void RefreshSettings()
		{
			// vysoka kvalita vykreslovani
			if(_Gate != null)
				_Gate.Painter.HighQualityRendering = Properties.Settings.Default.HighQualityDrawing;
			// panel historie
			splitContainer.Panel2Collapsed = !Properties.Settings.Default.ShowHistory;
			splitContainer.SplitterDistance = this.Width - Properties.Settings.Default.HistorySplitterDistance;
		}

		private void ChessForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			// stornujeme pripadne vymysleni
			StopGame();
			// a prehravani
			CancelReplay();

			// zapamatujem si
			Properties.Settings.Default.HistorySplitterDistance = this.Width - splitContainer.SplitterDistance;
			// ulozime nastaveni
			Properties.Settings.Default.Save();
			
			// uklid
			if(_Gate != null)
			{
				Painter malir = _Gate.Painter;
				_Gate.Dispose();
				_Gate = null;
				malir.Cleanup();
			}
			if(_CurrentGame != null)
				_CurrentGame = null;
		}

		/// <summary>
		/// Aktualizuje stav aplikace do stavu připravenosti.
		/// </summary>
		protected void RefreshStatus()
		{
			RefreshStatus("Připraven", false);

			// obnovime
			Application.DoEvents();
		}

		/// <summary>
		/// Aktualizuje informaci o stavu aplikace.
		/// </summary>
		/// <param name="statusMessage">Zpráva o stavu aplikace.</param>
		/// <param name="working">Příznak stanovující, zda je aplikace zaneprázdněna.</param>
		protected void RefreshStatus(string statusMessage, bool working)
		{
			// zmenime stav
			stateToolStripStatusLabel.Text = statusMessage;
			stateToolStripStatusLabel.DisplayStyle = working ? ToolStripItemDisplayStyle.ImageAndText :
				ToolStripItemDisplayStyle.Text;

			// obnovime
			Application.DoEvents();
		}

		/// <summary>
		/// Aktualizuje titulek okna na základě právě hrané partie.
		/// </summary>
		protected void RefreshTitle()
		{
			string titulek;

			// pokud nemame dokument, nic tu neni
			if(_Document == null)
				titulek = ProductName;
			// jinak zminime i druh hry
			else
				titulek = string.Format("{0} - {1}", _Document.Title, ProductName);

			// pokud jsme v ladici verzi, pripojime cislo verze
#if DEBUG
			titulek += " [ladící verze " + Application.ProductVersion + "]";
#endif

			// nastavime
			this.Text = titulek;
		}

		/// <summary>
		/// Aktualizuje informace, ovládací prvky a panely nástrojů aktuálně spuštěné partie.
		/// </summary>
		protected void RefreshGameControls()
		{
			if(_CurrentGame == null)
			{
				// pokud jsme v rezimu editace,
				if(_Gate is EditorGate)
				{
					// neco povolime
					gameControlsPanel.Visible = true;
					noGameLabel.Visible = false;

					saveLayoutToolStripButton.Enabled = saveLayoutMenuItem.Enabled = true;
				}
				else
				{
					// jinak neni spusteno nic
					gameControlsPanel.Visible = false;
					noGameLabel.Visible = true;

					// zakazeme polozky menu
					saveLayoutMenuItem.Enabled = false;
					saveLayoutToolStripButton.Enabled = saveLayoutMenuItem.Enabled;
				}

				saveGameMenuItem.Enabled = saveToolStripButton.Enabled = false;
				saveGameAsMenuItem.Enabled = false;
				resetGameMenuItem.Enabled = false;
			}
			else
			{
				// je spustena partie
				gameControlsPanel.Visible = true;
				noGameLabel.Visible = false;

				// povolime polozky menu
				saveLayoutMenuItem.Enabled = _CurrentGame.Running;
				saveGameMenuItem.Enabled = saveToolStripButton.Enabled = true;
				saveGameAsMenuItem.Enabled = true;
				resetGameMenuItem.Enabled = true;
				//changePlayersToolStripMenuItem.Enabled = true;
				// a panelu nastroju
				saveLayoutToolStripButton.Enabled = saveLayoutMenuItem.Enabled;

				//// Zobrazime poprve balonek, aby se pri prvnim zobrazeni neprojevila chyba
				//// .NET Frameworku, kdy se poprve zobrazi hrot balonku na spatnem miste
				//// http://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=98281
				//judgeToolTip.Show("", judgeLabel);
				//judgeToolTip.Hide(judgeLabel);
			}
		}

		/// <summary>
		/// Aktualizuje informace o hráčích spuštěné hry.
		/// </summary>
		protected void RefreshPlayers()
		{
			Debug.Assert(_CurrentGame != null);

			// prvni hrac
			player1ColorPictureBox.BackColor = 
				_CurrentGame.Player1.Color == PieceColor.Black ? Color.Black : Color.White;
			player1NameLabel.Text = _CurrentGame.Player1.Type == PlayerType.Computer ? "Počítač" : "Člověk";

			// druhy hrac
			player2ColorPictureBox.BackColor =
				_CurrentGame.Player2.Color == PieceColor.Black ? Color.Black : Color.White;
			player2NameLabel.Text = _CurrentGame.Player2.Type == PlayerType.Computer ? "Počítač" : "Člověk";
		}

		/// <summary>
		/// Zahájí v okně zadanou partii.
		/// </summary>
		/// <param name="game">Partie, která bude zahájena.</param>
		protected void BeginGame(Game game)
		{
			// ulozime si
			_CurrentGame = game;

			// pripadnou starou branu zrusime
			if(_Gate != null)
			{
				_Gate.Dispose();
			}

			editorToolStrip.Visible = false;

			// vytvorime branu pro partii
			GameGate gate = new GameGate(_CurrentGame, new Painter2D(_CurrentGame.Chessboard, boardPictureBox));
			// nastavime kvalitu vykreslovani
			gate.Painter.HighQualityRendering = Properties.Settings.Default.HighQualityDrawing;

			// pripojime se na zajimave udalosti
			_CurrentGame.PlayerOnMoveChanged += new EventHandler(m_CurrentGame_PlayerOnMoveChanged);
			_CurrentGame.Player1.PlayerChanged += new EventHandler(Player_PlayerChanged);
			_CurrentGame.Player2.PlayerChanged += new EventHandler(Player_PlayerChanged);
			_CurrentGame.NextMoveRequest += new EventHandler(m_CurrentGame_NextMoveRequest);
			_CurrentGame.MoveFinished += new EventHandler<MoveFinishedEventArgs>(m_CurrentGame_MoveFinished);
			_CurrentGame.ChessboardChanged += delegate(object sender, EventArgs e)
				{
					RefreshMoveControls();
				};
			_CurrentGame.Draw += new EventHandler<DrawEventArgs>(_CurrentGame_Draw);
			gate.HumanPlayerMove += new EventHandler<HumanMoveEventArgs>(m_Gate_HumanPlayerMove);
			gate.HumanPlayerInvalidMove += new EventHandler<HumanMoveEventArgs>(m_Gate_InvalidMove);

			_Gate = gate;

			// pokud ji mame spustit,
			if(_CurrentGame.State == GameState.NotStartedYet)
			{
				// spustime
				_CurrentGame.Start();

				Paused = false;
			}

			// aktualizujeme stav
			RefreshTitle();
			RefreshGameControls();
			RefreshPlayers();
			RefreshMoveControls();
			RefreshDrawOptions();
			RefreshReplay();

			Application.DoEvents();
		}

		void m_CurrentGame_NextMoveRequest(object sender, EventArgs e)
		{
			// pokud je na tahu lidsky hrac,
			if(_CurrentGame.PlayerOnMove.Type == PlayerType.Human)
			{
				// nechame ho hrat
			}
			else
			{
				// jinak spustime pocitacoveho hrace
				StartComputerPlayerThinking();
			}
		}

		void m_CurrentGame_MoveFinished(object sender, MoveFinishedEventArgs e)
		{
			bool konec = false;
			MoveInfo mji = e.MoveJudgementInfo;

			// pridame do seznamu tahu
			AddToHistory(mji);

			// zvyraznime ho
			HighlightMove(mji.Move);

			// je-li mat, hlaska
			if(mji.KingInCheckmate != null)
			{
				MessageBox.Show((mji.KingInCheckmate.Color == PieceColor.Black ? "Bílý" : "Černý") +
					" hráč vyhrává.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
				konec = true;
			}
			// je-li pat, hlaska
			if(mji.IsStalemate)
			{
				MessageBox.Show("Nastal pat.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
				konec = true;
			}

			// aktualizujeme zobrazeni podle tahu
			RefreshMoveControls();
			RefreshReplay();

			// soudce mlci
			RefreshJudgeComment();

			// je-li konec, zastavime to
			if(konec)
				StopGame();
			else
			{
				// jinak pokracujem
				Player hrac = Game.GetPlayer(e.MoveJudgementInfo.Move.Piece.Color);

				// pokud to byl tah pocitacoveho hrace,
				if(hrac.Type == PlayerType.Computer)
				{
					// zvazime podani remizy
					DecideForDraw(hrac);
				}

				RefreshDrawOptions();
			}

			// zmenilo se
			_Document.Changed();
		}

		/// <summary>
		/// Obnoví stav ovládacích prvků, které jsou spojeny s tahem.
		/// </summary>
		protected void RefreshMoveControls()
		{
			bool hra_bezi = Game != null && Game.Running;
			bool hra_bezi_a_hraje_clovek = hra_bezi && Game.PlayerOnMove.Type == PlayerType.Human;
			bool hra_zacala = Game != null && Game.State != GameState.NotStartedYet;
			bool bezi_replay = replayTimer.Enabled;

			undoToolStripButton.Enabled = hra_zacala && Game.CanUndo && !bezi_replay;
			redoToolStripButton.Enabled = hra_zacala && Game.CanRedo && !bezi_replay;
			undoToolStripMenuItem.Enabled = undoToolStripButton.Enabled;
			redoToolStripMenuItem.Enabled = redoToolStripButton.Enabled;

			// povolime/zakazeme napovedu tahu
			showBestMoveToolStripMenuItem.Enabled = hra_bezi_a_hraje_clovek;
			showBestMoveToolStripButton.Enabled = hra_bezi_a_hraje_clovek;
			// zmenu hracu
			changePlayersToolStripMenuItem.Enabled = hra_bezi;
			// pauza
			computerPlayersToolStrip.Visible = hra_bezi &&
											   (Game.Player1.Type == PlayerType.Computer) &&
											   (Game.Player2.Type == PlayerType.Computer);

			// ulozeni rozmisteni je mozne, dokud neni hra ukoncena
			saveLayoutToolStripButton.Enabled = saveLayoutMenuItem.Enabled = hra_bezi || Gate is EditorGate;
		}

		void m_CurrentGame_PlayerOnMoveChanged(object sender, EventArgs e)
		{
			// zvyraznime hrace na tahu
			if(_CurrentGame.PlayerOnMove == null)
				HighlightedPlayer = SelectedPlayer.None;
			else if(_CurrentGame.PlayerOnMove == _CurrentGame.Player1)
				HighlightedPlayer = SelectedPlayer.FirstPlayer;
			else
				HighlightedPlayer = SelectedPlayer.SecondPlayer;
		}

		void m_Gate_HumanPlayerMove(object sender, HumanMoveEventArgs e)
		{
			//RefreshStatus("Probíhá zpracování tahu...", true);
			// nastavime zpravu od soudce
			//RefreshJudgeComment(null);
		}

		void m_Gate_InvalidMove(object sender, HumanMoveEventArgs e)
		{
			// nastavime zpravu od soudce
			RefreshJudgeComment(e.Comment);
			// prehrajeme zakazovaci zvuk
			if(Properties.Settings.Default.PlaySounds)
				System.Media.SystemSounds.Exclamation.Play();
			// tah skoncen
			RefreshStatus();
		}

		void Player_PlayerChanged(object sender, EventArgs e)
		{
			Player player = (Player)sender;

			// obnovime ovladaci prvky
			RefreshPlayers();
			RefreshMoveControls();
			RefreshDrawOptions();

			// pokud je zmeneny hrac na tahu,
			if(player == Game.PlayerOnMove)
			{
				// pokud je to pocitac, spustime premysleni
				if(player.Type == PlayerType.Computer)
					StartComputerPlayerThinking();
				// jinak ho naopak zastavime
				else
					StopComputerPlayerThinking();
			}

			_Document.Changed();
		}

		private void newGameMenuItem_Click(object sender, EventArgs e)
		{
			NewGame();
		}

		private void newGameToolStripButton_Click(object sender, EventArgs e)
		{
			NewGame();
		}

		private void aboutMenuItem_Click(object sender, EventArgs e)
		{
			using(AboutForm frm = new AboutForm())
			{
				// zobrazime about
				frm.ShowDialog(this);
			}
		}

		private void saveLayoutMenuItem_Click(object sender, EventArgs e)
		{
			SaveLayout();
		}

		private void layoutSaveFileDialog_FileOk(object sender, CancelEventArgs e)
		{
			Debug.Assert(_Document != null, "Nemáme dokument.");

			try
			{
				// ulozime
				if(Game == null)
					((EditorGate)Gate).Save(layoutSaveFileDialog.FileName);
				else
					IO.SaveLayout(_CurrentGame, layoutSaveFileDialog.FileName, IO.SaveFormat.ForsythEdwardsNotation);
				// ulozeno
				_Document.SavedAs(layoutSaveFileDialog.FileName);
			}
			catch(GameSaveException ex)
			{
				ShowError("Soubor se nepodařilo uložit. " + ex.Message);
				e.Cancel = true;
			}
		}

		/// <summary>
		/// Zobrazí chybovou zprávu.
		/// </summary>
		protected void ShowError(string text)
		{
			MessageBox.Show(text, Application.ProductName, MessageBoxButtons.OK, 
				MessageBoxIcon.Error);
		}

		private void ChessForm_Shown(object sender, EventArgs e)
		{
			// automaticky zobrazime okno pro novou hrou
			NewGame();
			//NewGameLoadLayout(new NewGameForm.NewGameInfo(@"D:\Documents and Settings\Pepa\Plocha\Salvioli 1887.fen", PieceColor.White, PlayerType.Computer, PieceColor.Black, PlayerType.Human));
		}

		private void exitMenuItem_Click(object sender, EventArgs e)
		{
			// zavreme se
			this.Close();
		}

		/// <summary>
		/// Spustí příkaz Nová partie.
		/// </summary>
		protected void NewGame()
		{
			// pozastavime
			if(Game != null && Game.Running)
				Paused = true;

			// stornujeme prehravani
			CancelReplay();

			// dotaz na ulozeni
			if(!AskForSaving())
				return;

			// info o nove hre
			NewGameForm.NewGameInfo ngi;

			using(NewGameForm frm = new NewGameForm())
			{
				// zobrazime dialog
				ngi = frm.ShowNewGameDialog(this);
			}

			// pokud uzivatel zvolil zahajit,
			if(ngi != null)
			{
				// spustime
				NewGameLoadLayout(ngi);
				
				// vycistime obrazovku
				RefreshJudgeComment(null);
			}
			else
			{
				if(Game != null && Game.Running)
					Paused = false;
			}
		}

		/// <summary>
		/// Spustí novou hru načtením rozmístění figurek.
		/// </summary>
		protected void NewGameLoadLayout(NewGameForm.NewGameInfo newGameInfo)
		{
			RefreshStatus("Probíhá vytváření nové partie...", true);

			// vycistime historii
			historyListView.Items.Clear();

			try
			{
				Chessboard sachovnice;
				Player p1, p2;
				string title;
				Player on_move;

				// pokud nam nedali info, pujdem na to sami
				if(newGameInfo == null)
				{
					// posledni sachovnice
					sachovnice = _CurrentGame.StartupChessboard;
					// a posledni hraci
					p1 = (Player)_CurrentGame.Player1.Clone();
					p2 = (Player)_CurrentGame.Player2.Clone();
					// hrac na tahu
					on_move = _CurrentGame.Player1 == _CurrentGame.BeginningPlayer ? p1 : p2;
					// nazev
					title = _Document == null ? "partie" : _Document.Title;
				}
				else
				{
					PieceColor barva;

					// nacteme zadane rozmisteni figurek
					sachovnice = IO.LoadLayout(newGameInfo.LayoutFileName, out barva);
					// a hrace
					p1 = new Player(newGameInfo.Player1Type, newGameInfo.Player1Color);
					p2 = new Player(newGameInfo.Player2Type, newGameInfo.Player2Color);
					p1.ThinkingDepth = newGameInfo.Player1Depth;
					p2.ThinkingDepth = newGameInfo.Player2Depth;
					// hrac na tahu
					on_move = p1.Color == barva ? p1 : p2;
					// nazev
					title = System.IO.Path.GetFileNameWithoutExtension(newGameInfo.LayoutFileName);
				}

				// vytvorime novou hru
				Game partie = new Game(sachovnice, p1, p2, on_move);

				// vytvorime dokument
				_Document = new FileDocument(title);

				// zahajime hru
				BeginGame(partie);
			}
			finally
			{
				RefreshStatus();
			}
		}

		/// <summary>
		/// Nastaví komentář soudce o úspěšném tahu.
		/// </summary>
		protected void RefreshJudgeComment()
		{
			RefreshJudgeComment(string.Empty);
		}

		/// <summary>
		/// Nastaví komentář soudce k poslednímu tahu.
		/// </summary>
		/// <param name="comment">Komentář soudce. Je-li null, zobrazený komentář se skryje.
		/// Je-li prázdný, zobrazí se poznámka o úspěšném tahu.</param>
		protected void RefreshJudgeComment(string comment)
		{
			if(string.IsNullOrEmpty(comment))
			{
				// skryjeme
				//judgeToolTip.Hide(judgeLabel);
				judgeCommentLabel.Text = comment == null ? "Komentář soudce." : "Situace je v pořádku.";
				judgeCommentLabel.ForeColor = SystemColors.ControlText;
				judgeCommentPictureBox.Image = Properties.Resources.Information;
			}
			else
			{
				// zobrazime zpravu
				//judgeToolTip.Show(comment, judgeLabel, 3000);
				judgeCommentLabel.Text = comment;
				judgeCommentLabel.ForeColor = Color.Red;
				judgeCommentPictureBox.Image = Properties.Resources.WarningHS;
			}
		}

		/// <summary>
		/// Nevratně zastaví počítačové hráče.
		/// </summary>
		protected void StopGame()
		{
			// zastavime pocitac
			StopComputerPlayerThinking();

			// vycistime ovladaci prvky
			drawRequestLabel.Text = string.Empty;
			acceptDrawLinkLabel.Visible = false;
			drawToolStripButton.Enabled = drawToolStripMenuItem.Enabled = false;

			//showBestMoveToolStripButton.Enabled = showBestMoveToolStripMenuItem.Enabled = false;
			//undoToolStripButton.Enabled = undoToolStripMenuItem.Enabled = false;
			//changePlayersToolStripMenuItem.Enabled = false;

			// pockame, az pocitac dopremysli
			while(aiBackgroundWorker.IsBusy)
			{
				Application.DoEvents();
			}
		}

		private void computerPlayerTimer_Tick(object sender, EventArgs e)
		{
			// byla-li hra zrusena, nic
			if(Game == null)
				return;

			// vypneme premysleni
			StopComputerPlayerThinking();

			// je-li hra pauznuta, konec
			if(Game.Paused)
				return;

			// bude se cekat
			RefreshStatus("Počítačový hráč vymýšlí tah...", true);

			// spustime premysleni
			aiBackgroundWorker.RunWorkerAsync(Game);
		}

		private void settingsMenuItem_Click(object sender, EventArgs e)
		{
			using(SettingsForm frm = new SettingsForm())
			{
				// zobrazime nastaveni
				frm.ShowDialog(this);
			}
		}

		private void gameControlsPanel_Paint(object sender, PaintEventArgs e)
		{
			if(_HighlightedPlayer == SelectedPlayer.None)
			{
				// barvy popisek
				player1NameLabel.ForeColor = SystemColors.ControlText;
				player2NameLabel.ForeColor = SystemColors.ControlText;
			}
			else
			{
				Color HighlightColor = Color.Green;
				Rectangle rect;
				int sirka = gameControlsPanel.ClientSize.Width;// / 2;
				LinearGradientBrush stetec;

				rect = new Rectangle(0, 0, sirka, gameControlsPanel.ClientSize.Height);

				if(_HighlightedPlayer == SelectedPlayer.SecondPlayer)
				{
					// ramecek zvyrazneni a stetec
					//rect = new Rectangle(0, 0, sirka, gameControlsPanel.ClientSize.Height);
					// vytvorime stetec
					stetec = new LinearGradientBrush(rect, HighlightColor, gameControlsPanel.BackColor, LinearGradientMode.Vertical);
					// barvy popisek
					player1NameLabel.ForeColor = SystemColors.ControlText;
					player2NameLabel.ForeColor = Color.White;
				}
				else
				{
					// ramecek zvyrazneni a stetec
					//rect = new Rectangle(0, gameControlsPanel.ClientSize.Height / 2,
					//    sirka, gameControlsPanel.ClientSize.Height / 2);
					// vytvorime stetec
					stetec = new LinearGradientBrush(rect, gameControlsPanel.BackColor, HighlightColor, LinearGradientMode.Vertical);
					// barvy popisek
					player1NameLabel.ForeColor = Color.White;
					player2NameLabel.ForeColor = SystemColors.ControlText;
				}

				try
				{
					// vykreslime prechod
					e.Graphics.FillRectangle(stetec, rect);
				}
				finally
				{
					// uklid
					stetec.Dispose();
				}
			}
		}

		private void gameControlsPanel_Resize(object sender, EventArgs e)
		{
			// nechame prekreslit
			gameControlsPanel.Invalidate();
		}

		private void debugFormToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// zobrazime ladici okno
			DebugForm frm = new DebugForm();

			// zobrazime ho
			frm.Show(this);
		}

		private void resetGameMenuItem_Click(object sender, EventArgs e)
		{
			DialogResult dr;

			// bezi-li hra,
			if(_CurrentGame.Running)
				// bezpecnostni dotaz
				dr = MessageBox.Show("Chcete opravdu spustit tuto partii znovu?", ProductName,
					MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
			else
				// jinak ho nechame bez dotazu
				dr = DialogResult.Yes;

			// pripadne resetujeme
			if(dr == DialogResult.Yes)
				ResetGame();
		}

		/// <summary>
		/// Spustí znovu aktuální partii.
		/// </summary>
		protected void ResetGame()
		{
			Debug.Assert(_CurrentGame != null, "Není spuštěna partie.");

			StopGame();
			// stornujeme prehravani
			CancelReplay();

			// spustime tu samou
			NewGameLoadLayout(null);
		}

		/// <summary>
		/// Smaže ze seznamu historie vrácené tahy.
		/// </summary>
		private void ClearFuture()
		{
			// pokud jsou v historii nejake budouci tahy (pro redo)
			if(historyListView.Items.Count > 0 &&
			   (string)historyListView.Items[historyListView.Items.Count - 1].Tag == FutureMoveTag)
			{
				// musime je vsechny smazat
				historyListView.BeginUpdate();
				for(int i = 0; i < historyListView.Items.Count; i++)
				{
					if((string)historyListView.Items[i].Tag == FutureMoveTag)
					{
						historyListView.Items.RemoveAt(i--);
						continue;
					}
				}
				historyListView.EndUpdate();
			}
		}

		/// <summary>
		/// Přidá záznam do historie tahů.
		/// </summary>
		protected void AddToHistory(MoveInfo moveInfo)
		{
			ClearFuture();

			// pridame polozku
			ListViewItem li = historyListView.Items.Add(moveInfo.AlgebraicNotation);
			// barva hrace
			if(moveInfo.Move.Piece.Color == PieceColor.White)
				li.ImageKey = "white";
			else
				li.ImageKey = "black";
			// skupina
			li.Group = historyListView.Groups["finished"];
			// zobrazime
			li.EnsureVisible();
			// prizpusobime sirku sloupce
			historyListView.Columns[0].AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
		}

		/// <summary>
		/// Přidá do historie záznam o remíze.
		/// </summary>
		protected void AddDrawToHistory()
		{
			Debug.Assert(historyListView.Items.Count > 0, "Historie je prázdná, nemohlo dojít k remíze.");

			ClearFuture();

			// vezmeme posledni
			ListViewItem li = historyListView.Items[historyListView.Items.Count - 1];

			// pridame ji oznaceni remizy
			li.Text += Game.GetDrawDescription();
		}

		private void helpContentsMenuItem_Click(object sender, EventArgs e)
		{
			// zobrazime obsah
			Help.ShowHelp(this, helpProvider.HelpNamespace);
		}

		private void monitoringTimer_Tick(object sender, EventArgs e)
		{
			try
			{
				// ziskame velikost fyzicke pameti, kterou mame; bude v kB
				long pamet = Process.GetCurrentProcess().WorkingSet64 / 1024;
				// zobrazime vyuziti pameti
				memoryToolStripStatusLabel.Text = string.Format("Využití paměti: {0:#,##0} kB", pamet);
			}
			catch(PlatformNotSupportedException)
			{
				// jsme na Win98, WinMe, vyuziti pameti nezjistime
				monitoringTimer.Enabled = false;
				memoryToolStripStatusLabel.Visible = false;
			}
		}

		// tento handler se pouziva v aiBackgroundWorker i bestMoveBackgroundWorker
		private void aiBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
		{
			Game hra = (Game)e.Argument;
			
			// pokud bylo storno, konec
			if(((BackgroundWorker)sender).CancellationPending)
			{
				e.Cancel = true;
				return;
			}

			// najdeme nejlepsi tah
			Move tah = ComputerPlayer.FindBestMove(hra, hra.PlayerOnMove);

			// pokud nebylo storno, vratime ho
			if(((BackgroundWorker)sender).CancellationPending)
				e.Cancel = true;
			else
				e.Result = tah;
		}

		private void aiBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			// obnovime stav
			RefreshStatus();

			// pokud byla operace stornovana, konec
			if(e.Cancelled)
				return;

			// pokud doslo k chybe,
			if(e.Error != null)
			{
				// osetrime
				if(e.Error is NullReferenceException)
				{
					Debug.Fail("Při zavírání okna došlo k neočekávané chybě.");
				}
				else
					throw e.Error;
			}

			// pokud jsme mezitim prisli o partii, konec
			if(Game == null)
				return;

			// vysledny tah
			Move tah = (Move)e.Result;

			// neni-li tah, vyhodime chybu
			if(tah == null)
				throw new InvalidOperationException("Pro hráče " + _CurrentGame.PlayerOnMove + " už neexistuje žádný platný tah.");

			// pokud hraje za cloveka (ignorovane stornovani), konec
			if(Game.GetPlayer(tah.Piece.Color).Type == PlayerType.Human)
				return;

			Debug.Assert(tah.Piece.Color == Game.PlayerOnMove.Color, "Počítač vymyslel tah pro jiného hráče, než je na tahu.");

			// nechame zvazit nabidku k remize; pokud k ni nedoslo,
			if(!DecideForDraw(Game.GetPlayer(tah.Piece.Color)))
			{
				// provedeme tah
				_CurrentGame.DoMove(tah);
			}
		}

		private void počítadlaToolStripMenuItem_Click(object sender, EventArgs e)
		{
			CounterForm frm = new CounterForm();

			frm.Show(this);
		}

		/// <summary>
		/// Ukáže hráči jeho nejlepší možný tah.
		/// </summary>
		protected void ShowBestMove()
		{
			Debug.Assert(Game != null, "Partie je fuč.");
			Debug.Assert(Game.Running, "Partie už skončila.");
			Debug.Assert(Game.PlayerOnMove.Type == PlayerType.Human, "Na tahu není člověk.");

			// pokud uz hledani nejlepsiho tahu bezi, konec
			if(bestMoveBackgroundWorker.IsBusy)
				return;

			RefreshStatus("Hledání nejlepšího tahu...", true);

			// spustime vypocet
			bestMoveBackgroundWorker.RunWorkerAsync(Game);
		}

		private void showBestMoveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ShowBestMove();
		}

		private void showBestMoveToolStripButton_Click(object sender, EventArgs e)
		{
			ShowBestMove();
		}

		private void bestMoveBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			// obnovime stav
			RefreshStatus();
			
			// pokud bylo zruseno, konec
			if(e.Cancelled)
				return;

			// pokud doslo k chybe,
			if(e.Error != null)
			{
				// osetrime
				if(e.Error is NullReferenceException)
				{
					Debug.Fail("Při zavírání okna došlo k neočekávané chybě.");
				}
				else
					throw e.Error;
			}

			// vysledny tah
			Move tah = (Move)e.Result;

			// neni-li tah, vyhodime chybu
			if(tah == null)
				throw new InvalidOperationException("Pro hráče " + Game.PlayerOnMove + " už neexistuje žádný platný tah.");

			// zvyraznime ho
			HighlightMove(tah);
		}

		/// <summary>
		/// Zahájí přemýšlení počítačového hráče nad dalším tahem.
		/// </summary>
		private void StartComputerPlayerThinking()
		{
			// spustime casovac
			computerPlayerTimer.Enabled = true;
		}

		/// <summary>
		/// Ukončí přemýšlení počítačového hráče nad dalším tahem.
		/// </summary>
		private void StopComputerPlayerThinking()
		{
			// zastavime casovac
			computerPlayerTimer.Enabled = false;
			// stornujeme pripadne premysleni
			aiBackgroundWorker.CancelAsync();
			// pockame, az dopremysli
			while(aiBackgroundWorker.IsBusy)
				Application.DoEvents();
		}

		private void changePlayersToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// pozastavime hru
			Paused = true;

			// ziskame hrace
			Player bily = Game.GetPlayer(PieceColor.White);
			Player cerny = Game.GetPlayer(PieceColor.Black);

			using(ChangePlayersForm frm = new ChangePlayersForm(bily, cerny))
			{
				// zobrazime okno; pokud dal uzivatel ok,
				if(frm.ShowDialog(this) == DialogResult.OK)
				{
					// zmenime druhy hracu
					if(frm.WhitePlayerTypeChanged)
						bily.Type = frm.WhitePlayerType;
					if(frm.BlackPlayerTypeChanged)
						cerny.Type = frm.BlackPlayerType;
					bily.ThinkingDepth = frm.WhitePlayerDepth;
					cerny.ThinkingDepth = frm.BlackPlayerDepth;
				}
			}

			// obnovime hru
			Paused = false;
		}

		private void pauseToolStripButton_Click(object sender, EventArgs e)
		{
			Paused = pauseToolStripButton.Checked;
		}

		private void undoToolStripButton_Click(object sender, EventArgs e)
		{
			// obnovime pripadne ukoncenou hru
			Game.Resume();

			// pokud hrajou 2 pocitace,
			if(Game.Player1.Type == PlayerType.Computer && Game.Player2.Type == PlayerType.Computer)
			{
				// pauzneme a vratime se o jeden
				Paused = true;
				NavigateTime(-1);
			}
			else
			{
				// jinak vratime na tah lidskeho hrace
				NavigateTime(Game.GetPlayer(Player.ContraryColor(Game.PlayerOnMove.Color)).Type == PlayerType.Computer
					? -2 : -1);
			}
		}

		private void redoToolStripButton_Click(object sender, EventArgs e)
		{
			// obnovime pripadne ukoncenou hru
			Game.Resume();

			// pokud hrajou 2 pocitace,
			if(Game.Player1.Type == PlayerType.Computer && Game.Player2.Type == PlayerType.Computer)
			{
				// pauzneme a posuneme se o jeden
				Paused = true;
				NavigateTime(1);
			}
			else
			{
				// posuneme na tah lidskeho hrace
				NavigateTime(Game.GetPlayer(Player.ContraryColor(Game.PlayerOnMove.Color)).Type == PlayerType.Computer
					? 2 : 1);
			}
		}

		/// <summary>
		/// Posune partii o zadaný počet půltahů dopředu (nebo zpátky, když je zadáno záporné číslo).
		/// </summary>
		/// <param name="moveCount">Počet půltahů, které se mají posunout, kladné nebo záporné.</param>
		public void NavigateTime(int moveCount)
		{
			if(Game == null)
			{
				Debug.Fail("Nemáme partii.");
				return;
			}

			// pokud ted premysli pocitac,
			if(aiBackgroundWorker.IsBusy)
			{
				// stornujeme ho
				StopComputerPlayerThinking();
			}

			// posuneme se; pokud se zdarilo,
			if(Game.NavigateTime(moveCount, replayTimer.Enabled))
			{
				// pokud jsme sli dopredu,
				if(moveCount > 0)
				{
					// oznacime predchozi tah jako provedeny
					for(int i = moveCount - 1; i >= 0; i--)
					{
						RedoHistoryItem(Game.CurrentMoveNumber - (2 + i));
					}
				}
				else
				{
					// jinak ho oznacime jako minuly
					UndoHistoryItem(Game.CurrentMoveNumber - 1);
				}
			}

			RefreshDrawOptions();
			RefreshReplay();
		}

		/// <summary>
		/// Přesune položku historie do vrácených.
		/// </summary>
		private void UndoHistoryItem(int index)
		{
			historyListView.BeginUpdate();

			// vezmeme polozku
			ListViewItem li = historyListView.Items[index];
			// odebereme ji
			li.Remove();

			// udelame si seznam vracenych
			List<ListViewItem> vracene = new List<ListViewItem>(historyListView.Items.Count - index);
			vracene.Add(li);
			// projdeme polozky na ni
			for(int i = index; i < historyListView.Items.Count; i++)
			{
				// pridame si je do seznamu
				vracene.Add(historyListView.Items[i]);
				// a odebereme je z ListViewu 
				historyListView.Items.RemoveAt(i);
				i--;
				continue;
			}

			// nyni je tam pridame zpatky
			foreach(ListViewItem p in vracene)
			{
				historyListView.Items.Add(p);
				// oznacime je jako vracene
				p.ForeColor = SystemColors.ButtonShadow;
				p.Tag = FutureMoveTag;
				p.Group = historyListView.Groups["undo"];
			}

			historyListView.EndUpdate();
		}

		/// <summary>
		/// Přesune položku historie do provedených.
		/// </summary>
		private void RedoHistoryItem(int index)
		{
			// vezmeme polozku
			ListViewItem li = historyListView.Items[index];

			li.ForeColor = SystemColors.WindowText;
			li.Tag = "";
			li.Group = historyListView.Groups["finished"];
		}

		/// <summary>
		/// Zvýrazní na šachovnici zadaný tah.
		/// </summary>
		/// <param name="move">Tah, který se má zvýraznit.</param>
		protected void HighlightMove(Move move)
		{
			Position from = move.From;
			Position to = move.To;

			// pridame policka tahu do zvyraznenych
			Gate.Painter.HighlightedTiles.Add(from);
			Gate.Painter.HighlightedTiles.Add(to);

			// vytvorime casovac pro jejich opetovne skryti
			Timer casovac = new Timer();
			casovac.Interval = MoveHighlightDuration;
			casovac.Tick += delegate(object tsender, EventArgs te)
				{
					try
					{
						casovac.Stop();
						// skryjeme zvyraznena pole
						Gate.Painter.HighlightedTiles.Remove(from);
						Gate.Painter.HighlightedTiles.Remove(to);
						// prekreslime sachovnici
						Gate.Painter.Refresh();
					}
					finally
					{
						// uklid
						casovac.Dispose();
					}
				};
			// spustime
			casovac.Start();
		}

		private void drawToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Debug.Assert(Game != null && Game.Running, "Hra skončila.");

			// pokud hraji dva lide,
			if(Game.Player1.Type == PlayerType.Human && Game.Player2.Type == PlayerType.Human)
			{
				// ukoncime hru odeslanim dvou nabidek
				if(!Game.OfferDraw(Game.Player1))
					Game.OfferDraw(Game.Player2);
			}
			else
			{
				Debug.Assert(Game.PlayerOnMove.Type == PlayerType.Human, "Teď není na tahu člověk.");

				// nabidneme
				Game.OfferDraw(Game.PlayerOnMove);
			}
		}

		private void saveGameMenuItem_Click(object sender, EventArgs e)
		{
			Save();
		}

		/// <summary>
		/// Provede uložení partie.
		/// </summary>
		/// <returns>Hodnota určující, zda bylo uložení provedeno.</returns>
		protected bool Save()
		{
			return Save(false);
		}

		/// <summary>
		/// Provede uložení partie.
		/// </summary>
		/// <param name="forceDialog">Hodnota určující, zda zobrazit dialog pro pojmenování souboru.</param>
		/// <returns>Hodnota určující, zda bylo uložení provedeno.</returns>
		protected bool Save(bool forceDialog)
		{
			Debug.Assert(_Document != null, "Nemáme co ukládat.");
			Debug.Assert(Game != null, "Nemáme partii.");

			// stornujeme prehravani
			CancelReplay();

			// pokud mame zobrazit dialog nebo nemame jmeno,
			if(forceDialog || !_Document.IsNamed)
			{
				// ucinime tak
				if(_Document.IsNamed)
					saveGameFileDialog.FileName = _Document.FileName;
				// pokud dal storno, konec
				if(saveGameFileDialog.ShowDialog() == DialogResult.Cancel)
					return false;

				// zapamatujem si jmeno
				_Document.FileName = saveGameFileDialog.FileName;
			}

			// ulozime
			IO.Save(Game, _Document.FileName);

			// hotovo
			_Document.Saved();
			return true;
		}

		/// <summary>
		/// Provede uložení otevřeného rozvržení šachovnice.
		/// </summary>
		/// <returns>True, pokud se uložení podařilo.</returns>
		protected bool SaveLayout()
		{
			// nemame-li partii,
			if(_CurrentGame == null)
			{
				EditorGate brana = Gate as EditorGate;

				Debug.Assert(brana != null, "Nemáme partii a přitom nejsme v režimu editace.");

				// pokud neni situace ok,
				if(!brana.IsValid)
				{
					// upozornime
					MessageBox.Show("Situace na šachovnici není platná podle pravidel. " +
						"Odstraňte problém, který popisuje komentář soudce.",
						ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
					// a ulozit nepovolime
					return false;
				}
			}
			// otevreme dialog; se nezdarilo, konec
			if(layoutSaveFileDialog.ShowDialog() != DialogResult.OK)
			{
				return false;
			}

			return true;
		}

		/// <summary>
		/// Pokud je potřeba, dotáže se uživatele, zda aktuální partii uložit a vrátí hodnotu určující,
		/// zda byla operace dokončena.
		/// </summary>
		protected bool AskForSaving()
		{
			// pokud byla predchozi a nebyla ulozena,
			if(_Document != null && _Document.NeedsSave)
			{
				// zjistime, zda mame partii
				bool partie = Game != null;
				string dotaz;

				// zvolime hlasku
				if(partie)
					dotaz = "Spuštěná partie nebyla uložena. Chcete ji nyní uložit?";
				else
					dotaz = "Otevřené rozvržení nebylo uloženo. Chcete ho nyní uložit?";

				// zeptame se
				DialogResult dr = MessageBox.Show(dotaz,
					ProductName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);

				// pokud dal storno, stornujeme otevreni
				if(dr == DialogResult.Cancel)
				{
					return false;
				}
				// pokud dal ano, ulozime
				else if(dr == DialogResult.Yes)
				{
					if(partie)
					{
						if(!Save())
							return false;
					}
					else
					{
						if(!SaveLayout())
							return false;
					}
				}
				// jinak nic
			}

			return true;
		}

		/// <summary>
		/// Provede načtení partie.
		/// </summary>
		/// <returns>Hodnota určující, zda k načtení došlo.</returns>
		protected bool LoadGame()
		{
			// pozastavime
			if(Game != null && Game.Running)
				Paused = true;

			// stornujeme prehravani
			CancelReplay();

			// dotaz na ulozeni
			if(!AskForSaving())
				return false;

			// zobrazime dialog; pokud dal ok,
			if(loadGameFileDialog.ShowDialog() == DialogResult.OK)
			{
				Game partie;

				try
				{
					// nacteme
					partie = IO.Load(loadGameFileDialog.FileName);
				}
				catch(GameLoadException ex)
				{
					MessageBox.Show("Při načítání partie došlo k chybě. " + ex.Message, ProductName,
						MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
					return false;
				}

				// vytvorime dokument
				_Document = new FileDocument(
					System.IO.Path.GetFileNameWithoutExtension(loadGameFileDialog.FileName));
				_Document.FileName = loadGameFileDialog.FileName;

				// vycistime obrazovku
				RefreshJudgeComment(null);

				// zahajime hru
				BeginGame(partie);

				// nasypeme tam nactenou historii
				FillHistory(partie);

				// rucne zvyraznime hrace na tahu
				m_CurrentGame_PlayerOnMoveChanged(this, EventArgs.Empty);

				return true;
			}
			return false;
		}

		/// <summary>
		/// Naplní seznam historií zadané partie.
		/// </summary>
		/// <param name="game"></param>
		protected void FillHistory(Game game)
		{
			// vycistime historii
			historyListView.Items.Clear();

			// ziskame historii
			MoveInfo[] historie = game.GetHistory();

			// naplnime
			for(int i = 0; i < historie.Length; i++)
			{
				AddToHistory(historie[i]);
			}
			// konci-li remizou, ale nikoliv patem, pridame
			if(game.State == GameState.Draw && historie.Length > 0 && !historie[historie.Length - 1].IsStalemate)
				AddDrawToHistory();
		}

		private void saveGameAsMenuItem_Click(object sender, EventArgs e)
		{
			Save(true);
		}

		private void ChessForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			// dotaz na ulozeni
			e.Cancel = !AskForSaving();
		}

		private void loadGameMenuItem_Click(object sender, EventArgs e)
		{
			LoadGame();
		}

		private void designLayoutMenuItem_Click(object sender, EventArgs e)
		{
			DesignNewLayout();
		}
	}
}