﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Střelci
{
	/// <summary>
	/// Brána pro editor šachovnice.
	/// </summary>
	class EditorGate : Gate
	{
		private EditorTool _Tool = EditorTool.Eraser;
		private Chessboard _Chessboard;
		private PieceColor _PlayerOnMove;
		private PieceColor _Color = PieceColor.White;
		private Game _DummyGame;
		private string _JudgeComment;
		private bool _IsValid;

		/// <summary>
		/// Vrátí komentář soudce.
		/// </summary>
		public string JudgeComment
		{
			get { return _JudgeComment; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda je rozmístění platné podle pravidel.
		/// </summary>
		public bool IsValid
		{
			get { return _IsValid; }
		}

		/// <summary>
		/// Vrátí pomocnou partii.
		/// </summary>
		protected Game DummyGame
		{
			get { return _DummyGame; }
		}

		/// <summary>
		/// Vrátí/nastaví nástroj editoru.
		/// </summary>
		public EditorTool Tool
		{
			get { return _Tool; }
			set { _Tool = value; }
		}

		/// <summary>
		/// Vrátí/nastaví aktuálně zvolenou barvu figurek.
		/// </summary>
		public PieceColor Color
		{
			get { return _Color; }
			set { _Color = value; }
		}

		/// <summary>
		/// Vrátí editovanou šachovnici.
		/// </summary>
		public Chessboard Chessboard
		{
			get { return _Chessboard; }
		}

		/// <summary>
		/// Vrátí/nastaví barvu hráče na tahu.
		/// </summary>
		public PieceColor PlayerOnMoveColor
		{
			get { return _PlayerOnMove; }
			set { _PlayerOnMove = value; }
		}

		/// <summary>
		/// Nastane při změně posouzení.
		/// </summary>
		public event EventHandler JudgementChanged;

		/// <summary>
		/// Vytvoří novou bránu pro editor.
		/// </summary>
		/// <param name="chessboard">Šachovnice, která se bude navrhovat.</param>
		/// <param name="painter">Malíř, který bude vykreslovat šachovnici.</param>
		public EditorGate(Chessboard chessboard, Painter painter) : base(painter)
		{
			_Chessboard = chessboard;

			// vytvorime falesnou partii
			_DummyGame = new Game(Chessboard, new Player(PlayerType.Human, PieceColor.White),
											  new Player(PlayerType.Human, PieceColor.Black));
			_DummyGame.Start();

			RefreshJudgement();

			// zacinat bude bila
			PlayerOnMoveColor = PieceColor.White;
		}

		/// <summary>
		/// Aktualizuje posouzení soudcem.
		/// </summary>
		protected void RefreshJudgement()
		{
			// overime
			_IsValid = Judge.IsValid(DummyGame, PieceColor.White, out _JudgeComment) &&
				Judge.IsValid(DummyGame, PieceColor.Black, out _JudgeComment);

			// oznamime
			if(JudgementChanged != null)
				JudgementChanged(this, EventArgs.Empty);
		}

		protected override void m_OutputControl_MouseClick(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// policko, na ktere se kliklo
			Position? policko = Painter.GetTileFromPoint(e.X, e.Y);

			// pokud klikl na sachovnici, zavolame obsluhu
			if(policko != null)
				ToolClick(policko.Value);
		}

		protected void ToolClick(Position position)
		{
			Piece figurka = Chessboard.GetPieceOnTile(position);

			// pokud na sachovnici neco je, dame to pryc
			if(figurka != null)
				Chessboard.RemovePiece(figurka);

			// podle nastroje vytvorime novou figurku
			switch(Tool)
			{
				case EditorTool.Eraser:
					// nic, figurka je smazana
					figurka = null;
					break;
				case EditorTool.Pawn:
					figurka = new Pawn(Chessboard, position, Color);
					break;
				case EditorTool.Bishop:
					figurka = new Bishop(Chessboard, position, Color);
					break;
				case EditorTool.King:
					figurka = new King(Chessboard, position, Color);
					break;
				case EditorTool.Queen:
					figurka = new Queen(Chessboard, position, Color);
					break;
				default:
					Debug.Fail("Neznámý nástroj " + Tool);
					figurka = null;
					break;
			}

			// pokud byla vytvorena nejaka figurka, pridame ji
			if(figurka != null)
				Chessboard.AddPiece(figurka);

			RefreshJudgement();

			// prekreslime
			Painter.Refresh();
		}

		/// <summary>
		/// Uloží návrh do souboru ve formátu FEN.
		/// </summary>
		/// <param name="fileName">Název souboru.</param>
		public void Save(string fileName)
		{
			Debug.Assert(IsValid, "Situace na šachovnici není platná.");

			// ulozime
			IO.SaveLayout(DummyGame, fileName, IO.SaveFormat.ForsythEdwardsNotation);
		}
	}

	/// <summary>
	/// Nátroj editoru.
	/// </summary>
	enum EditorTool
	{
		/// <summary>
		/// Guma.
		/// </summary>
		Eraser,
		/// <summary>
		/// Pěšec.
		/// </summary>
		Pawn,
		/// <summary>
		/// Střelec.
		/// </summary>
		Bishop,
		/// <summary>
		/// Král.
		/// </summary>
		King,
		/// <summary>
		/// Královnka.
		/// </summary>
		Queen
	}
}
