﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;

namespace Střelci
{
	/// <summary>
	/// Představuje výjimku, která nastane, když se v aplikaci objeví neznámý druh figurky.
	/// </summary>
	public class UnknownPieceException : Exception
	{
		/// <summary>
		/// Vytvoří novou výjimku.
		/// </summary>
		public UnknownPieceException() : base()
		{
		}

		/// <summary>
		/// Vytvoří novou výjimku se zadanou chybovou zprávou.
		/// </summary>
		/// <param name="message">Zpráva, která popisuje chybu.</param>
		public UnknownPieceException(string message) : base(message)
		{
		}

		/// <summary>
		/// Vytvoří novou výjimku se zadanou chybovou zprávou a předchozí výjimkou.
		/// </summary>
		/// <param name="message">Zpráva, která popisuje chybu.</param>
		/// <param name="innerException">Výjimka, která způsobila vyvolání této výjimky.</param>
		public UnknownPieceException(string message, Exception innerException)
			: base(message, innerException)
		{
		}

		/// <summary>
		/// Vytvoří novou výjimku s implicitní zprávou.
		/// </summary>
		/// <param name="piece">Figurka, jejíž druh je neznámý.</param>
		public UnknownPieceException(Piece piece) :
			base("Figurka \"" + piece.ToString() + "\" je neznámého typu " + piece.GetType().ToString() + ".")
		{
		}
	}

	/// <summary>
	/// Představuje obecnou šachovou figurku.
	/// </summary>
	public abstract class Piece
	{
		private Chessboard _Chessboard;
		private Position _Position;
		private PieceColor _Color;

		/// <summary>
		/// Barva figurky.
		/// </summary>
		public PieceColor Color
		{
			get { return _Color; }
		}
	
		/// <summary>
		/// Pozice figurky na šachovnici.
		/// </summary>
		public Position Position
		{
			get { return _Position; }
			set { _Position = value; }
		}

		/// <summary>
		/// Vrátí textově vyjádřenou pozici figurky na šachovnici, například c4.
		/// </summary>
		public string GetTextPosition()
		{
			return Chessboard.GetTextPosition(_Position);
		}

		/// <summary>
		/// Šachovnice, na které je figurka umístěna.
		/// </summary>
		public Chessboard Chessboard
		{
			get { return _Chessboard; }
		}

		/// <summary>
		/// Vytvoří novou figurku na zadané šachovnici.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na kterou bude figurka umístěna.</param>
		/// <param name="position">Pozice na šachovnici, kde bude figurka umístěna.</param>
		/// <param name="color">Barva figurky.</param>
		/// <exception cref="ArgumentNullException">Není zadána šachovnice.</exception>
		/// <exception cref="OutOfChessboardException">Zadané políčko je mimo šachovnici.</exception>
		/// <exception cref="InvalidOperationException">Zadané políčko je již obsazeno.</exception>
		protected Piece(Chessboard chessboard, Position position, PieceColor color)
		{
			// pokud nam dali null, vyhodime vyjimku
			if(chessboard == null) throw new ArgumentNullException("chessboard");
			
			// pokud nas umistili mimo, vyhodime vyjimku
			if(!Chessboard.IsPositionInside(position))
				throw new OutOfChessboardException(position);

			// pokud nas umistili na nekoho, taky vyhodime vyjimku
			Piece figurka = chessboard.GetPieceOnTile(position);
			if(figurka != null)
				throw new InvalidOperationException("Pozice " + position +
					" na šachovnici je již obsazena figurkou " + figurka + ".");

			_Chessboard = chessboard;
			_Position = position;
			_Color = color;
		}

		/// <summary>
		/// Vytvoří stejnou figurku na jiné šachovnici.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které bude umístěna nová figurka.</param>
		/// <param name="original">Vzor figurky.</param>
		/// <exception cref="ArgumentNullException">Není zadána šachovnice nebo figurka.</exception>
		protected Piece(Chessboard chessboard, Piece original)
		{
			// pokud nam dali null, vyhodime vyjimku
			if(chessboard == null) throw new ArgumentNullException("chessboard");
			if(original == null) throw new ArgumentNullException("original");

			// nastavime vlastnosti
			_Chessboard = chessboard;
			_Position = original.Position;
			_Color = original.Color;
		}

		/// <summary>
		/// Vytvoří nový tah touto figurkou na zadanou pozici.
		/// </summary>
		/// <param name="targetPosition">Cílová pozice tahu.</param>
		public Move MoveTo(Position targetPosition)
		{
			return new Move(this, _Position, targetPosition);
		}

		/// <summary>
		/// Vrátí textové označení figurky.
		/// </summary>
		public override string ToString()
		{
			return (_Color == PieceColor.Black ? "Černá" : "Bílá") + " figurka na " + _Position.ToString();
		}
	}
}
