﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Střelci
{
	/// <summary>
	/// Eviduje změny a ukládání dokumentu do souboru.
	/// </summary>
	public class FileDocument
	{
		private bool _Changed;
		private string _Title;
		private string _FileName;

		/// <summary>
		/// Vrátí/nastaví titulek dokumentu.
		/// </summary>
		public string Title
		{
			get { return _Title; }
			set { _Title = value; }
		}

		/// <summary>
		/// Vrátí/nastaví název souboru, kde je dokument uložen. Nebyl-li dosud uložen do souboru,
		/// je zde null.
		/// </summary>
		public string FileName
		{
			get { return _FileName; }
			set
			{
				// nulovy ani prazdny nebereme
				if(value == null)
					throw new ArgumentNullException("value");
				if(value == string.Empty)
					throw new ArgumentException("value");
				_FileName = value;
			}
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda byl dokument pojmenován.
		/// </summary>
		public bool IsNamed
		{
			get { return _FileName != null; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda je třeba dokument uložit.
		/// </summary>
		public bool NeedsSave
		{
			get { return _Changed; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda třeba dokument pojmenovat a uložit.
		/// </summary>
		public bool NeedsSaveAndName
		{
			get { return NeedsSave && !IsNamed; }
		}

		/// <summary>
		/// Označí dokument jako změněný.
		/// </summary>
		public void Changed()
		{
			_Changed = true;
		}

		/// <summary>
		/// Označí dokument jako uložený.
		/// </summary>
		/// <exception cref="System.InvalidOperationException">Dokument není pojmenován.</exception>
		public void Saved()
		{
			// neni-li pojmenovan, vyhodime chybu
			if(!IsNamed)
				throw new InvalidOperationException("Dokument není pojmenován, nemůže být uložen.");
			_Changed = false;
		}

		/// <summary>
		/// Označí dokument jako uložený pod zadaným jménem.
		/// </summary>
		/// <param name="fileName">Název soubor, kde byl dokument uložen.</param>
		public void SavedAs(string fileName)
		{
			FileName = fileName;
			Saved();
		}

		/// <summary>
		/// Označí dokument jako nezměněný po návratu k uložené verzi.
		/// </summary>
		public void Reverted()
		{
			_Changed = false;
		}

		/// <summary>
		/// Vytvoří nový objekt evidující změny a ukládání dokumentu.
		/// </summary>
		/// <param name="title">Titulek dokumentu.</param>
		public FileDocument(string title)
			: this(title, null, false)
		{
		}

		/// <summary>
		/// Vytvoří nový objekt evidující změny a ukládání dokumentu.
		/// </summary>
		/// <param name="title">Titulek dokumentu.</param>
		/// <param name="changed">Hodnota určující, zda byl dokument od poslední změny uložen.</param>
		public FileDocument(string title, bool changed)
			: this(title, null, changed)
		{
		}

		/// <summary>
		/// Vytvoří nový objekt evidující změny a ukládání dokumentu.
		/// </summary>
		/// <param name="title">Titulek dokumentu.</param>
		/// <param name="fileName">Název souboru, kde je dokument uložen.</param>
		public FileDocument(string title, string fileName)
			: this(title, fileName, false)
		{
		}

		/// <summary>
		/// Vytvoří nový objekt evidující změny a ukládání dokumentu.
		/// </summary>
		/// <param name="title">Titulek dokumentu.</param>
		/// <param name="fileName">Název souboru, kde je dokument uložen.</param>
		/// <param name="changed">Hodnota určující, zda byl dokument od poslední změny uložen.</param>
		public FileDocument(string title, string fileName, bool changed)
		{
			_Title = title;
			_FileName = fileName;
			_Changed = changed;
		}
	}
}
