﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Střelci
{
	/// <summary>
	/// Brána pro hru.
	/// </summary>
	class GameGate : Gate
	{
		private Piece _SelectedPiece;
		private Game _Game;

		/// <summary>
		/// Vybraná figurka, nebo null.
		/// </summary>
		protected Piece SelectedPiece
		{
			get { return _SelectedPiece; }
			set { _SelectedPiece = Painter.SelectedPiece = value; }
		}

		/// <summary>
		/// Aktuálně spuštěná partie.
		/// </summary>
		public Game Game
		{
			get { return _Game; }
		}

		/// <summary>
		/// Nastane, když uživatel provede neplatný tah.
		/// </summary>
		public event EventHandler<HumanMoveEventArgs> HumanPlayerInvalidMove;

		/// <summary>
		/// Nastane, když uživatel provede platný tah.
		/// </summary>
		public event EventHandler<HumanMoveEventArgs> HumanPlayerMove;

		protected void OnInvalidMove(HumanMoveEventArgs e)
		{
			if(HumanPlayerInvalidMove != null)
				HumanPlayerInvalidMove(this, e);
		}
		protected void OnHumanPlayerMove(HumanMoveEventArgs e)
		{
			if(HumanPlayerMove != null)
				HumanPlayerMove(this, e);
		}

		/// <summary>
		/// Vytvoří novou bránu pro partii.
		/// </summary>
		/// <param name="game">Partie, která bude vykreslována a ovládána.</param>
		/// <param name="painter">Malíř, který se bude používat k vykreslování šachovnice</param>
		/// <exception cref="ArgumentNullException">Některý z parametrů je null.</exception>
		public GameGate(Game game, Painter painter) : base(painter)
		{
			_Game = game;

			// vychozi hodnoty
			SelectedPiece = null;

			// zavesime se na zajimave udalosti
			game.ChessboardChanged += delegate(object sender, EventArgs e)
			{
				// prekresleni
				Painter.Refresh();
			};
		}

		/// <summary>
		/// Provede výběr figurky na políčku.
		/// </summary>
		protected void SelectClick(Position tile)
		{
			Piece nakliknuto = null;
			// ziskame figurku
			Piece figurka = _Game.Chessboard.GetPieceOnTile(tile);

			// pokud uzivatel klikl na vlastni figurku,
			if(figurka != null && _Game.GetPlayer(figurka.Color) == _Game.PlayerOnMove)
			{
				// vezmeme ji
				nakliknuto = figurka;
			}

			// vybereme nakliknute
			SelectedPiece = nakliknuto;
		}

		/// <summary>
		/// Provede tah vybrané figurky na políčko.
		/// </summary>
		protected void MoveClick(Position tile)
		{
			// pokud mame neco vybrano,
			if(SelectedPiece != null)
			{
				Player hrac = _Game.GetPlayer(SelectedPiece.Color);
				string komentar;
				// vytvorime tah
				Move tah = SelectedPiece.MoveTo(tile);

				// je-li tah platny,
				if(_Game.Judge.IsValid(tah, out komentar))
				{
					// provedeme tah
					_Game.DoMove(tah);
					// oznamime
					OnHumanPlayerMove(new HumanMoveEventArgs(hrac, komentar, tah));
					// a zrusime vyber
					SelectedPiece = null;
				}
				else
				{
					// jinak vyvolame udalost
					OnInvalidMove(new HumanMoveEventArgs(hrac, komentar, tah));
				}
			}
		}

		protected override void m_OutputControl_MouseClick(object sender, MouseEventArgs e)
		{
			// pokud hra nebezi nebo se pozastavena, konec
			if(_Game.Running == false || _Game.Paused)
				return;

			// pokud nehraje clovek, konec
			if(_Game.PlayerOnMove == null || _Game.PlayerOnMove.Type != PlayerType.Human)
				return;

			// policko, na ktere se kliklo
			Position? policko = Painter.GetTileFromPoint(e.X, e.Y);

			// pokud klikl mimo sachovnici, konec
			if(policko == null)
				return;

			// mame-li jednotlacitkove ovladani,
			if(Properties.Settings.Default.OneButtonControl)
			{
				if(e.Button == MouseButtons.Left)
				{
					// pokud jsme klikli na vlastni figurku, je to vyber
					Piece figurka = _Game.Chessboard.GetPieceOnTile(policko.Value);
					if(figurka != null && _Game.GetPlayer(figurka.Color) == _Game.PlayerOnMove)
						SelectClick(policko.Value);
					else
						// jinak je to tah
						MoveClick(policko.Value);
				}
			}
			else
			{
				// pri stisku LM vybereme
				if(e.Button == MouseButtons.Left)
					SelectClick(policko.Value);
				// pri stisku PM posleme
				else if(e.Button == MouseButtons.Right)
					MoveClick(policko.Value);
			}
		}
	}

	/// <summary>
	/// Představuje argumenty pro událost tahu člověka.
	/// </summary>
	class HumanMoveEventArgs : PlayerEventArgs
	{
		private string _Comment;
		private Move _Move;

		/// <summary>
		/// Komentář soudce k tahu.
		/// </summary>
		public string Comment
		{
			get { return _Comment; }
		}

		/// <summary>
		/// Tah lidského hráče.
		/// </summary>
		public Move Move
		{
			get { return _Move; }
		}

		/// <summary>
		/// Vytvoří argumenty pro událost tahu člověka.
		/// </summary>
		/// <param name="player">Hráč, který tah provádí.</param>
		/// <param name="comment">Komentář soudce k tahu.</param>
		/// <param name="move">Tah lidského hráče.</param>
		public HumanMoveEventArgs(Player player, string comment, Move move)
			: base(player)
		{
			_Comment = comment;
			_Move = move;
		}
	}
}
