using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace Stelci
{
	partial class IO
	{
		/// <summary>
		/// Nate partii ze souboru.
		/// </summary>
		/// <param name="fileName">Soubor, z nho se partie nate.</param>
		/// <returns>Vrt natenou partii.</returns>
		/// <exception cref="System.NotImplementedException">Formt souboru je neznm.</exception>
		/// <exception cref="System.ArgumentNullException">Nkter parametr nebyl zadn.</exception>
		/// <exception cref="Stelci.GameLoadException">Soubor se nepodailo nast.</exception>
		public static Game Load(string fileName)
		{
			if(fileName == null) throw new ArgumentNullException("fileName");

			try
			{
				// otevreme soubor
				using(StreamReader soubor = new StreamReader(fileName, Encoding.ASCII))
				{
					// je-li soubor prazdny, vyhodime vyjimku
					if(soubor.EndOfStream)
						throw new GameLoadException(string.Format("Soubor {0} je przdn.", fileName));

					string radek;
					Game partie = null;
					Chessboard sachovnice = null;
					Player bily = null, cerny = null;
					Player na_tahu = null;

					// cteme radky
					while((radek = soubor.ReadLine()) != null)
					{
						// hlavicka
						if(radek.StartsWith("["))
						{
							// hraci
							if(radek.StartsWith("[White "))
							{
								bily = GetPlayer(radek);
							}
							else if(radek.StartsWith("[Black "))
							{
								cerny = GetPlayer(radek);
							}
							// rozlozeni sachovnice
							else if(radek.StartsWith("[FEN "))
							{
								// precteme hodnotu
								string fen = GetPgnTagValue(radek);
								string[] casti = LoadFen(fen);
								// udelame z ni sachovnici
								AddPiecesFromFen(casti[0], out sachovnice);
								// zjistime hrace na tahu
								na_tahu = casti[1] == "w" ? bily : cerny;
							}
						}
						// zacinaji tahy
						else if(radek.StartsWith("1. "))
						{
							// pokud nam neco chybi, vyhodime vyjimku
							if(sachovnice == null) throw new GameLoadException("V souboru nen poten rozmstn figurek.");
							if(bily == null || cerny == null || na_tahu == null)
								throw new GameLoadException("V souboru nejsou kompletn informace o hrch.");

							// rozdelime to pres mezery
							string[] casti = radek.Split(' ');
							// sestrojime partii
							partie = new Game(sachovnice, bily, cerny, bily);
							// a spustime ji
							partie.Start();

							// projdeme casti
							int i = 0;
							int cislo = 1, cislo_nactene = 0;
							while(i < casti.Length)
							{
								// pokud je to prazdne, preskocime
								if(casti[i].Length == 0)
								{
									i++;
									continue;
								}

								// pokud zde je poradove cislo tahu,
								if(casti[i].EndsWith(".") &&
									int.TryParse(casti[i].Substring(0, casti[i].Length-1), out cislo_nactene))
								{
									// pokud se neshoduje s aktualnim, vyhodime chybu
									if(cislo != cislo_nactene)
										throw new GameLoadException(
											string.Format("Na mst, kde byl oekvn tah slo {0}, " +
													"byl nalezen tah slo {1}.",
												cislo, cislo_nactene));
									// dalsi bude tah bileho
									i++;
									// pokud tam neni, vyhodime chybu
									if(i == casti.Length)
										throw new GameLoadException("Vet tah byl pedasn ukonen.");
									// pokud je tam rosada, vyhodime chybu
									if(casti[i] == "O-O" || casti[i] == "O-O-O")
										throw new GameLoadException("Aplikace nepodporuje rodu, " +
											"kter se v souboru vyskytuje.");

									Move tah;
									try
									{
										// zkusime precist tah
										tah = partie.GetMoveFromSan(bily, casti[i]);
										i++;
									}
									catch(ArgumentException ex)
									{
										throw new GameLoadException(ex.Message, ex);
									}
									
									// nechame tah provest
									partie.DoMove(tah);

									// nasleduje tah cerneho
									// pokud tam neni, konec
									if(i == casti.Length)
										break;
									// pokud je tam rosada, vyhodime chybu
									if(casti[i] == "O-O" || casti[i] == "O-O-O")
										throw new GameLoadException("Aplikace nepodporuje rodu, " +
											"kter se v souboru vyskytuje.");
									// pokud je tam konec, konec
									if(casti[i] == "1-0" || casti[i] == "0-1" || casti[i] == "1/2-1/2")
										break;

									try
									{
										// zkusime precist tah
										tah = partie.GetMoveFromSan(cerny, casti[i]);
										i++;
									}
									catch(ArgumentException ex)
									{
										throw new GameLoadException(ex.Message, ex);
									}

									// nechame tah provest
									partie.DoMove(tah);

									// dalsi tah
									cislo++;
								}
								else
								{
									// jinak je tu neco, co nechceme
									throw new GameLoadException("V souboru byla mezi tahy nalezena neznm data.");
								}
							}
						}
						else
						{
							// jinak je tam neco jineho, chyba
							throw new GameLoadException("V souboru byla nalezena neznm data.");
						}
					}

					// neni-li dosud partie, seznam tahu byl prazdny; vytvorime prazdnou hru
					if(partie == null)
						partie = new Game(sachovnice, bily, cerny);

					return partie;
				}
			}
			catch(Exception) { throw; }
		}

		/// <summary>
		/// Ulo partii do souboru.
		/// </summary>
		/// <param name="game">Partie, kter se m uloit.</param>
		/// <param name="fileName">Soubor, do kterho se partie ulo.</param>
		/// <exception cref="System.NotImplementedException">Je zadn formt, do nho nelze ukldat.</exception>
		/// <exception cref="System.ArgumentNullException">Nkter parametr nebyl zadn.</exception>
		/// <exception cref="Stelci.GameSaveException">Soubor se nepodailo uloit.</exception>
		public static void Save(Game game, string fileName)
		{
			if(game == null) throw new ArgumentNullException("game");
			if(fileName == null) throw new ArgumentNullException("fileName");

			try
			{
				// otevreme soubor
				using(StreamWriter soubor = new StreamWriter(fileName, false, Encoding.ASCII))
				{
					// novy radek bude LF
					soubor.NewLine = "\x00a";
					
					// zapiseme hlavicku
					soubor.WriteLine("[Event \"Savegame\"]");
					soubor.WriteLine("[Site \"\"]");
					soubor.WriteLine("[Date \"{0:yyyy.MM.dd}\"", DateTime.Now);
					soubor.WriteLine("[Round \"1\"]");
					soubor.WriteLine("[White \"{0}\"]", GetPlayerType(game.GetPlayer(PieceColor.White)));
					soubor.WriteLine("[Black \"{0}\"]", GetPlayerType(game.GetPlayer(PieceColor.Black)));
					soubor.WriteLine("[Result \"{0}\"]", GetGameResult(game));
					soubor.WriteLine("[SetUp \"1\"]");
					soubor.WriteLine("[FEN \"{0}\"]", MakeFen(game, false));

					// ziskame tahy
					MoveInfo[] historie = game.GetHistory();

					// buffer pro hru
					StringBuilder buffer = new StringBuilder();

					// nyni projdeme tahy
					int i = 0;
					int cislo = 1;
					bool posledni_mezera = false;
					while(i < historie.Length)
					{
						// zapiseme cislo tahu a prvni tah
						buffer.AppendFormat("{0}. {1}", cislo, historie[i].AlgebraicNotation);
						// pokud je i protihracuv tah,
						if(++i < historie.Length)
						{
							// zapiseme ho tez
							buffer.AppendFormat(" {0} ", historie[i].AlgebraicNotation);
							posledni_mezera = true;
							i++;
						}
						else
						{
							posledni_mezera = false;
						}
						cislo++;
					}
					// posledni mezeru orizneme
					if(posledni_mezera)
						buffer.Length--;

					// zapiseme buffer
					soubor.Write(buffer.ToString());

					//// pokud hra skoncila remizou (nikoliv patem), pridame jeste info
					//if(game.State == GameState.Draw && !historie[i - 1].IsStalemate)
					//    soubor.Write(game.GetDrawDescription());
				}

			}
			catch(ArgumentException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(DirectoryNotFoundException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(PathTooLongException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(IOException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(UnauthorizedAccessException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(System.Security.SecurityException ex) { throw new GameSaveException(ex.Message, ex); }
		}

		/// <summary>
		/// Vrt druh zadanho hre pro poteby uloen do formtu PGN.
		/// </summary>
		private static string GetPlayerType(Player player)
		{
			return player.Type == PlayerType.Human ? "Human" : "Computer";
		}

		/// <summary>
		/// Vrt hodnotu zadanho tagu PGN.
		/// </summary>
		private static string GetPgnTagValue(string tag)
		{
			Regex rx = new Regex("\\[(.*) \"(.*)\"\\]");

			// test
			Match m = rx.Match(tag);
			// pokud se nepodarilo, vyhodime chybu
			if(!m.Success)
				throw new GameLoadException("Nepodailo se pest hodnotu znaky " + tag);

			// druha nalezena hodnota je ta prava
			return m.Groups[2].Value;
		}

		/// <summary>
		/// Vrt hre podle tagu formtu PGN.
		/// </summary>
		private static Player GetPlayer(string playerTag)
		{
			Regex rx_bily = new Regex("\\[White \"(.*)\"\\]");
			Regex rx_cerny = new Regex("\\[Black \"(.*)\"\\]");

			// test na bileho
			Match m = rx_bily.Match(playerTag);
			// pokud se podarilo,
			if(m.Success)
			{
				// urcime typ hrace
				PlayerType typ = m.Groups[1].Value == "Computer" ? PlayerType.Computer : PlayerType.Human;
				// a vratime ho
				return new Player(typ, PieceColor.White);
			}

			// test na cerneho
			m = rx_cerny.Match(playerTag);
			// pokud se podarilo,
			if(m.Success)
			{
				// urcime typ hrace
				PlayerType typ = m.Groups[1].Value == "Computer" ? PlayerType.Computer : PlayerType.Human;
				// a vratime ho
				return new Player(typ, PieceColor.Black);
			}

			// jinak se nepodarilo
			throw new GameLoadException("Nepodailo se nast informace o hri: " + playerTag);
		}

		/// <summary>
		/// Vrt vsledek zadan partie pro poteby uloen do formtu PGN.
		/// </summary>
		private static string GetGameResult(Game game)
		{
			switch(game.State)
			{
				case GameState.NotStartedYet:
				case GameState.Running: return "*";
				case GameState.WhiteWon: return "1-0";
				case GameState.BlackWon: return "0-1";
				case GameState.Draw: return "1/2-1/2";
				default:
					Debug.Fail("Neznm stav hry " + game.State);
					return "*";
			}
		}
	}
}