using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace Stelci
{
	partial class IO
	{
		/// <summary>
		/// Nate rozmstn figurek ze souboru.
		/// </summary>
		/// <remarks>
		/// Formt souboru se rozpoznv pouze podle ppony.
		/// </remarks>
		/// <param name="fileName">Soubor, ve kterm je rozmstn uloeno.</param>
		/// <param name="beginningPlayer">Obdr barvu zanajcho hre.</param>
		/// <exception cref="System.NotImplementedException">Formt souboru je neznm.</exception>
		/// <exception cref="Stelci.GameLoadException">Soubor se nepodailo nast.</exception>
		public static Chessboard LoadLayout(string fileName, out PieceColor beginningPlayer)
		{
			// format budeme rozpoznavat podle pripony
			string pripona = Path.GetExtension(fileName);

			try
			{
				switch(pripona)
				{
					case ".fen":
						// nacteme
						return LoadLayoutFen(fileName, out beginningPlayer);
					default:
						// takovy format nezname
						throw new NotImplementedException("Soubor \"" + fileName + "\" m neznm formt.");
				}
			}
			catch(ArgumentNullException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(ArgumentException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(DirectoryNotFoundException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(FileNotFoundException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(NotSupportedException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(IOException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(OutOfMemoryException ex) { throw new GameLoadException(ex.Message, ex); }
			catch(InvalidDataException ex) { throw new GameLoadException(ex.Message, ex); }
		}

		/// <summary>
		/// Ulo rozmstn figurek na achovnici do souboru.
		/// </summary>
		/// <param name="game">Partie, jej figurky se budou ukldat.</param>
		/// <param name="fileName">Soubor, do kterho se rozmstn ulo.</param>
		/// <param name="saveFormat">Formt, ve kterm se soubor ulo.</param>
		/// <exception cref="System.NotImplementedException">Je zadn formt, do nho nelze ukldat.</exception>
		/// <exception cref="Stelci.GameSaveException">Soubor se nepodailo uloit.</exception>
		public static void SaveLayout(Game game, string fileName, SaveFormat saveFormat)
		{
			try
			{
				switch(saveFormat)
				{
					case SaveFormat.ForsythEdwardsNotation:
						// ulozime
						SaveLayoutFen(game, fileName);
						break;
					default:
						throw new NotImplementedException("Ukldn do formtu " + saveFormat +
							" nen implementovno.");
				}
			}
			catch(ArgumentNullException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(ArgumentException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(DirectoryNotFoundException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(FileNotFoundException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(System.Security.SecurityException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(PathTooLongException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(IOException ex) { throw new GameSaveException(ex.Message, ex); }
			catch(UnauthorizedAccessException ex) { throw new GameSaveException(ex.Message, ex); }
		}

		/// <summary>
		/// Nate rozvren figurek ze souboru ve formtu FEN do achovnice.
		/// </summary>
		public static Chessboard LoadLayoutFen(string fileName, out PieceColor beginningPlayer)
		{
			string[] slozky;

			// otevreme soubor
			using(StreamReader sr = new StreamReader(fileName, Encoding.ASCII))
			{
				// precteme ho cely najednou cely
				slozky = LoadFen(sr.ReadToEnd());
			}

			Chessboard sachovnice;

			// pridame tam figurky
			AddPiecesFromFen(slozky[0], out sachovnice);
			// a zjistime hrace na tahu
			beginningPlayer = slozky[1] == "b" ? PieceColor.Black : PieceColor.White;

			return sachovnice;
		}

		/// <summary>
		/// Nate formt FEN a vrt ho jako pole etzc (rozdlen podle mezer).
		/// </summary>
		private static string[] LoadFen(string data)
		{
			// rozdelime podle mezer
			Regex rx = new Regex(" ");
			string[] slozky = rx.Split(data);

			// zkontrolujeme jejich pocet
			if(slozky.Length != 6)
				throw new InvalidDataException("Soubor m nesprvn formt: nalezeno " +
					slozky.Length + " sloek oddlench mezerou.");

			return slozky;
		}

		/// <summary>
		/// Ulo rozvren figurek hry ve formtu FEN.
		/// </summary>
		private static void SaveLayoutFen(Game game, string fileName)
		{
			// otevreme soubor
			using(StreamWriter sw = new StreamWriter(fileName, false, Encoding.ASCII))
			{
				// zapiseme FEN
				sw.Write(MakeFen(game, true));
			}
		}
	}
}