﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace Střelci
{
	/// <summary>
	/// Výjimka nastávající, když se nepodaří načíst partii nebo rozložení figurek.
	/// </summary>
	[Serializable]
	public class GameLoadException : Exception
	{
		public GameLoadException() { }
		public GameLoadException(string message) : base(message) { }
		public GameLoadException(string message, Exception inner) : base(message, inner) { }
		protected GameLoadException(
		  System.Runtime.Serialization.SerializationInfo info,
		  System.Runtime.Serialization.StreamingContext context)
			: base(info, context) { }
	}

	/// <summary>
	/// Výjimka nastávající, když se nepodaří uložit partii nebo rozložení figurek.
	/// </summary>
	[Serializable]
	public class GameSaveException : Exception
	{
		public GameSaveException() { }
		public GameSaveException(string message) : base(message) { }
		public GameSaveException(string message, Exception inner) : base(message, inner) { }
		protected GameSaveException(
		  System.Runtime.Serialization.SerializationInfo info,
		  System.Runtime.Serialization.StreamingContext context)
			: base(info, context) { }
	}

	/// <summary>
	/// Představuje správce ukládání a načítání souborů.
	/// </summary>
	static partial class IO
	{
		/// <summary>
		/// Formáty souborů, které lze ukládat.
		/// </summary>
		public enum SaveFormat
		{
			PortableGameNotation,
			ForsythEdwardsNotation
		}

		/// <summary>
		/// Sestaví řetězec rozmístění figurek partie ve Forsyth-Edwardsově notaci.
		/// </summary>
		/// <param name="game">Partie, ze které se má rozmístění převzít.</param>
		/// <param name="current">Hodnota určující, zda vzít aktuální rozmístění. Hodnota False
		/// znamená vzít počáteční rozmístění.
		/// </param>
		public static string MakeFen(Game game, bool current)
		{
			StringBuilder sb = new StringBuilder();

			// figurky
			sb.Append(MakePiecesFen(current ? game.Chessboard : game.StartupChessboard));
			sb.Append(" ");

			// zacinajici barva
			if(current)
				sb.Append(game.PlayerOnMove.Color == PieceColor.White ? "w" : "b");
			else
				sb.Append(game.BeginningPlayer.Color == PieceColor.White ? "w" : "b");
			sb.Append(" ");

			// rosada
			sb.Append("-"); // rosada nebude, nemame veze
			sb.Append(" ");

			// brani mimochodem nemame
			sb.Append("-");
			sb.Append(" ");

			// Halfmove clock
			// pro koncovky nema smysl
			sb.Append("0");
			sb.Append(" ");

			// pocet tahu
			sb.Append(game.GetHistory().Length / 2);

			return sb.ToString();
		}

		/// <summary>
		/// Sestaví řetězec rozmístění figurek ve Forsyth-Edwardsově notaci.
		/// </summary>
		/// <exception cref="UnknownPieceException">Nastane, když narazí na figurku neznámého typu.</exception>
		private static string MakePiecesFen(Chessboard chessboard)
		{
			StringBuilder sb = new StringBuilder(Chessboard.Tiles + Chessboard.Size - 1); // vsechna policka + pocet rozdelovniku

			// vezmeme si sachovnici jako pole
			Piece[,] pole = chessboard.ToArray();

			// projdeme radky
			for(int j = 0; j < Chessboard.Size; j++)
			{
				int pocet_prazdnych = 0;

				// projdeme sloupce
				for(int i = 0; i < Chessboard.Size; i++)
				{
					// pokud zde nic neni, zvetsime pocet prazdnych
					if(pole[i, j] == null)
						pocet_prazdnych++;
					else
					{
						// jinak pripadne zapiseme pocet predchozich prazdnych
						if(pocet_prazdnych > 0)
							sb.Append(pocet_prazdnych.ToString());
						// vynulujeme ho
						pocet_prazdnych = 0;
						// a kod figurky
						char kod = PieceToSan(pole[i, j]);
						// pokud figurku nezname, vyhodime vyjimku
						if(kod == 0)
							throw new UnknownPieceException(pole[i, j]);
						else
							sb.Append(kod);
					}
				}
				// konec sloupce

				// pripadne zapiseme pocet predchozich prazdnych
				if(pocet_prazdnych > 0)
					sb.Append(pocet_prazdnych.ToString());
				// a krome posledniho radku pridame oddelovac
				if(j != Chessboard.Size - 1)
					sb.Append("/");
			}

			return sb.ToString();
		}

		/// <summary>
		/// Vrátí označení pro figurku v algebraické notaci. Pro neznámou figurku vráti nulu.
		/// </summary>
		/// <exception cref="ArgumentNullException">Nastane, když je zadaná figurka null.</exception>
		public static char PieceToSan(Piece piece)
		{
			char pismeno;

			// kdyz nam nedaji figurku, vyhodime vyjimku
			if(piece == null) throw new ArgumentNullException("piece");

			// zvolime male pismeno
			if(piece is Pawn)
				pismeno = 'p';
			else if(piece is Bishop)
				pismeno = 'b';
			else if(piece is Queen)
				pismeno = 'q';
			else if(piece is King)
				pismeno = 'k';
			else
				pismeno = '\0'; // nezname

			// pokud je figurka bila, bude pismeno velke
			if(pismeno > 0 && piece.Color == PieceColor.White)
				pismeno = Char.ToUpper(pismeno);

			// vratime ho
			return pismeno;
		}

		/// <summary>
		/// Vrátí figurku, kterou označuje zadaný kód v algebraické notaci.
		/// Je-li figurka neznámá, vrátí null.
		/// </summary>
		/// <param name="sanCode">Kód označující figurku.</param>
		/// <param name="chessboard">Šachovnice, na kterou bude figurka umístěna.</param>
		/// <param name="position">Umístění nové figurky.</param>
		public static Piece SanToPiece(char sanCode, Chessboard chessboard, Position position)
		{
			Piece figurka;
			// bile maji velka pismena
			PieceColor barva = Char.IsUpper(sanCode) ? PieceColor.White : PieceColor.Black;

			// prevedeme na mala pismena
			sanCode = Char.ToLower(sanCode);

			switch(sanCode)
			{
				case 'p':
					figurka = new Pawn(chessboard, position, barva);
					break;
				case 'b':
					figurka = new Bishop(chessboard, position, barva);
					break;
				case 'q':
					figurka = new Queen(chessboard, position, barva);
					break;
				case 'k':
					figurka = new King(chessboard, position, barva);
					break;
				default:
					// nezname
					figurka = null;
					break;
			}

			return figurka;
		}

		/// <summary>
		/// Přidá figurky ze seznamu figurek ve formátu FEN na šachovnici.
		/// </summary>
		/// <returns>Počet přeskočených figurek. Přeskočeny budou neznámé figurky.</returns>
		/// <param name="piecesFen">Seznam figurek ve formátu FEN.</param>
		/// <param name="chessboard">Šachovnice, na kterou budou figurky umístěny.</param>
		private static int AddPiecesFromFen(string piecesFen, out Chessboard chessboard)
		{
			// prevedeme retezec na pole znaku
			char[] znaky = piecesFen.ToCharArray();
			// pocet preskocenych
			int preskoceno = 0;
			// aktualni souradnice na sachovnici
			int x = 0, y = 0;

			// vytvorime sachovnici
			chessboard = new Chessboard();

			// projdeme je
			foreach(char znak in znaky)
			{
				// pokud je znak cislo,
				if(Char.IsNumber(znak))
				{
					// znamena to pocet volnych mist
					int pocet = (int)Char.GetNumericValue(znak);
					// posunem se
					if(pocet == -1 || pocet == 0 || pocet > 8)
						throw new InvalidDataException("V souboru byla nalezena neplatná data: " + 
							pocet.ToString() + " volných polí v seznamu figurek.");
					else
						x += pocet;
				}
				// jinak, je-li to lomitko,
				else if(znak == '/')
				{
					// znamena to novy radek
					y++;
					x = 0;
				}
				// jinak, je-li to pismeno,
				else if(Char.IsLetter(znak))
				{
					// znamena to figurku
					Piece figurka = SanToPiece(znak, chessboard, new Position(x, y));
					// pridame ji
					if(figurka == null)
						preskoceno++;
					else
						chessboard.AddPiece(figurka);
					// posunem se
					x++;
				}
				// jinak je to neznamy znak
				else
					// je to v haji
					throw new InvalidDataException("V souboru byla nalezena neplatná data: znak " +
						znak.ToString() + " v seznamu figurek.");
			}

			// vratime pocet preskocenych
			return preskoceno;
		}
	}
}
