﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Collections;

namespace Střelci
{
	/// <summary>
	/// Představuje argumenty pro událost s figurkou.
	/// </summary>
	public class PieceEventArgs : EventArgs
	{
		private Piece m_Piece;

		/// <summary>
		/// Figurka, jíž se událost týká.
		/// </summary>
		public Piece Piece
		{
			get { return m_Piece; }
		}

		/// <summary>
		/// Vytvoří nové argumenty pro událost s figurkou.
		/// </summary>
		/// <param name="piece">Figurka, jíž se událost týká.</param>
		public PieceEventArgs(Piece piece)
		{
			m_Piece = piece;
		}
	}

	/// <summary>
	/// Představuje šachovnici.
	/// </summary>
	[DebuggerDisplay("Šachovnice, {_Pieces.Count} figurek")]
	public class Chessboard : IEnumerable<Piece>, ICloneable
	{
		/// <summary>
		/// Velikost šachovnice v políčkách.
		/// </summary>
		public const int Size = 8;
		/// <summary>
		/// Počet políček na šachovnici.
		/// </summary>
		public const int Tiles = Size * Size;
		/// <summary>
		/// Označení sloupců.
		/// </summary>
		public static readonly string[] ColumnLabels = { "a", "b", "c", "d", "e", "f", "g", "h" };
		/// <summary>
		/// Označení řádků.
		/// </summary>
		public static readonly string[] RowLabels = { "1", "2", "3", "4", "5", "6", "7", "8" };

		private List<Piece> _Pieces;

		/// <summary>
		/// Nastane, když je na šachovnici přidána figurka.
		/// </summary>
		public event EventHandler<PieceEventArgs> PieceAdded;
		/// <summary>
		/// Nastane, když je ze šachovnice odebrána figurka.
		/// </summary>
		public event EventHandler<PieceEventArgs> PieceRemoved;

		protected void OnPieceAdded(Piece piece)
		{
			if(PieceAdded != null)
				PieceAdded(this, new PieceEventArgs(piece));
		}

		protected void OnPieceRemoved(Piece piece)
		{
			if(PieceRemoved != null)
				PieceRemoved(this, new PieceEventArgs(piece));
		}

		/// <summary>
		/// Vytvoří novou prázdnou šachovnici.
		/// </summary>
		public Chessboard() : this(4)
		{
			// aspon 4 figurky tam urcite budou
		}

		/// <summary>
		/// Vytvoří novou prázdnou šachovnici.
		/// </summary>
		/// <param name="capacity">Předpokládaný počet figurek.</param>
		public Chessboard(int capacity)
		{
			// vytvorime prazdny seznam
			_Pieces = new List<Piece>(capacity);

			ThinkingCounter.Instance["Chessboard.Chessboard"]++;
		}

		/// <summary>
		/// Přidá figurku na šachovnici.
		/// </summary>
		/// <param name="piece">Figurka, která bude přidána.</param>
		/// <exception cref="ArgumentNullException">Nastane, když je zadaná figurka null.</exception>
		/// <exception cref="InvalidOperationException">Figurka má nastavenu jinou šachovnici, než je tato.</exception>
		public void AddPiece(Piece piece)
		{
			// kdyz nedostaneme figurku, vyhodime vyjimku
			if(piece == null) throw new ArgumentNullException("piece");
			// figurka musi patrit sem
			if(piece.Chessboard != this) throw new InvalidOperationException("Figurka " + piece + " nebyla vytvořena pro tuto šachovnici.");

			// pridame
			_Pieces.Add(piece);

			// oznamime
			OnPieceAdded(piece);
		}

		/// <summary>
		/// Odebere figurku ze šachovnice.
		/// </summary>
		/// <param name="piece">Figurka, která bude odebrána.</param>
		/// <exception cref="ArgumentNullException">Nastane, když je zadaná figurka null.</exception>
		/// <exception cref="InvalidOperationException">Figurka má nastavenu jinou šachovnici, než je tato.</exception>
		public void RemovePiece(Piece piece)
		{
			// kdyz nedostaneme figurku, vyhodime vyjimku
			if(piece == null) throw new ArgumentNullException("piece");
			// figurka musi patrit sem
			if(piece.Chessboard != this) throw new InvalidOperationException("Figurka " + piece + " není na této šachovnici.");

			// odebereme
			_Pieces.Remove(piece);

			// oznamime
			OnPieceRemoved(piece);
		}

		/// <summary>
		/// Odebere všechny figurky ze šachovnice.
		/// </summary>
		public void ClearPieces()
		{
			Piece[] pole = _Pieces.ToArray();

			// smazeme
			_Pieces.Clear();

			// oznamime
			foreach(Piece figurka in pole)
				OnPieceRemoved(figurka);
		}

		/// <summary>
		/// Zjistí, zda je zadaná pozice uvnitř šachovnice.
		/// </summary>
		public static bool IsPositionInside(Position position)
		{
			int x = position.X;
			int y = position.Y;
			return x >= 0 && x < Size && y >= 0 && y < Size;
		}

		/// <summary>
		/// Vrátí textově vyjádřenou pozici na šachovnici, například c4.
		/// </summary>
		/// <exception cref="OutOfChessboardException">Nastane, když je zadané políčko mimo šachovnici.</exception>
		public static string GetTextPosition(Position position)
		{
			string str = string.Empty;

			// pokud jsme mimo sachovnici, vyhodime chybu
			if(!IsPositionInside(position))
				throw new OutOfChessboardException(position);

			// nejdrive souradnice x
			str = ColumnLabels[position.X];

			// potom y
			str += RowLabels[RowLabels.Length - position.Y - 1];

			return str;
		}

		/// <summary>
		/// Vrátí pozici z textového vyjádření.
		/// </summary>
		/// <exception cref="OutOfChessboardException">Nastane, když je zadané políčko mimo šachovnici.</exception>
		public static Position GetPositionFromText(string text)
		{
			if(text.Length != 2)
				throw new OutOfChessboardException(text + " není platná pozice.");

			// ziskame souradnice
			int x = Array.IndexOf(ColumnLabels, char.ToLower(text[0]).ToString());
			int y = Array.IndexOf(RowLabels, char.ToLower(text[1]).ToString());
			if(y != -1)
				y = RowLabels.Length - y - 1;

			// jsou-li neplatne, konec
			if(x == -1 || y == -1)
				throw new OutOfChessboardException(text + " není platná pozice.");

			// jinak vratime
			return new Position(x, y);
		}

		/// <summary>
		/// Vrátí sloupec z textového vyjádření.
		/// </summary>
		public static int GetColumnFromText(string text)
		{
			if(text.Length != 1)
				return -1;

			return Array.IndexOf(ColumnLabels, char.ToLower(text[0]).ToString());
		}

		/// <summary>
		/// Vrátí řádek z textového vyjádření.
		/// </summary>
		public static int GetRowFromText(string text)
		{
			if(text.Length != 1)
				return -1;

			int y = Array.IndexOf(RowLabels, char.ToLower(text[0]).ToString());
			if(y != -1)
				y = RowLabels.Length - y - 1;

			return y;
		}

		/// <summary>
		/// Vrátí figurku na zadané pozici, nebo null, když na pozici žádná figurka není.
		/// </summary>
		/// <param name="position">Pozice na šachovnici.</param>
		public Piece GetPieceOnTile(Position position)
		{
			// projdeme figurky a hledame
			foreach(Piece figurka in _Pieces)
				if(figurka.Position == position)
					return figurka;
			
			// nenasli jsme
			return null;
		}

		/// <summary>
		/// Vrátí figurky patřící danému hráči.
		/// </summary>
		/// <param name="player">Hráč, jehož figurky se získávají.</param>
		public Piece[] GetPlayerPieces(PieceColor playerColor)
		{
			// pripravime seznam
			List<Piece> seznam = new List<Piece>(PieceCount / 2);
			
			// projdeme figurky
			foreach(Piece figurka in this)
			{
				// a pridavame
				if(figurka.Color == playerColor)
					seznam.Add(figurka);
			}

			// vratime
			return seznam.ToArray();
		}

		/// <summary>
		/// Vrátí počet figurek na šachovnici.
		/// </summary>
		public int PieceCount
		{
			get { return _Pieces.Count; }
		}

		/// <summary>
		/// Vrátí šachovnici jako pole figurek. Kde figurka není, bude null.
		/// </summary>
		public Piece[,] ToArray()
		{
			Piece[,] pole = new Piece[Chessboard.Size, Chessboard.Size];

			// projdeme figurky a zapisem je do pole
			foreach(Piece figurka in _Pieces)
				pole[figurka.Position.X, figurka.Position.Y] = figurka;

			return pole;
		}

		/// <summary>
		/// Zjistí, zda je trasa tahu volná.
		/// </summary>
		/// <param name="move">Tah, jehož cesta se bude procházet.</param>
		public bool IsPathClear(Move move)
		{
			// zacneme ve vychozi pozici
			Position p = move.From;
			Position cil = move.To;
			int x = p.X;
			int y = p.Y;

			// dokud nebudeme v cili
			while(p != cil)
			{
				// presuneme se potrebnym smerem
				if(cil.X > x)
					x++;
				else if(cil.X < x)
					x--;
				if(cil.Y > y)
					y++;
				else if(cil.Y < y)
					y--;
				p = new Position(x, y);
				// pokud neni pozice krome cile volna, cesta neni volna
				if(p != cil && GetPieceOnTile(p) != null)
					return false;
			}

			return true;
		}

		/// <summary>
		/// Zjistí, zda jsou na zadané šachovnici figurky rozestavěny stejně, jak na této.
		/// </summary>
		/// <param name="chessboard">Šachovnice, se kterou se porovnává.</param>
		public bool HasSamePosition(Chessboard chessboard)
		{
			// ziskame pole figurek
			Piece[,] nase = ToArray();
			Piece[,] druha = chessboard.ToArray();

			// projdeme pole
			for(int i = 0; i < nase.GetLength(0); i++)
			{
				for(int j = 0; j < nase.GetLength(1); j++)
				{
					// pokud nic nemame,
					if(nase[i, j] == null)
					{
						if(druha[i, j] != null)
							// na druhe neco je
							return false;
					}
					// neco mame
					else if(druha[i, j] == null)
					{
						// na druhe neni nic
						return false;
					}
					else if(nase[i,j].GetType() != druha[i,j].GetType())
					{
						// je tam jina figurka
						return false;
					}

				}
			}

			// stejne
			return true;
		}

		#region Technické detaily

		/// <summary>
		/// Enumerátor pro figurky.
		/// </summary>
		public IEnumerator<Piece> GetEnumerator()
		{
			return _Pieces.GetEnumerator();
		}

		IEnumerator IEnumerable.GetEnumerator()
		{
			return _Pieces.GetEnumerator();
		}

		#endregion

		#region ICloneable Members

		/// <summary>
		/// Vrátí hloubkovou kopii této šachovnice.
		/// </summary>
		/// <exception cref="Střelci.UnknownPieceException">Na šachovnici je neznámý druh figurky.</exception>
		public object Clone()
		{
			// vytvorime novou sachovnici
			Chessboard nova = new Chessboard(PieceCount);
			
			// a dame na ni vsechny figurky
			foreach(Piece figurka in this)
			{
				Piece nf;

				if(figurka is Pawn)
					nf = new Pawn(nova, (Pawn)figurka);
				else if(figurka is Bishop)
					nf = new Bishop(nova, (Bishop)figurka);
				else if(figurka is Queen)
					nf = new Queen(nova, (Queen)figurka);
				else if(figurka is King)
					nf = new King(nova, (King)figurka);
				else
					throw new UnknownPieceException(figurka);

				nova.AddPiece(nf);
			}

			return nova;
		}

		#endregion
	}
}
