﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Střelci
{
	/// <summary>
	/// Představuje partii.
	/// </summary>
	public class Game
	{
		private Chessboard _Chessboard;
		private Judge _Judge;
		private Player _Player1;
		private Player _Player2;
		private readonly Dictionary<Player, List<Type>> _CapturedPieces = new Dictionary<Player, List<Type>>(2);
		private Player _PlayerOnMove;
		private bool _Running = false;
		private Stack<MoveInfo> _History;
		private Stack<MoveInfo> _Future;
		private bool _MoveRequested = false;
		private bool _Paused = false;

		private int _CurrentMoveNumber = 1;

		private Chessboard _StartupChessboard;
		private readonly Player _BeginningPlayer;

		private GameState _State = GameState.NotStartedYet;

		/// <summary>
		/// Vrátí hráče, který začínal.
		/// </summary>
		public Player BeginningPlayer
		{
			get { return _BeginningPlayer; }
		}

		/// <summary>
		/// Vrátí stav hry.
		/// </summary>
		public GameState State
		{
			get { return _State; }
			protected set { _State = value; }
		}

		/// <summary>
		/// Vrátí šachovnici ve stavu, v jakém partie začala.
		/// </summary>
		public Chessboard StartupChessboard
		{
			get { return _StartupChessboard; }
		}

		/// <summary>
		/// Vrátí číslo aktuálně prováděného půltahu (od jedničky).
		/// </summary>
		public int CurrentMoveNumber
		{
			get { return _CurrentMoveNumber; }
			private set { _CurrentMoveNumber = value; }
		}

		/// <summary>
		/// Šachovnice, na které se partie odehrává. Odráží aktuální stav hry.
		/// </summary>
		public Chessboard Chessboard
		{
			get { return _Chessboard; }
		}

		/// <summary>
		/// Vrátí/nastav příznak, zda byl požadován nový tah.
		/// </summary>
		private bool MoveRequested
		{
			get { return _MoveRequested; }
			set { _MoveRequested = value; }
		}

		/// <summary>
		/// První hráč.
		/// </summary>
		public Player Player1
		{
			get { return _Player1; }
		}

		/// <summary>
		/// Druhý hráč.
		/// </summary>
		public Player Player2
		{
			get { return _Player2; }
		}

		/// <summary>
		/// Vrátí true, pokud hra běží.
		/// </summary>
		public bool Running
		{
			get { return _Running; }
			protected set { _Running = value; }
		}

		/// <summary>
		/// Vrátí/nastaví hodnotu určující, zda je hra pozastavena.
		/// </summary>
		public bool Paused
		{
			get { return _Paused; }
			set { _Paused = value; }
		}
	
		/// <summary>
		/// Vrátí hráče, který je na tahu, nebo null, pokud hra skončila.
		/// </summary>
		public Player PlayerOnMove
		{
			get
			{
				return _Running ? _PlayerOnMove : null;
			}
			protected set
			{
				// zmenime
				_PlayerOnMove = value;
				// oznamime to
				OnPlayerOnMoveChanged(EventArgs.Empty);
			}
		}

		/// <summary>
		/// Soudce, který dohlíží na partii.
		/// </summary>
		public Judge Judge
		{
			get { return _Judge; }
		}

		/// <summary>
		/// Vrátí/nastaví seznam historie tahů.
		/// </summary>
		protected Stack<MoveInfo> History
		{
			get { return _History; }
			set { _History = value; }
		}

		/// <summary>
		/// Vrátí/nastaví seznam budoucnosti tahů.
		/// </summary>
		protected Stack<MoveInfo> Future
		{
			get { return _Future; }
			set { _Future = value; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda je možno vrátit tah.
		/// </summary>
		public bool CanUndo
		{
			get { return History.Count > 0; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda je možno provést znovu tah.
		/// </summary>
		public bool CanRedo
		{
			get { return Future.Count > 0; }
		}

		/// <summary>
		/// Vrátí zajaté figurky daného hráče.
		/// </summary>
		public Type[] GetCapturedPieces(Player player)
		{
			return _CapturedPieces[player].ToArray();
		}

		/// <summary>
		/// Vrátí seznam historie půltahů.
		/// </summary>
		public MoveInfo[] GetHistory()
		{
			// prevedeme na pole
			MoveInfo[] pole = _History.ToArray();
			// obratime, aby zacinalo nejstarsi polozkou
			Array.Reverse(pole);
			// vratime
			return pole;
		}

		/// <summary>
		/// Nastane po dokončení tahu.
		/// </summary>
		public event EventHandler<MoveFinishedEventArgs> MoveFinished;

		/// <summary>
		/// Nastane po změně dění na šachovnici.
		/// </summary>
		public event EventHandler ChessboardChanged;

		/// <summary>
		/// Nastane, když se změní hráč na tahu.
		/// </summary>
		public event EventHandler PlayerOnMoveChanged;

		/// <summary>
		/// Nastane, když je třeba provést další tah.
		/// </summary>
		public event EventHandler NextMoveRequest;

		protected void OnChessboardChanged(EventArgs e)
		{
			if(ChessboardChanged != null)
				ChessboardChanged(this, e);
		}
		protected void OnMoveFinished(MoveFinishedEventArgs e)
		{
			if(MoveFinished != null)
				MoveFinished(this, e);
		}
		protected void OnPlayerOnMoveChanged(EventArgs e)
		{
			if(PlayerOnMoveChanged != null)
				PlayerOnMoveChanged(this, e);
		}
		protected void OnNextMoveRequest(EventArgs e)
		{
			if(NextMoveRequest != null)
				NextMoveRequest(this, e);
		}

		/// <summary>
		/// Vytvoří novou partii, kde je na tahu bílý hráč.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které se partie bude odehrávat.</param>
		/// <param name="firstPlayer">První hráč.</param>
		/// <param name="secondPlayer">Druhý hráč.</param>
		/// <exception cref="ArgumentNullException">Nastane, když je některý z hráčů null.</exception>
		/// <exception cref="InvalidOperationException">Nastane, když jsou oba hráči stejné barvy.</exception>
		public Game(Chessboard chessboard, Player firstPlayer, Player secondPlayer)
			: this(chessboard, firstPlayer, secondPlayer,
				   firstPlayer.Color == PieceColor.White ? firstPlayer : secondPlayer)
		{
		}

		/// <summary>
		/// Vytvoří novou partii.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které se partie bude odehrávat.</param>
		/// <param name="firstPlayer">První hráč.</param>
		/// <param name="secondPlayer">Druhý hráč.</param>
		/// <param name="playerOnMove">Hráč, který je na tahu.</param>
		/// <exception cref="ArgumentNullException">Nastane, když je některý z hráčů null.</exception>
		/// <exception cref="InvalidOperationException">Nastane, když jsou oba hráči stejné barvy.</exception>
		public Game(Chessboard chessboard, Player firstPlayer, Player secondPlayer, Player playerOnMove)
		{
			// kdyz nedostaneme hrace, vyhodime vyjimku
			if(firstPlayer == null) throw new ArgumentNullException("firstPlayer");
			if(secondPlayer == null) throw new ArgumentNullException("secondPlayer");
			// pokud maji hraci stejnou barvu, vyhodime vyjimku
			if(firstPlayer.Color == secondPlayer.Color)
				throw new InvalidOperationException("Hráči nemohou mít stejnou barvu.");

			// ulozime si hrace
			_Player1 = firstPlayer;
			_Player2 = secondPlayer;
			// napojime se na ne
			//_Player1.PlayerChanged += new EventHandler(m_Player_PlayerChanged);
			//_Player2.PlayerChanged += new EventHandler(m_Player_PlayerChanged);

			// pripravime seznam zajatych
			_CapturedPieces.Add(_Player1, new List<Type>());
			_CapturedPieces.Add(_Player2, new List<Type>());

			// pripravime seznam historie tahu
			_History = new Stack<MoveInfo>(20);
			_Future = new Stack<MoveInfo>();

			_Chessboard = chessboard;

			// zapamatujem si pocatecni sachovnici
			_StartupChessboard = (Chessboard)chessboard.Clone();

			// a soudce
			_Judge = new Judge(this);

			// prvni hrac
			_PlayerOnMove = _BeginningPlayer = playerOnMove;
		}

		/// <summary>
		/// Spustí hru, vyžádá si provedení prvního tahu.
		/// </summary>
		public void Start()
		{
			Running = true;
			State = GameState.Running;

			// oznamime hrace na tahu
			OnPlayerOnMoveChanged(EventArgs.Empty);

			// vyzadame si prvni tah
			OnNextMoveRequest(EventArgs.Empty);
		}

		/// <summary>
		/// Vrátí hráče zadané barvy.
		/// </summary>
		/// <param name="color">Barva hráče.</param>
		public Player GetPlayer(PieceColor color)
		{
			if(color == _Player1.Color)
				return _Player1;
			else
				return _Player2;
		}

		/// <summary>
		/// Vrátí informace o tahu z algebraické notace.
		/// </summary>
		/// <param name="player">Hráč, jemuž tah náleží.</param>
		/// <param name="moveText">Popis tahu.</param>
		/// <exception cref="System.ArgumentException">Zadaný text nepopisuje platný tah.</exception>
		public Move GetMoveFromSan(Player player, string moveText)
		{
			Position? odkud = null, kam = null;
			Piece figurka = null;
			//bool zajmuti = false;
			Type typ;
			int pokracovani = 0;

			// ziskame figurku podle prvniho znaku
			Piece prototyp = IO.SanToPiece(moveText[0], Chessboard, new Position());
			// urcime typ
			if(prototyp != null)
			{
				typ = prototyp.GetType();
				pokracovani = 1;
			}
			else
			{
				typ = typeof(Pawn);
			}
			prototyp = null;

			// je-li tu x, zajimalo se
			if(moveText[pokracovani] == 'x')
			{
				//zajmuti = true;
				pokracovani++;
			}

			// nasleduje prasacky kod rizeny vyjimkami :)

			try
			{
				Position? pozice = null;

				try
				{
					// zkusime precist pozici
					pozice = Chessboard.GetPositionFromText(moveText.Substring(pokracovani, 2));
					pokracovani += 2;

					bool je_dalsi = false;

					// zkusime nacist dalsi pozici
					if(pokracovani + 2 < moveText.Length)
					{
						try
						{
							// pokud se podari, bude to cilova
							kam = Chessboard.GetPositionFromText(moveText.Substring(pokracovani, 2));
							pokracovani += 2;
							// a predchozi byla zdrojova
							odkud = pozice;
							// ziskame figurku
							figurka = Chessboard.GetPieceOnTile(odkud.Value);
							je_dalsi = true;
						}
						catch(OutOfChessboardException)
						{
							// ticho
						}
					}

					// pokud nebyla dalsi,
					if(!je_dalsi)
					{
						// nepodarilo se, predtim prectena byla cilova
						kam = pozice;
						// nyni musime najit figurku, kt sem muze skocit
						foreach(Piece f in Chessboard)
						{
							if(f.GetType() != typ || f.Color != player.Color)
								continue;

							if(Judge.IsValid(new Move(f, kam.Value)))
							{
								// mame ji
								figurka = f;
								break;
							}
						}
						// pokud jsme ji nenasli, vyhodime chybu
						if(figurka == null)
							throw new ArgumentException(moveText + " není platný popis tahu. Figurka, se kterou se táhne, nebyla nalezena.");
					}
				}
				catch(OutOfChessboardException)
				{
					// neni tam pozice, musi to tedy byt urceni radku nebo sloupce
					// je-li tam pismeno, je to sloupec
					if(char.IsLetter(moveText[pokracovani]))
					{
						// precteme ho
						int x = Chessboard.GetColumnFromText(moveText.Substring(pokracovani, 1));
						// pokud se nepodarilo, vyhodime chybu
						if(x == -1)
							throw new ArgumentException(moveText + " není platný popis tahu.");

						// projdeme figurky
						foreach(Piece f in Chessboard)
						{
							if(f.GetType() != typ || f.Color != player.Color)
								continue;

							if(f.Position.X == x)
							{
								// mame ji
								figurka = f;
								break;
							}
						}
					}
					else
					{
						// jinak to musi byt urceni radku
						// precteme ho
						int y = Chessboard.GetRowFromText(moveText.Substring(pokracovani, 1));
						// pokud se nepodarilo, vyhodime chybu
						if(y == -1)
							throw new ArgumentException(moveText + " není platný popis tahu.");

						// projdeme figurky
						foreach(Piece f in Chessboard)
						{
							if(f.GetType() != typ || f.Color != player.Color)
								continue;

							if(f.Position.Y == y)
							{
								// mame ji
								figurka = f;
								break;
							}
						}
					}
					pokracovani++;
					// pokud uz nemame figurku, vyhodime chybu
					if(figurka == null)
						throw new ArgumentException(moveText + " není platný popis tahu. Nejasný tah nebyl identifikován.");
					odkud = figurka.Position;

					// zkusime nacist dalsi pozici
					try
					{
						// pokud se podari, bude to cilova
						kam = Chessboard.GetPositionFromText(moveText.Substring(pokracovani, 2));
						pokracovani += 2;
					}
					catch(OutOfChessboardException)
					{
						// nenasleduje pozice, vyhodime chybu
						throw new ArgumentException(moveText + " není platný popis tahu.");
					}
				}
				// nyni mame zdrojovou a cilovou pozici a figurku

				// vratime
				return new Move(figurka, kam.Value);
			}
			catch(IndexOutOfRangeException ex)
			{
				// je to spatne
				throw new ArgumentException(moveText + " není platný popis tahu.", ex);
			}
			catch(ArgumentOutOfRangeException ex)
			{
				// je to spatne
				throw new ArgumentException(moveText + " není platný popis tahu.", ex);
			}
		}

		/// <summary>
		/// Podle potřeby přidá do řetězce v SAN počáteční pozici figurky.
		/// </summary>
		private void AppendFromPosition(StringBuilder str, MoveInfo moveInfo)
		{
			// HACK: kvuli spravne fci musime na chvilku figurku premistit zpatky
			moveInfo.Move.Piece.Position = moveInfo.Move.From;
			// a pripadne sebranou obnovit
			if(moveInfo.CapturedPiece != null)
				moveInfo.Move.Piece.Chessboard.AddPiece(moveInfo.CapturedPiece);

			try
			{
				// projdeme figurky
				foreach(Piece figurka in moveInfo.Move.Piece.Chessboard)
				{
					// pouze figurky stejnych typu
					if(figurka.GetType() != moveInfo.Move.Piece.GetType() ||
					   figurka.Color != moveInfo.Move.Piece.Color)
						continue;
					// a jine nez nase
					if(figurka == moveInfo.Move.Piece)
						continue;

					// muze-li tahnout na stejne misto, je to nejasny tah
					if(Judge.IsValid(new Move(figurka, moveInfo.Move.To)))
					{
						Position pozice = moveInfo.Move.From;

						// pokud se daji rozlisit podle sloupce, ucinime tak
						if(figurka.Position.X != pozice.X)
						{
							str.Append(Chessboard.ColumnLabels[pozice.X]);
							return;
						}

						// pokud se daji rozlisit podle radku, ucinime tak
						if(figurka.Position.Y != pozice.Y)
						{
							str.Append(Chessboard.RowLabels[Chessboard.RowLabels.Length - pozice.Y - 1]);
							return;
						}

						// jinak musime pridat obe souradnice
						str.Append(Chessboard.GetTextPosition(pozice));

						return;
					}
				}
			}
			finally
			{
				// konec hacku
				if(moveInfo.CapturedPiece != null)
					moveInfo.Move.Piece.Chessboard.RemovePiece(moveInfo.CapturedPiece);
				moveInfo.Move.Piece.Position = moveInfo.Move.To;
			}
		}

		/// <summary>
		/// Vrátí popis tahu podle algebraické notace.
		/// </summary>
		/// <param name="moveInfo">Informace o tahu.</param>
		public string GetMoveDescription(MoveInfo moveInfo)
		{
			StringBuilder str = new StringBuilder(10);

			// nejdriv figurka
			if(!(moveInfo.Move.Piece is Pawn))
				str.Append(IO.PieceToSan(moveInfo.Move.Piece));

			// pokud zajala, pridame krizek
			if(moveInfo.CapturedPiece != null)
				str.Append('x');

			// je-li to nejasny tah, pocatecni pozice
			AppendFromPosition(str, moveInfo);
			// koncova pozice
			str.Append(Chessboard.GetTextPosition(moveInfo.Move.To));

			// info o premene pindika
			if(moveInfo.PawnConvertedTo != null)
				str.Append(IO.PieceToSan(moveInfo.PawnConvertedTo));

			// je-li mat nebo sach,
			if(moveInfo.KingInCheckmate != null)
			{
				// pridame znak
				str.Append('#');
				// pridame vysledek
				if(moveInfo.KingInCheckmate.Color == PieceColor.Black)
					str.Append(" 1-0");
				else
					str.Append(" 0-1");
			}
			// je-li sach, pridame znak
			else if(moveInfo.KingInCheck != null)
				str.Append('+');
			// je-li pat, pridame vysledek
			else if(moveInfo.IsStalemate)
				str.Append(GetDrawDescription());

			return str.ToString();
		}

		/// <summary>
		/// Vrátí popis remízy podle algebraické notace.
		/// </summary>
		public string GetDrawDescription()
		{
			return " 1/2-1/2";
		}

		/// <summary>
		/// Provede v aktuální partii zadaný tah.
		/// </summary>
		/// <param name="move">Tah, který se má provést.</param>
		public void DoMove(Move move)
		{
			Debug.Assert(PlayerOnMove != null && PlayerOnMove.Color == move.Piece.Color,
				string.Format("Místo hráče {0} se pokusil hrát {1}.",
				PlayerOnMove == null ? "(žádný, hra skončena)" : PlayerOnMove.ToString(),
				GetPlayer(move.Piece.Color)));

			// zrusime nabidky na remizu
			Player1DrawOffer = false;
			Player2DrawOffer = false;
			
			// provedeme tah
			MoveInfo mji = _Judge.DoMoveAndDescribe(move);

			Piece zajata = mji.CapturedPiece;

			// je-li nekdo zajat,
			if(zajata != null)
			{
				Debug.Assert(_Chessboard == zajata.Chessboard, "Figurka " + zajata + " nebyla odstraněna " +
					"ze správné šachovnice.");

				// pridame ho do seznamu zajatych
				_CapturedPieces[GetPlayer(zajata.Color)].Add(zajata.GetType());
			}

			// je-li pat nebo mat, hra skoncila
			if(mji.IsStalemate || mji.KingInCheckmate != null)
			{
				Running = false;
				// zaver hry
				State = GetGameOutcome(mji);
			}
			else
			{
				// hraje dalsi
				SwitchPlayerOnMove();
			}

			// pridame tah do historie
			History.Push(mji);
			// zaroven musime smazat budoucnost, ktera uz prestala platit
			Future.Clear();

			CurrentMoveNumber++;

			// a oznamime
			OnChessboardChanged(EventArgs.Empty);
			OnMoveFinished(new MoveFinishedEventArgs(mji));

			// pokud hra neskoncila,
			if(Running)
			{
				// vyzadame si dalsi tah
				OnNextMoveRequest(EventArgs.Empty);				
			}
		}

		/// <summary>
		/// Vrátí výsledek hry podle informace o posledním tahu.
		/// </summary>
		private static GameState GetGameOutcome(MoveInfo mi)
		{
			// remiza
			if(mi.IsDraw)
			{
				return GameState.Draw;
			}
			// mat
			else if(mi.KingInCheckmate != null)
			{
				return mi.KingInCheckmate.Color == PieceColor.White ?
						GameState.BlackWon : GameState.WhiteWon;
			}
			// jinak hra bezi dal
			else
				return GameState.Running;
		}

		/// <summary>
		/// Přepne hráče na tahu na druhého.
		/// </summary>
		protected void SwitchPlayerOnMove()
		{
			PlayerOnMove = _PlayerOnMove == _Player1 ? _Player2 : _Player1;
		}

		/// <summary>
		/// Posune partii o zadaný počet půltahů dopředu (nebo zpátky, když je zadáno záporné číslo).
		/// </summary>
		/// <param name="moveCount">Počet půltahů, které se mají posunout, kladné nebo záporné.</param>
		/// <returns>True, pokud se podařilo.</returns>
		public bool NavigateTime(int moveCount, bool replaying)
		{
			// jdeme-li dopredu,
			if(moveCount > 0)
			{
				// pokud se mame posunout o vic, nez je v budoucnosti, zkratime to
				if(moveCount > Future.Count)
					moveCount = Future.Count;

				int i = moveCount;

				// posunem se
				while(i-- > 0)
					GoForward(replaying);
			}
			else
			{
				// jinak jdeme zpet

				// pokud se mame vratit o vic, nez je v historii, zkratime to
				if(-moveCount > History.Count)
					moveCount = -History.Count;

				int i = moveCount;

				// povracime je
				while(i++ < 0)
					GoBack(replaying);
			}

			// zvetsime/zmensime cislo pultahu
			CurrentMoveNumber += moveCount;

			// oznamime zmenu deni
			OnChessboardChanged(EventArgs.Empty);

			// pokud hra neskoncila a neni pauznuta,
			if(Running)
			{
				// pokud je potreba, prepneme hrace
				if(moveCount % 2 != 0)
					SwitchPlayerOnMove();

				// pokud neni hra pauznuta, vyzadame si novy tah
				if(!Paused)
					OnNextMoveRequest(EventArgs.Empty);
			}

			return true;
		}

		/// <summary>
		/// Posune partii o půtah dopředu.
		/// </summary>
		protected void GoForward(bool replay)
		{
			Debug.Assert(Future.Count > 0, "Budoucnost je prázdná.");

			// odebereme tah z budoucnosti
			MoveInfo tah = Future.Pop();
			// a pridame do historie
			History.Push(tah);

			// provedeme tah
			Judge.DoMove(tah);

			// pokud neprobiha prehravani,
			if(!replay)
			{
				// pokud ma tahem hra skoncit,
				if(tah.KingInCheckmate != null || tah.IsDraw)
				{
					// zastavime ji
					State = GetGameOutcome(tah);
					Running = State == GameState.Running;
				}
			}
		}

		/// <summary>
		/// Vrátí partii o půltah nazpátek.
		/// </summary>
		protected void GoBack(bool replay)
		{
			Debug.Assert(History.Count > 0, "Historie je prázdná.");

			// odebereme tah z vrcholu historie
			MoveInfo tah = History.Pop();
			// a pridame do budoucnosti
			Future.Push(tah);

			// vratime tah
			Judge.RevertMove(tah);

			// pokud neprobiha prehravani,
			if(!replay)
			{
				// pokud tahem hra skoncila,
				if(tah.KingInCheckmate != null || tah.IsStalemate)
				{
					// znovu ji spustime
					Running = true;
				}
			}
		}

		/// <summary>
		/// Obnoví běh ukončené partie.
		/// </summary>
		public void Resume()
		{
			// pokud bezi, konec
			if(Running)
				return;

			// spustime
			Running = true;
			State = GameState.Running;

			// nastavime hrace na tahu
			PlayerOnMove = GetPlayer(Player.ContraryColor(History.Peek().Move.Piece.Color));
		}

		/// <summary>
		/// Nastane, když dojde k remíze.
		/// </summary>
		public event EventHandler<DrawEventArgs> Draw;

		/// <summary>
		/// Ukončí hru remízou.
		/// </summary>
		/// <param name="reason">Důvod remízy.</param>
		protected void MakeDraw(string reason)
		{
			// konec
			Running = false;
			State = GameState.Draw;

			// HACK: Pridame info o remize do historie
			History.Peek().AlgebraicNotation += GetDrawDescription();
			History.Peek().IsDraw = true;

			// musime smazat budoucnost
			Future.Clear();

			// oznamime
			if(Draw != null)
				Draw(this, new DrawEventArgs(reason));
		}

		private bool _Player1DrawOffer = false;
		private bool _Player2DrawOffer = false;

		/// <summary>
		/// Vrátí hodnotu určující, zda první hráč nabízí remízu.
		/// </summary>
		public bool Player1DrawOffer
		{
			get { return _Player1DrawOffer; }
			protected set
			{
				_Player1DrawOffer = value;
				OnDrawOffersChanged(EventArgs.Empty);
			}
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda druhý hráč nabízí remízu.
		/// </summary>
		public bool Player2DrawOffer
		{
			get { return _Player2DrawOffer; }
			protected set
			{
				_Player2DrawOffer = value;
				OnDrawOffersChanged(EventArgs.Empty);
			}
		}

		/// <summary>
		/// Nastane při změně nabídek na remízu.
		/// </summary>
		public event EventHandler DrawOffersChanged;

		protected void OnDrawOffersChanged(EventArgs e)
		{
			if(DrawOffersChanged != null)
				DrawOffersChanged(this, e);
		}

		/// <summary>
		/// Odešle protiháči nabídku na remízu.
		/// </summary>
		/// <param name="player">Hráč, který dává nabídku.</param>
		/// <returns>Vrátí hodnotu určující, zda k remíze skutečně došlo.</returns>
		public bool OfferDraw(Player player)
		{
			// hrac zada
			if(player == Player1)
				Player1DrawOffer = true;
			else
				Player2DrawOffer = true;

			// pokud zadal i druhy, hra konci
			if((player == Player1 && Player2DrawOffer) || (player == Player2 && Player1DrawOffer))
			{
				MakeDraw("Hráči se dohodli na remíze.");
				return true;
			}

			if(History.Count > 0)
			{
				// zjistime moznosti remizey
				bool fifty = History.Peek().CanDrawByFiftyMoveRule;
				bool three = History.Peek().CanDrawByThreefoldRepetition;

				// pokud ma hrac pravo na remizu,
				if(fifty || three)
				{
					// hra konci
					MakeDraw(string.Format("{0} hráč využil svého práva na remízu podle pravidla o {1}.",
						player.Color == PieceColor.Black ? "Černý" : "Bílý",
						three ? "trojím opakování" : "padesátém tahu"));
					return true;
				}
			}

			return false;
		}
	}

	/// <summary>
	/// Stav hry.
	/// </summary>
	public enum GameState
	{
		/// <summary>
		/// Ještě nezačala.
		/// </summary>
		NotStartedYet,
		/// <summary>
		/// Hra probíhá.
		/// </summary>
		Running,
		/// <summary>
		/// Bílý vyhrál.
		/// </summary>
		WhiteWon,
		/// <summary>
		/// Černý vyhrál.
		/// </summary>
		BlackWon,
		/// <summary>
		/// Remíza.
		/// </summary>
		Draw
	}

	/// <summary>
	/// Argumenty pro události remízy.
	/// </summary>
	public class DrawEventArgs : EventArgs
	{
		private string _Reason;

		/// <summary>
		/// Vrátí důvod remízy.
		/// </summary>
		public string Reason
		{
			get { return _Reason; }
		}

		/// <summary>
		/// Vytvoří nové argumenty pro událost remízy.
		/// </summary>
		/// <param name="reason">Důvod remízy.</param>
		public DrawEventArgs(string reason)
		{
			_Reason = reason;
		}
	}

	/// <summary>
	/// Představuje argumenty pro událost dokončení tahu.
	/// </summary>
	public class MoveFinishedEventArgs : EventArgs
	{
		MoveInfo m_mji;

		/// <summary>
		/// Informace o provedeném tahu a jeho následcích.
		/// </summary>
		public MoveInfo MoveJudgementInfo
		{
			get { return m_mji; }
		}

		/// <summary>
		/// Vytvoří nové argumenty pro událost dokončení tahu.
		/// </summary>
		/// <param name="moveJudgementInfo">Informace o provedeném tahu a jeho následcích.</param>
		public MoveFinishedEventArgs(MoveInfo moveJudgementInfo)
		{
			m_mji = moveJudgementInfo;
		}
	}
}
