﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Střelci
{
	/// <summary>
	/// Důvod remízy.
	/// </summary>
	public enum DrawReason
	{
		/// <summary>
		/// Nedošlo k remíze.
		/// </summary>
		NoDraw,
		/// <summary>
		/// Dohoda hráčů.
		/// </summary>
		Agreement,
		/// <summary>
		/// Pat.
		/// </summary>
		Stalemate,
		/// <summary>
		/// Trojnásobné opakování stejných tahů.
		/// </summary>
		ThreefoldRepetition,
		/// <summary>
		/// Žádná zajmutá figurka po 50 tazích.
		/// </summary>
		FiftyMoveRule
	}

	/// <summary>
	/// Kompletní záznam o provedeném půltahu.
	/// </summary>
	public class MoveInfo
	{
		private King _KingInCheck;
		private King _KingInCheckmate;
		private bool _IsStalemate;
		private bool _IsDraw;
		private Piece _CapturedPiece;
		private Piece _PawnConvertedTo;
		private Move _Move;
		private Pawn _PawnCapturedEnPassant;
		private bool _CanDrawByThreefoldRepetition;
		private bool _CanDrawByFiftyMoveRule;
		private string _AlgebraicNotation;
		
		/// <summary>
		/// Vrátí tah, který byl proveden.
		/// </summary>
		public Move Move
		{
			get { return _Move; }
		}

		/// <summary>
		/// Vrátí popis tahu ve standardní algebraické notaci.
		/// </summary>
		public string AlgebraicNotation
		{
			get { return _AlgebraicNotation; }
			// HACK: Toto neni dobre
			set { _AlgebraicNotation = value; }
		}

		/// <summary>
		/// Vrátí figurku, která byla při tahu zajata, nebo null.
		/// </summary>
		public Piece CapturedPiece
		{
			get { return _CapturedPiece; }
		}

		/// <summary>
		/// Vrátí krále, který je v šachu, nebo null.
		/// </summary>
		public King KingInCheck
		{
			get { return _KingInCheck; }
		}

		/// <summary>
		/// Vrátí krále, který je v matu, nebo null.
		/// </summary>
		public King KingInCheckmate
		{
			get { return _KingInCheckmate; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda nastal pat.
		/// </summary>
		public bool IsStalemate
		{
			get { return _IsStalemate; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda došlo k remíze.
		/// </summary>
		public bool IsDraw
		{
			get { return _IsDraw; }
			// HACK: Toto by tu taky byt nemelo
			set { _IsDraw = value; }
		}

		/// <summary>
		/// Vrátí figurku, na kterou byl přeměněn pěšec, nebo null.
		/// </summary>
		public Piece PawnConvertedTo
		{
			get { return _PawnConvertedTo; }
		}

		/// <summary>
		/// Vrátí pěšce sebraného mimochodem, nebo null.
		/// </summary>
		public Pawn PawnCapturedEnPassant
		{
			get { return _PawnCapturedEnPassant; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda je možné po tomto tahu žádat remízu 
		/// podle pravidla třetího opakování.
		/// </summary>
		public bool CanDrawByThreefoldRepetition
		{
			get { return _CanDrawByThreefoldRepetition; }
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda je možné po tomto tahu žádat remízu
		/// podle pravidla 50. tahu.
		/// </summary>
		public bool CanDrawByFiftyMoveRule
		{
			get { return _CanDrawByFiftyMoveRule; }
		}

		/// <summary>
		/// Vytvoří nové informace o posouzení tahu.
		/// </summary>
		/// <param name="move">Tah, který se provedl.</param>
		/// <param name="capturedPiece">Figurka, která byla při tahu zajata.</param>
		/// <param name="kingInCheck">Král, který je v šachu, nebo null.</param>
		/// <param name="kingInCheckmate">Král, který je v matu, nebo null.</param>
		/// <param name="isStalemate">True, když nastal pat.</param>'
		/// <param name="isDraw">True, když došlo k remíze.</param>
		/// <param name="pawnConvertedTo">Figurka, na kterou byl přeměněn pěšec.</param>
		/// <param name="pawnCapturedEnPassant">Pěšec sebraný mimochodem, nebo null.</param>
		/// <param name="canDrawByThreefoldRepetition">Možná remíza podle třetího opakování.</param>
		/// <param name="canDrawByFiftyMoveRule">Možná remíza podle pravidla 50. tahu.</param>
		public MoveInfo(Move move, Piece capturedPiece, 
			King kingInCheck, King kingInCheckmate, bool isStalemate, bool isDraw,
			Piece pawnConvertedTo, Pawn pawnCapturedEnPassant,
			bool canDrawByThreefoldRepetition, bool canDrawByFiftyMoveRule)
		{
			_Move = move;
			_CapturedPiece = capturedPiece;
			_KingInCheck = kingInCheck;
			_KingInCheckmate = kingInCheckmate;
			_IsStalemate = isStalemate;
			_IsDraw = isDraw;
			_PawnConvertedTo = pawnConvertedTo;
			_PawnCapturedEnPassant = pawnCapturedEnPassant;
			_CanDrawByThreefoldRepetition = canDrawByThreefoldRepetition;
			_CanDrawByFiftyMoveRule = canDrawByFiftyMoveRule;
		}
	}

	/// <summary>
	/// Představuje soudce, který zaručuje, že je situace na šachovnici platná podle pravidel.
	/// </summary>
	public sealed partial class Judge
	{
		private Game _Game;
		private static Dictionary<Type, List<RelativePosition>> _PieceMoves;

		/// <summary>
		/// Partie, na kterou soudce dohlíží.
		/// </summary>
		public Game Game
		{
			get { return _Game; }
			set { _Game = value; }
		}

		/// <summary>
		/// Vytvoří nového soudce pro zadanou partii.
		/// </summary>
		/// <exception cref="ArgumentNullException">Nebyla předána partie.</exception>
		public Judge(Game game)
		{
			// pokud nam nic nedali, vyhodime vyjimku
			if(game == null) throw new ArgumentNullException("game");
			
			// pokud neni naplnen seznam moznych tahu, naplnime ho
			if(_PieceMoves == null)
				FillPossibleDraws();

			Game = game;
		}

		/// <summary>
		/// Vrátí pole možných tahů figurky.
		/// </summary>
		/// <param name="pieceType">Typ figurky.</param>
		/// <exception cref="UnknownPieceException">Soudce nezná pravidla pro danou figurku.</exception>
		public static RelativePosition[] GetPiecePossibleDraws(Type pieceType)
		{
			try
			{
				// ziskame mozne tahy dane figurky
				return _PieceMoves[pieceType].ToArray();
			}
			catch(KeyNotFoundException ex)
			{
				// narazili jsme na neznamou figurku
				throw new UnknownPieceException("Soudce nezná figurku " + pieceType + ".", ex);
			}
		}

		/// <summary>
		/// Naplní seznam možných tahů pro figurky.
		/// </summary>
		private static void FillPossibleDraws()
		{
			List<RelativePosition> seznam;

			// vytvorime pole
			_PieceMoves = new Dictionary<Type, List<RelativePosition>>(4);

			// pesec
			seznam = new List<RelativePosition>(8);
			seznam.Add(new RelativePosition(-1, -1)); // brani vlevo
			seznam.Add(new RelativePosition(1, -1)); // brani vpravo
			seznam.Add(new RelativePosition(0, -1)); // vpred o jedno
			seznam.Add(new RelativePosition(0, -2)); // vpred o dve
			seznam.Add(new RelativePosition(1, 1)); // brani vlevo
			seznam.Add(new RelativePosition(-1, 1)); // brani vpravo
			seznam.Add(new RelativePosition(0, 1)); // vpred o jedno
			seznam.Add(new RelativePosition(0, 2)); // vpred o dve
			// pridame
			_PieceMoves.Add(typeof(Pawn), seznam);

			// kral
			seznam = new List<RelativePosition>(8);
			seznam.AddRange(RelativePosition.CreateNeighborhood());
			// pridame
			_PieceMoves.Add(typeof(King), seznam);

			// strelec
			seznam = new List<RelativePosition>(4 * 8);
			seznam.AddRange(RelativePosition.CreateLRDiagonal(8));
			seznam.AddRange(RelativePosition.CreateRLDiagonal(8));
			// pridame
			_PieceMoves.Add(typeof(Bishop), seznam);

			// kralovna
			seznam = new List<RelativePosition>(4 * 8 * 2);
			seznam.AddRange(RelativePosition.CreateLRDiagonal(8));
			seznam.AddRange(RelativePosition.CreateRLDiagonal(8));
			seznam.AddRange(RelativePosition.CreateRow(8));
			seznam.AddRange(RelativePosition.CreateColumn(8));
			// pridame
			_PieceMoves.Add(typeof(Queen), seznam);
		}

		/// <summary>
		/// Zkontroluje, zda je daný tah platný podle pravidel.
		/// </summary>
		/// <param name="move">Tah provedený v zadané partii.</param>
		/// <exception cref="InvalidOperationException">Aktuální partie byla zrušena.</exception>
		/// <exception cref="UnknownPieceException">Soudce nezná pravidla pro danou figurku.</exception>
		public bool IsValid(Move move)
		{
			string str;
			// overime
			return IsValid(move, false, out str);
		}

		/// <summary>
		/// Zkontroluje, zda je daný tah nebo ohrožování platné podle pravidel.
		/// </summary>
		/// <param name="move">Tah provedený v zadané partii.</param>
		/// <param name="onlyThreaten">True, má-li se testovat pouze ohrožování tímto tahem.</param>
		/// <exception cref="InvalidOperationException">Aktuální partie byla zrušena.</exception>
		/// <exception cref="UnknownPieceException">Soudce nezná pravidla pro danou figurku.</exception>
		public bool IsValid(Move move, bool onlyThreaten)
		{
			string str;
			// overime
			return IsValid(move, onlyThreaten, out str);
		}
		
		/// <summary>
		/// Zkontroluje, zda je daný tah platný podle pravidel.
		/// </summary>
		/// <param name="move">Tah provedený v zadané partii.</param>
		/// <param name="status">Komentář soudce k tahu.</param>
		/// <exception cref="InvalidOperationException">Aktuální partie byla zrušena.</exception>
		/// <exception cref="UnknownPieceException">Soudce nezná pravidla pro danou figurku.</exception>
		public bool IsValid(Move move, out string status)
		{
			// overime
			return IsValid(move, false, out status);
		}

		/// <summary>
		/// Zkontroluje, zda je daný tah nebo ohrožování platné podle pravidel.
		/// </summary>
		/// <param name="move">Tah provedený v zadané partii.</param>
		/// <param name="onlyThreaten">True, má-li se testovat pouze ohrožování tímto tahem.</param>
		/// <param name="status">Komentář soudce k tahu.</param>
		/// <exception cref="InvalidOperationException">Aktuální partie byla zrušena.</exception>
		/// <exception cref="UnknownPieceException">Soudce nezná pravidla pro danou figurku.</exception>
		public bool IsValid(Move move, bool onlyThreaten, out string status)
		{
			// pokud nam sebrali partii, vyhodime vyjimku
			if(_Game == null) throw new InvalidOperationException("Partie byla zrušena.");

			status = string.Empty;

			ThinkingCounter.Instance["Judge.IsValid"]++;

			// figurka se pri tahu musi posunout
			if(move.From == move.To)
			{
				status = "Figurka se při tahu musí posunout z místa.";
				return false;
			}

			// zkontrolujeme, zda je tah v mezich sachovnice
			if(!Chessboard.IsPositionInside(move.To))
			{
				status = "Figurka se nesmí přesunout mimo šachovnici.";
				return false;
			}

			// zkontrolujeme zakladni pravidla pro pohyb figurek
			if(CheckPieceDraw(move, ref status) == false)
				return false;

			// pokud tahne pesec, zkontrolujeme pro nej dalsi pravidla
			if(move.Piece is Pawn && CheckPawnDraw(move, _Game, onlyThreaten, ref status) == false)
				return false;

			// kontrola preskakovani
			if(CheckNonKnightDraw(move, ref status) == false)
				return false;

			// pokud se nekdo pokousi zajmout krale, neni to mozne
			if(!onlyThreaten && move.Piece.Chessboard.GetPieceOnTile(move.To) is King)
			{
				status = "Krále není možno zajmout.";
				return false;
			}

			// pokud jen nezjistujeme ohrozeni,
			if(onlyThreaten == false)
			{
				// zjistime novy stav sachovnice
				Chessboard novy = ChessboardAfterMove(_Game, move);
				// budem-li mit sach, neni to dobre
				if(IsCheck(novy, _Game, move.Piece.Color))
				{
					status = "Hráč se nesmí vlastním tahem ocitnout v šachu.";
					return false;
				}
			}

			ThinkingCounter.Instance["Judge.IsValid do konce"]++;

			// bude to ok
			status = "Tah proběhne podle pravidel.";
			return true;
		}

		/// <summary>
		/// Zjistí, zda je daný tah platný pro figurku.
		/// </summary>
		private static bool CheckPieceDraw(Move move, ref string status)
		{
			RelativePosition[] tahy;
			Chessboard chessboard = move.Piece.Chessboard;

			// nejdriv se podivame, kdo stoji na cilove pozici
			Piece figurka = chessboard.GetPieceOnTile(move.To);
			// pokud tam nekdo stoji a je to vlastni figurka, tah neni platny
			if(figurka != null && figurka.Color == move.Piece.Color)
			{
				status = "Figurka se nemůže přesunout na místo, kde stojí vlastní figurka.";
				return false;
			}

			// ziskame mozne tahy dane figurky
			tahy = GetPiecePossibleDraws(move.Piece.GetType());

			// zjistime relativni pozici
			RelativePosition rp = move.From.RelativeTo(move.To);
			bool nasli = false;

			// projdeme mozne pozice
			foreach(RelativePosition mozna in tahy)
			{
				// pokud je to ona, muze se tam presunout
				if(rp == mozna)
				{
					nasli = true;
					break;
				}
			}
			if(nasli == false)
			{
				status = "Figurka takto nemůže táhnout.";
				return false;
			}

			// tah je platny
			return true;
		}

		/// <summary>
		/// Zjistí, zda je daný tah platný pro pěšce.
		/// </summary>
		private static bool CheckPawnDraw(Move move, Game game, bool onlyThreaten, ref string status)
		{
			Debug.Assert(move.Piece is Pawn, 
				"Funkce CheckPawnDraw nekontroluje figurky typu " + move.Piece.GetType() + ".");
			const string PawnMustNotRetreat = "Pěšec se nesmí vracet.";
			const string PawnMustNotJump = "Pěšec se může posunout o 2 pole pouze ze základní pozice.";
			// zjistime relativni pozici
			RelativePosition rp = move.From.RelativeTo(move.To);

			Chessboard chessboard = move.Piece.Chessboard;

			// pokud je to spodni hrac,
			if(game.GetPlayer(move.Piece.Color) == game.Player1)
			{
				// pesec nesmi jit dolu
				if(rp.dY > 0)
				{
					status = PawnMustNotRetreat;
					return false;
				}
				// jde-li o 2 policka, musi to byt ze zakladni pozice
				if(rp.dY == -2 && move.From.Y != Chessboard.Size - 2)
				{
					status = PawnMustNotJump;
					return false;
				}
			}
			else
			{
				// jinak, je-li to horni hrac,
				// pesec nesmi jit nahoru
				if(rp.dY < 0)
				{
					status = PawnMustNotRetreat;
					return false;
				}
				// jde-li o 2 policka, musi to byt ze zakladni pozice
				if(rp.dY == 2 && move.From.Y != 1)
				{
					status = PawnMustNotJump;
					return false;
				}
			}

			// pokud mame testovat jenom ohrozovani,
			if(onlyThreaten)
			{
				// musi to byt do boku
				if(rp.dX == 0)
				{
					status = "Pěšec může zajímat pouze šikmým tahem.";
					return false;
				}
			}
			else
			{
				// jinak, mame-li testovat tah
				// podivame se, kdo stoji na cilove pozici
				Piece figurka = chessboard.GetPieceOnTile(move.To);
				Pawn pesec;
				// jde-li do boku, musi to byt pri zajimani jine figurky nebo pri brani mimochodem
				if(rp.dX != 0 && figurka == null && !CanCaptureEnPassant(game, move, out pesec))
				{
					status = "Pěšec může šikmo pouze zajímat figurky.";
					return false;
				}
				// jde-li dopredu, musi tam byt volno
				else if(rp.dX == 0 && figurka != null)
				{
					status = "Směrem dopředu nemůže pěšec zajímat.";
					return false;
				}
			}

			// ok
			return true;
		}

		/// <summary>
		/// Zjistí, zda je daný tah platný pro figurku kromě koně.
		/// </summary>
		private static bool CheckNonKnightDraw(Move move, ref string status)
		{
			// figurka nesmi preskakovat
			if(move.Piece.Chessboard.IsPathClear(move) == false)
			{
				status = "Tato figurka nemůže přeskakovat jiné.";
				return false;
			}

			// je to ok
			return true;
		}

		/// <summary>
		/// Vrátí řádek šachovnice, na kterém se daný pěšec promění.
		/// </summary>
		/// <param name="pawn">Pěšec, jehož cílový řádek se má vrátit.</param>
		public int GetPawnDestinationRow(Pawn pawn)
		{
			// pokud patri spodnimu hraci, jde nahoru
			if(_Game.GetPlayer(pawn.Color) == _Game.Player1)
				return 0;
			// jinak jde dolu
			else
				return Chessboard.Size - 1;
		}

		/// <summary>
		/// Vrátí krále dané barvy nebo null.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které se bude hledat.</param>
		/// <param name="color">Barva hledaného krále.</param>
		public static King GetKing(Chessboard chessboard, PieceColor color)
		{
			// projdeme figurky
			foreach(Piece figurka in chessboard)
			{
				// pokud je to kral zadane barvy, vratime ho
				if(figurka is King && figurka.Color == color)
					return (King)figurka;
			}

			// nenasli
			return null;
		}

		/// <summary>
		/// Zjistí, zda je daná barva v šachu.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které se bude testovat.</param>
		/// <param name="game">Partie, která se bude testovat.</param>
		/// <param name="color">Barva, pro níž se šach má testovat.</param>
		public static bool IsCheck(Chessboard chessboard, Game game, PieceColor color)
		{
			// vezmeme krale
			King kral = GetKing(chessboard, color);
			Debug.Assert(kral != null, "Král je pryč.");
			// neni-li kral, neni ani sach
			if(kral == null)
				return false;

			// nechame vygenerovat mozne utocne tahy pro protivnikovy figurky
			Move[] pole = Generator.GeneratePossibleMoves(chessboard, game, Player.ContraryColor(color),
				ValidThreatensFilter);
			
			// projdeme je
			foreach(Move tah in pole)
			{
				// pokud je mezi nimi utok na krale, jsme v sachu
				if(tah.To == kral.Position)
					return true;
			}

			// neni sach
			return false;
		}

		/// <summary>
		/// Zjistí, zda je daná barva v patu.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které se bude testovat.</param>
		/// <param name="game">Partie, která se bude testovat.</param>
		/// <param name="color">Barva, pro níž se pat má testovat.</param>
		public static bool IsStalemate(Chessboard chessboard, Game game, PieceColor color)
		{
			// nechame vygenerovat mozne tahy vlastnich figurek
			Move[] pole = Generator.GeneratePossibleMoves(chessboard, game, color, ValidMovesFilter);

			// nemuzem-li se pohnout, je pat
			return pole.Length == 0;

			//// projdeme je
			//foreach(Move tah in pole)
			//{
			//    // zjistime stav sachovnice po tahu
			//    Chessboard stav = ChessboardAfterMove(game, tah);
			//    // neni-li tam sach, nejsme v patu
			//    if(IsCheck(stav, game, color) == false)
			//        return false;
			//}

			//// jsme v patu
			//return true;
		}

		/// <summary>
		/// Vytvoří novou šachovnici vzniklou provedením tahu na zadané šachovnici.
		/// </summary>
		/// <remarks>
		/// Vytváří nové kopie všech figurek.
		/// </remarks>
		/// <param name="game">Partie, ve které se tah provádí.</param>
		/// <param name="move">Tah, jehož cílová pozice novou pozicí dané figurky na nové šachovnici.</param>
		/// <exception cref="UnknownPieceException">Nalezena neznámá figurka.</exception>
		/// <exception cref="ArgumentNullException">Některý parametr je null.</exception>
		public static Chessboard ChessboardAfterMove(Game game, Move move)
		{
			return ChessboardAfterMove(game, move, move.Piece.Chessboard);
		}

		/// <summary>
		/// Vytvoří novou šachovnici vzniklou provedením tahu na zadané šachovnici.
		/// </summary>
		/// <remarks>
		/// Vytváří nové kopie všech figurek.
		/// </remarks>
		/// <param name="game">Partie, ve které se tah provádí.</param>
		/// <param name="move">Tah, jehož cílová pozice novou pozicí dané figurky na nové šachovnici.</param>
		/// <param name="chessboard">Šachovnice, na které se provede tah.</param>
		/// <exception cref="UnknownPieceException">Nalezena neznámá figurka.</exception>
		/// <exception cref="ArgumentNullException">Některý parametr je null.</exception>
		public static Chessboard ChessboardAfterMove(Game game, Move move, Chessboard chessboard)
		{
			// zkontrolujeme parametry
			if(move == null) throw new ArgumentNullException("move");
			if(move.Piece == null) throw new ArgumentNullException("move.Piece");
			if(move.Piece.Chessboard == null) throw new ArgumentNullException("move.Piece.Chessboard");

			ThinkingCounter.Instance["Judge.ChessboardAfterMove"]++;

			// nova sachovnice
			Chessboard po_tahu = new Chessboard(chessboard.PieceCount);

			// figurka, ktera se bude presouvat
			Piece presunout = null;

			// zkopirujeme seznam figurek
			foreach(Piece figurka in chessboard)
			{
				Piece nova;

				// vytvorime prislusnou kopii
				if(figurka is Pawn)
					nova = new Pawn(po_tahu, (Pawn)figurka);
				else if(figurka is Bishop)
					nova = new Bishop(po_tahu, (Bishop)figurka);
				else if(figurka is King)
					nova = new King(po_tahu, (King)figurka);
				else if(figurka is Queen)
					nova = new Queen(po_tahu, (Queen)figurka);
				else
					// neznama figurka
					throw new UnknownPieceException(figurka);

				// pokud je to figurka, se kterou se tahne, presuneme ji
				if(figurka == move.Piece)
					presunout = nova;

				// pridame novou
				po_tahu.AddPiece(nova);
			}

			Debug.Assert(presunout != null, "Nenašli jsme figurku, která se má přesunout.");

			// provedeme tah
			game.Judge.DoMove(new Move(presunout, move.To));

			// vratime novou sachovnici
			return po_tahu;
		}

		/// <summary>
		/// Vrátí hodnotu určující, zda může pěšec tímto tahem vzít soupeřova pěšce mimochodem.
		/// </summary>
		/// <param name="game">Partie, ve které se tah odehrává.</param>
		/// <param name="move">Tah útočícího pěšce.</param>
		/// <param name="takenPawn">Obdrží soupeřova pěšce sebraného mimochodem.</param>
		private static bool CanCaptureEnPassant(Game game, Move move, out Pawn takenPawn)
		{
			takenPawn = null;

			return false;

			//// pokud to neni pesec,
			//if(!(move.Piece is Pawn))
			//{
			//    // neni mozno
			//    Debug.Fail("Jiná figurka než pěšec nemůže brát mimochodem.");
			//    return false;
			//}

			//// pokud neni na cili volno (coz vlastne vylucuje, aby souper skocil o 2), nemuze
			//if(move.Piece.Chessboard.GetPieceOnTile(move.To) != null)
			//    return false;

			//// vezmeme historii
			//MoveInfo[] historie = game.GetHistory();
			//// pokud predtim nic nebylo, nelze
			//if(historie.Length == 0)
			//    return false;

			//// vezmeme predchozi tah
			//Move predchozi = historie[historie.Length - 1].Move;
			//Debug.Assert(predchozi.Piece.Color != move.Piece.Color, "Předchozí tah byl také náš.");
			//// pokud to nebyl tah pindikem, nelze
			//if(!(predchozi.Piece is Pawn))
			//    return false;

			//// pokud nebyl predchozi souperuv tah skok pindikem vedle, nelze
			//bool je_na_stejnem_radku = predchozi.To.Y == move.From.Y;
			//bool je_vedle_ve_sloupci = Math.Abs(predchozi.To.X - move.From.X) == 1;
			//if(predchozi.OverleapedTiles != 3 || !je_na_stejnem_radku || !je_vedle_ve_sloupci)
			//    return false;

			//// je mozno
			//takenPawn = (Pawn)predchozi.Piece;
			//return true;
		}

		/// <summary>
		/// Provede tah a aplikuje pravidla.
		/// </summary>
		public void DoMove(Move move)
		{
			//string str;
			Chessboard sachovnice = move.Piece.Chessboard;
			// figurka na cili
			Piece cil = sachovnice.GetPieceOnTile(move.To);

			// zpusobuje preteceni zasobniku
			//Debug.Assert(IsValid(move, false, out str), "Tah není platný. " + str);

			// provedeme tah figurkou
			move.Piece.Position = move.To;

			Pawn pesak_sebrany;

			// je-li na cili figurka,
			if(cil != null)
			{
				// odstranime ji ze sachovnice
				sachovnice.RemovePiece(cil);
			}
			// jinak pokud bere pesak mimochodem,
			else if(move.Piece is Pawn && CanCaptureEnPassant(Game, move, out pesak_sebrany))
			{
				// odstranime sebraneho pesaka
				sachovnice.RemovePiece(pesak_sebrany);
			}

			Pawn pesak = move.Piece as Pawn;
			// je-li to pesak a dosel-li na konec,
			if(pesak != null && pesak.Position.Y == GetPawnDestinationRow(pesak))
			{
				// dame ho pryc ze sachovnice
				sachovnice.RemovePiece(pesak);
				// promenime ho v damu
				sachovnice.AddPiece(new Queen(sachovnice, pesak.Position, pesak.Color));
			}
		}

		/// <summary>
		/// Provede přesně popsaný tah.
		/// </summary>
		/// <param name="moveInfo">Informace o tahu, který se má provést.</param>
		public void DoMove(MoveInfo moveInfo)
		{
			Chessboard sachovnice = moveInfo.Move.Piece.Chessboard;

			// provedeme tah
			moveInfo.Move.Piece.Position = moveInfo.Move.To;

			// pokud byla nejaka figurka zajmuta, odstranime ji
			if(moveInfo.CapturedPiece != null)
				sachovnice.RemovePiece(moveInfo.CapturedPiece);
			// jinak pokud byl nejaky pindik sebran mimochodem, taky ho odstranime
			else if(moveInfo.PawnCapturedEnPassant != null)
				sachovnice.RemovePiece(moveInfo.PawnCapturedEnPassant);

			// pokud se pesak v neco promenil,
			if(moveInfo.PawnConvertedTo != null)
			{
				Debug.Assert(moveInfo.Move.Piece is Pawn, "Na konci šachovnice se proměnilo něco jiného než pěšák.");

				// dame ho pryc ze sachovnice
				sachovnice.RemovePiece(moveInfo.Move.Piece);
				// a pridame tam novou figuru
				sachovnice.AddPiece(moveInfo.PawnConvertedTo);
				// a umistime ji na spravne misto
				moveInfo.PawnConvertedTo.Position = moveInfo.Move.To;
			}
		}

		/// <summary>
		/// Provede tah, aplikuje pravidla a vrátí informace o provedeném tahu.
		/// </summary>
		/// <param name="move">Tah podle pravidel, který se má provést.</param>
		/// <returns>Vrací informace o provedeném tahu a jeho následcích.</returns>
		public MoveInfo DoMoveAndDescribe(Move move)
		{
			King v_matu = null;
			PieceColor barva_nepritele = Player.ContraryColor(move.Piece.Color);
			King kral = GetKing(move.Piece.Chessboard, barva_nepritele);

			Debug.Assert(kral != null, "Král je pryč.");

			// figurka na cili
			Piece cil = move.Piece.Chessboard.GetPieceOnTile(move.To);

			// provedeme tah
			DoMove(move);

			// kontrola, zda nepritel nedostal sach
			King v_sachu = IsCheck(move.Piece.Chessboard, _Game, barva_nepritele) ? kral : null;

			// kontrola, zda nepriteli nenastal pat
			bool pat = IsStalemate(move.Piece.Chessboard, _Game, barva_nepritele);

			// pokud dostal sach a pat,
			if(v_sachu != null && pat)
			{
				// je to mat
				v_matu = v_sachu;
				pat = false;
			}

			Debug.Assert(v_matu == null || !pat, "Nemůže být zároveň mat a pat.");
			Debug.Assert(v_matu == null ? (v_sachu == null || !pat) : true, "Nemůže být šach a pat, ale zároveň nebýt mat.");

			Piece promena = null;
			// zjistime, tahne-li pesec
			bool byl_pesec = move.Piece is Pawn;
			// promena pesce
			if(byl_pesec)
			{
				// figurka na pozici
				Piece nova = move.Piece.Chessboard.GetPieceOnTile(move.Piece.Position);
				promena = (nova == move.Piece) ? null : nova;
			}

			// vytvorime info
			MoveInfo mi = new MoveInfo(move, cil, v_sachu, v_matu, pat, pat, promena, null,
				IsThreefoldRepetition(), IsFiftyMoveRule());

			// pripojime popis
			mi.AlgebraicNotation = Game.GetMoveDescription(mi);

			// vratime vysledek
			return mi;
		}

		/// <summary>
		/// Zjistí, zda nyní v partii došlo k třetímu opakování stejné pozice.
		/// </summary>
		private bool IsThreefoldRepetition()
		{
			// pokud je v historii malo tahu, konec
			if(Game.CurrentMoveNumber < 7)
				return false;

			// vezmeme historii
			MoveInfo[] historie = Game.GetHistory();

			// vezmeme pocatecni pozici
			Chessboard sachovnice = Game.StartupChessboard;

			int pocet_stejnych = 0;

			// projdeme historii
			foreach(MoveInfo tah in historie)
			{
				// pokud je stejna, zvysime pocet
				if(Game.Chessboard.HasSamePosition(sachovnice))
					pocet_stejnych++;

				// jsou-li 2, opakuje se (posledni tah neni jeste v historii, lec na sachovnici uz ano)
				if(pocet_stejnych == 2)
					return true;

				// vytvorime novy fiktivni tah na sachovnici
				// toto je hack kvuli spatnemu navrhu:(
				Piece figurka = sachovnice.GetPieceOnTile(tah.Move.From);
				Debug.Assert(figurka != null, "Nenašli jsme figurku.");
				Move novy_tah = new Move(figurka, tah.Move.To);
				// posunem se o tah
				sachovnice = ChessboardAfterMove(Game, novy_tah, sachovnice);
			}

			// neopakuji se
			return false;
		}

		/// <summary>
		/// Zjistí, zda nyní v partii došlo k 50. tahu bez zajmutí figurky či posunutí pěšce.
		/// </summary>
		private bool IsFiftyMoveRule()
		{
			// pokud je v historii malo tahu, konec
			if(Game.CurrentMoveNumber <= 50*2)
				return false;

			// vezmeme historii
			MoveInfo[] historie = Game.GetHistory();
			
			// zjistime 1. tah z poslednich 50
			int prvni = historie.Length - 50*2;

			Debug.Assert(prvni >= 0 && prvni < historie.Length, "První tah z 50 je mimo historii.");

			// projdeme 50 poslednich tahu
			for(int i = prvni; i < historie.Length; i++)
			{
				// je-li tu zajmuti, konec
				if(historie[i].CapturedPiece != null || historie[i].PawnCapturedEnPassant != null)
					return false;

				// je-li tu pohyb pindika, konec
				if(historie[i].Move.Piece is Pawn)
					return false;
			}

			// jinak muze byt remiza vyhlasena
			return true;
		}

		/// <summary>
		/// Vrátí provedené zadaného tahu.
		/// </summary>
		/// <param name="move">Informace o tahu, který má být vrácen.</param>
		public void RevertMove(MoveInfo move)
		{
			Chessboard sachovnice = move.Move.Piece.Chessboard;

			Debug.Assert(move.Move.Piece.Position == move.Move.To, "Figurka nestojí na cílové pozici.");

			// pokud byl nekdo zajat,
			if(move.CapturedPiece != null)
			{
				// vratime ho
				sachovnice.AddPiece(move.CapturedPiece);
			}

			// pokud se pindik premenil,
			if(move.PawnConvertedTo != null)
			{
				// novou figuru odebereme
				sachovnice.RemovePiece(move.PawnConvertedTo);
				// a vratime tam pindika
				sachovnice.AddPiece(move.Move.Piece);
			}

			// presuneme figurku zpatky
			move.Move.Piece.Position = move.Move.From;
		}

		/// <summary>
		/// Zjistí, zda je daná herní situace platná podle pravidel.
		/// </summary>
		/// <param name="game">Partie.</param>
		/// <param name="color">Barva, z jejíhož pohledu se má testovat.</param>
		/// <param name="comment">Komentář k situaci.</param>
		public static bool IsValid(Game game, PieceColor color, out string comment)
		{
			King kral = null;
			int pocet_kralu = 0, pocet_strelcu = 0, pocet_kraloven = 0, pocet_pescu = 0;

			// projdeme figurky
			foreach(Piece f in game.Chessboard)
			{
				if(f.Color == color)
				{
					if(f is King)
					{
						kral = (King)f;
						pocet_kralu++;
					}
					else if(f is Bishop)
					{
						pocet_strelcu++;
					}
					else if(f is Queen)
					{
						pocet_kraloven++;
					}
					else if(f is Pawn)
					{
						pocet_pescu++;
					}
				}
			}

			// pokud neni kral, konec
			if(pocet_kralu == 0)
			{
				comment = string.Format("Na šachovnici chybí {0} král.",
					color == PieceColor.White ? "bílý" : "černý");
				return false;
			}
			// pokud jich je vic, je to taky spatne
			if(pocet_kralu > 1)
			{
				comment = string.Format("Na šachovnici je více než jeden {0} král.",
					color == PieceColor.White ? "bílý" : "černý");
				return false;
			}

			// strelcu nemuze byt vic, nez 8+2
			if(pocet_strelcu > 10)
			{
				comment = "Střelců nemůže být více než 10.";
				return false;
			}
			// dam vic nez 8+1
			if(pocet_kraloven > 9)
			{
				comment = "Dam nemůže být více než 9.";
				return false;
			}
			// pescu vic nez 8
			if(pocet_pescu > 8)
			{
				comment = "Pěšáků nemůže být více než 8.";
				return false;
			}

			// projdeme okolni pozice krale
			foreach(RelativePosition rel in _PieceMoves[typeof(King)])
			{
				// vezmeme okolni pole
				Position pozice = rel.Place(kral.Position);
				// pokud tam stoji jiny kral, je to spatne
				if(game.Chessboard.GetPieceOnTile(pozice) is King)
				{
					comment = "Králové stojí vedle sebe.";
					return false;
				}
			}

			// projdeme pesaky
			foreach(Piece pesak in game.Chessboard)
			{
				if(pesak is Pawn)
				{
					// pokud stoji za svou pocatecni pozici, konec
					if((game.GetPlayer(color) == game.Player1 && pesak.Position.Y == Chessboard.Size - 1) ||
					   (game.GetPlayer(color) == game.Player2 && pesak.Position.Y == 0))
					{
						comment = "Pěšák stojí za svou počáteční pozicí.";
						return false;
					}
				}
			}

			comment = "Situace na šachovnici je v pořádku.";
			return true;
		}
	}
}
