using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;

namespace Stelci
{
	/// <summary>
	/// Pedstavuje vjimku nastvajc pi pokusu o pstup mimo achovnici.
	/// </summary>
	public class OutOfChessboardException : Exception
	{
		/// <summary>
		/// Vytvo novou vjimku.
		/// </summary>
		public OutOfChessboardException()
			: base()
		{
		}

		/// <summary>
		/// Vytvo novou vjimku se zadanou chybovou zprvou.
		/// </summary>
		/// <param name="message">Zprva, kter popisuje chybu.</param>
		public OutOfChessboardException(string message)
			: base(message)
		{
		}

		/// <summary>
		/// Vytvo novou vjimku s implicitn zprvou.
		/// </summary>
		/// <param name="position">Pozice na achovnici.</param>
		public OutOfChessboardException(Position position)
			: base("Pozice " + position.ToString() + " je mimo achovnici o rozmrech " +
				Chessboard.Size.ToString() + "x" + Chessboard.Size.ToString() + ".")
		{
		}
	}

	/// <summary>
	/// Pedstavuje pozici na achovnici.
	/// </summary>
	public struct Position
	{
		private int _X;
		private int _Y;

		/// <summary>
		/// Souadnice X na achovnici.
		/// </summary>
		public int X
		{
			get { return _X; }
		}

		/// <summary>
		/// Souadnice Y na achovnici.
		/// </summary>
		public int Y
		{
			get { return _Y; }
		}

		/// <summary>
		/// Vytvo novou pozici na achovnici.
		/// </summary>
		/// <param name="x">Souadnice X na achovnici.</param>
		/// <param name="y">Souadnice Y na achovnici.</param>
		public Position(int x, int y)
		{
			_X = x;
			_Y = y;
		}

		/// <summary>
		/// Pevede pozici do textov podoby.
		/// </summary>
		public override string ToString()
		{
			return "[" + _X.ToString() + ", " + _Y.ToString() + "]";
		}

		/// <summary>
		/// Vrt relativn pozici na zadanou pozici.
		/// </summary>
		/// <param name="position">Pozice, do kter povede relativn pozice z aktuln.</param>
		public RelativePosition RelativeTo(Position position)
		{
			return new RelativePosition(position.X - _X, position.Y - _Y);
		}

		#region Technick detaily - opertory

		public override bool Equals(System.Object obj)
		{
			try
			{
				Position p = (Position)obj;
				// porovname pozici
				return _X == p._X && _Y == p._Y;
			}
			catch(InvalidCastException)
			{
				// obj neni pozice
				return false;
			}
		}

		public bool Equals(Position p)
		{
			// porovname pozici
			return _X == p._X && _Y == p._Y;
		}

		public override int GetHashCode()
		{
			return _X ^ _Y;
		}

		public static bool operator ==(Position a, Position b)
		{
			return a.Equals(b);
		}
		public static bool operator !=(Position a, Position b)
		{
			return !(a == b);
		}

		#endregion
	}

	/// <summary>
	/// Pedstavuje pltah figurky.
	/// </summary>
	public class Move
	{
		private Position _From;
		private Position _To;
		private Piece _Piece;

		/// <summary>
		/// Vchoz pozice tahu.
		/// </summary>
		public Position From
		{
			get { return _From; }
		}

		/// <summary>
		/// Clov pozice tahu.
		/// </summary>
		public Position To
		{
			get { return _To; }
		}

		/// <summary>
		/// Figurka, se kterou se thne.
		/// </summary>
		public Piece Piece
		{
			get { return _Piece; }
		}

		/// <summary>
		/// Vrt poet polek pekroench tahem. Tahy jezdce nejsou implementovny.
		/// </summary>
		public int OverleapedTiles
		{
			get
			{
				return (int)Math.Sqrt(Math.Pow((double)To.X - From.X, 2.0) +
									  Math.Pow((double)To.Y - From.Y, 2.0)) + 1;
			}
		}

		/// <summary>
		/// Vytvo nov tah figurkou na zadanou pozici.
		/// </summary>
		/// <param name="piece">Figurka, se kterou se thne.</param>
		/// <param name="to">Clov pozice tahu.</param>
		public Move(Piece piece, Position to)
		{
			_Piece = piece;
			_From = _Piece.Position;
			_To = to;
		}

		/// <summary>
		/// Vytvo nov tah figurkou mezi pozicemi.
		/// </summary>
		/// <param name="piece">Figurka, se kterou se thne.</param>
		/// <param name="from">Vchoz pozice tahu.</param>
		/// <param name="to">Clov pozice tahu.</param>
		public Move(Piece piece, Position from, Position to)
		{
			_Piece = piece;
			_From = from;
			_To = to;
		}

		/// <summary>
		/// Vytvo nov tah jako relativn posun figurky.
		/// </summary>
		/// <param name="piece">Figurka, se kterou se thne.</param>
		/// <param name="relativeMove">Relativn pozice, o kterou se figurka posune.</param>
		public Move(Piece piece, RelativePosition relativeMove)
		{
			_Piece = piece;
			_From = _Piece.Position;
			_To = relativeMove.Place(_From);
		}

		/// <summary>
		/// Pevede tah do textov podoby.
		/// </summary>
		public override string ToString()
		{
			return "Tah z " + _From + " na " + _To + " figurkou " + _Piece;
		}
	}

	/// <summary>
	/// Pedstavuje relativn pozici na achovnici.
	/// </summary>
	public struct RelativePosition
	{
		private int _dX;
		private int _dY;

		/// <summary>
		/// Relativn souadnice X na achovnici.
		/// </summary>
		public int dX
		{
			get { return _dX; }
		}

		/// <summary>
		/// Relativn souadnice Y na achovnici.
		/// </summary>
		public int dY
		{
			get { return _dY; }
		}

		/// <summary>
		/// Vytvo novou relativn pozici.
		/// </summary>
		/// <param name="dx">Relativn souadnice X na achovnici.</param>
		/// <param name="dy">Relativn souadnice Y na achovnici.</param>
		public RelativePosition(int dx, int dy)
		{
			_dX = dx;
			_dY = dy;
		}

		/// <summary>
		/// Pevede pozici do textov podoby.
		/// </summary>
		public override string ToString()
		{
			return "(" + _dX + ", " + _dY + ")";
		}

		/// <summary>
		/// Umst relativn pozici na danou pozici.
		/// </summary>
		/// <param name="basePosition">Zkladn pozice, do kter se umst relativn pozice.</param>
		public Position Place(Position basePosition)
		{
			return new Position(basePosition.X + _dX, basePosition.Y + _dY);
		}

		/// <summary>
		/// Vytvo adu relativnch pozic zadanho polomru.
		/// </summary>
		/// <param name="length">Polomr ady relativnch pozic.</param>
		/// <returns>Pole relativnch pozic od (-radius, 0) do (radius, 0).</returns>
		public static RelativePosition[] CreateRow(int radius)
		{
			int pocet = 2 * radius + 1;
			RelativePosition[] pole = new RelativePosition[pocet];

			// naplnime
			for(int i = 0; i < pocet; i++)
				pole[i] = new RelativePosition(-radius + i, 0);

			return pole;
		}

		/// <summary>
		/// Vytvo sloupec relativnch pozic zadanho polomru.
		/// </summary>
		/// <param name="length">Polomr sloupce relativnch pozic.</param>
		/// <returns>Pole relativnch pozic od (0, -radius) do (0, radius).</returns>
		public static RelativePosition[] CreateColumn(int radius)
		{
			int pocet = 2 * radius + 1;
			RelativePosition[] pole = new RelativePosition[pocet];

			// naplnime
			for(int i = 0; i < pocet; i++)
				pole[i] = new RelativePosition(0, -radius + i);

			return pole;
		}

		/// <summary>
		/// Vytvo diagonlu relativnch pozic zadanho polomru zleva doprava.
		/// </summary>
		/// <param name="length">Polomr diagonly relativnch pozic.</param>
		/// <returns>Pole relativnch pozic od (-radius, -radius) do (radius, radius).</returns>
		public static RelativePosition[] CreateLRDiagonal(int radius)
		{
			int pocet = 2 * radius + 1;
			RelativePosition[] pole = new RelativePosition[pocet];

			// naplnime
			for(int i = 0; i < pocet; i++)
				pole[i] = new RelativePosition(-radius + i, -radius + i);

			return pole;
		}

		/// <summary>
		/// Vytvo diagonlu relativnch pozic zadanho polomru zprava doleva.
		/// </summary>
		/// <param name="length">Polomr diagonly relativnch pozic.</param>
		/// <returns>Pole relativnch pozic od (radius, -radius) do (-radius, radius).</returns>
		public static RelativePosition[] CreateRLDiagonal(int radius)
		{
			int pocet = 2 * radius + 1;
			RelativePosition[] pole = new RelativePosition[pocet];

			// naplnime
			for(int i = 0; i < pocet; i++)
				pole[i] = new RelativePosition(radius - i, -radius + i);

			return pole;
		}

		/// <summary>
		/// Vytvo tverec okolnch relativnch pozic.
		/// </summary>
		public static RelativePosition[] CreateNeighborhood()
		{
			RelativePosition[] pole = new RelativePosition[8];
			int i = 0;

			// naplnime
			for(int x = -1; x < 2; x++)
			{
				for(int y = -1; y < 2; y++)
				{
					if(x != 0 || y != 0)
						pole[i++] = new RelativePosition(x, y);
				}
			}

			return pole;
		}

		#region Technick detaily - opertory

		public override bool Equals(System.Object obj)
		{
			try
			{
				RelativePosition p = (RelativePosition)obj;
				// porovname pozici
				return _dX == p._dX && _dY == p._dY;
			}
			catch(InvalidCastException)
			{
				// obj neni pozice
				return false;
			}
		}

		public bool Equals(RelativePosition p)
		{
			// porovname pozici
			return _dX == p._dX && _dY == p._dY;
		}

		public override int GetHashCode()
		{
			return _dX ^ _dY;
		}

		public static bool operator ==(RelativePosition a, RelativePosition b)
		{
			return a.Equals(b);
		}
		public static bool operator !=(RelativePosition a, RelativePosition b)
		{
			return !(a == b);
		}

		#endregion

	}
}