﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;

namespace Střelci
{
	/// <summary>
	/// Představuje výjimku, která nastane, když se v aplikaci objeví neznámý druh figurky.
	/// </summary>
	public class UnknownPieceException : Exception
	{
		/// <summary>
		/// Vytvoří novou výjimku.
		/// </summary>
		public UnknownPieceException() : base()
		{
		}

		/// <summary>
		/// Vytvoří novou výjimku se zadanou chybovou zprávou.
		/// </summary>
		/// <param name="message">Zpráva, která popisuje chybu.</param>
		public UnknownPieceException(string message) : base(message)
		{
		}

		/// <summary>
		/// Vytvoří novou výjimku se zadanou chybovou zprávou a předchozí výjimkou.
		/// </summary>
		/// <param name="message">Zpráva, která popisuje chybu.</param>
		/// <param name="innerException">Výjimka, která způsobila vyvolání této výjimky.</param>
		public UnknownPieceException(string message, Exception innerException)
			: base(message, innerException)
		{
		}

		/// <summary>
		/// Vytvoří novou výjimku s implicitní zprávou.
		/// </summary>
		/// <param name="piece">Figurka, jejíž druh je neznámý.</param>
		public UnknownPieceException(Piece piece) :
			base("Figurka \"" + piece.ToString() + "\" je neznámého typu " + piece.GetType().ToString() + ".")
		{
		}
	}

	/// <summary>
	/// Druh figurky.
	/// </summary>
	public enum PieceType
	{
		/// <summary>
		/// Pěšec.
		/// </summary>
		Pawn,
		/// <summary>
		/// Střelec.
		/// </summary>
		Bishop,
		/// <summary>
		/// Dáma.
		/// </summary>
		Queen,
		/// <summary>
		/// Král.
		/// </summary>
		King
	}

	/// <summary>
	/// Představuje šachovou figurku.
	/// </summary>
	public class Piece
	{
		private Chessboard _Chessboard;
		private Position _Position;
		private PieceColor _Color;
		private PieceType _Type;

		/// <summary>
		/// Nastane, když se změní pozice figurky na šachovnici.
		/// </summary>
		public event EventHandler PositionChanged;

		private void OnPositionChanged(EventArgs e)
		{
			if(PositionChanged != null)
				PositionChanged(this, e);
		}

		/// <summary>
		/// Barva figurky.
		/// </summary>
		public PieceColor Color
		{
			get { return _Color; }
		}
	
		/// <summary>
		/// Pozice figurky na šachovnici.
		/// </summary>
		public Position Position
		{
			get { return _Position; }
			set
			{
				_Position = value;
				OnPositionChanged(EventArgs.Empty);
			}
		}
		
		/// <summary>
		/// Vrátí druh figurky.
		/// </summary>
		public PieceType Type
		{
			get { return _Type; }
		}
	
		/// <summary>
		/// Vrátí textově vyjádřenou pozici figurky na šachovnici, například c4.
		/// </summary>
		public string GetTextPosition()
		{
			return Chessboard.GetTextPosition(Position);
		}

		/// <summary>
		/// Šachovnice, na které je figurka umístěna.
		/// </summary>
		public Chessboard Chessboard
		{
			get { return _Chessboard; }
		}

		/// <summary>
		/// Vytvoří novou figurku na zadané šachovnici.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na kterou bude figurka umístěna.</param>
		/// <param name="type">Druh figurky.</param>
		/// <param name="position">Pozice na šachovnici, kde bude figurka umístěna.</param>
		/// <param name="color">Barva figurky.</param>
		/// <exception cref="ArgumentNullException">Není zadána šachovnice.</exception>
		/// <exception cref="OutOfChessboardException">Zadané políčko je mimo šachovnici.</exception>
		/// <exception cref="InvalidOperationException">Zadané políčko je již obsazeno.</exception>
		public Piece(Chessboard chessboard, PieceType type, Position position, PieceColor color)
		{
			// pokud nam dali null, vyhodime vyjimku
			if(chessboard == null) throw new ArgumentNullException("chessboard");
			
			// pokud nas umistili mimo, vyhodime vyjimku
			if(!Chessboard.IsPositionInside(position))
				throw new OutOfChessboardException(position);

			// pokud nas umistili na nekoho, taky vyhodime vyjimku
			Piece figurka = chessboard.GetPieceOnTile(position);
			if(figurka != null)
				throw new InvalidOperationException("Pozice " + position +
					" na šachovnici je již obsazena figurkou " + figurka + ".");

			_Chessboard = chessboard;
			_Type = type;
			_Position = position;
			_Color = color;
		}

		/// <summary>
		/// Vytvoří stejnou figurku na jiné šachovnici.
		/// </summary>
		/// <param name="chessboard">Šachovnice, na které bude umístěna nová figurka.</param>
		/// <param name="original">Vzor figurky.</param>
		/// <exception cref="ArgumentNullException">Není zadána šachovnice.</exception>
		public Piece(Chessboard chessboard, Piece original)
		{
			// pokud nam dali null, vyhodime vyjimku
			if(chessboard == null) throw new ArgumentNullException("chessboard");

			// nastavime vlastnosti
			_Chessboard = chessboard;
			_Type = original.Type;
			_Position = original.Position;
			_Color = original.Color;
		}

		/// <summary>
		/// Vytvoří nový tah touto figurkou na zadanou pozici.
		/// </summary>
		/// <param name="targetPosition">Cílová pozice tahu.</param>
		public Move MoveTo(Position targetPosition)
		{
			return new Move(this, _Position, targetPosition);
		}

		/// <summary>
		/// Vrátí textové označení figurky.
		/// </summary>
		public override string ToString()
		{
			StringBuilder str = new StringBuilder(25);

			// barva a druh figurky
			switch(Type)
			{
				case PieceType.Pawn:
					str.AppendFormat("{0} pěšec", Color == PieceColor.Black ? "Černý" : "Bílý");
					break;
				case PieceType.Bishop:
					str.AppendFormat("{0} střelec", Color == PieceColor.Black ? "Černý" : "Bílý");
					break;
				case PieceType.Queen:
					str.AppendFormat("{0} dáma", Color == PieceColor.Black ? "Černá" : "Bílá");
					break;
				case PieceType.King:
					str.AppendFormat("{0} král", Color == PieceColor.Black ? "Černý" : "Bílý");
					break;
				default:
					str.AppendFormat("{0} figurka", Color == PieceColor.Black ? "Černá" : "Bílá");
					break;
			}
			// pozice
			str.AppendFormat(" na {0}", Position);

			return str.ToString();
		}
	}
}
