﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Střelci
{
	/// <summary>
	/// Barva figurky.
	/// </summary>
	public enum PieceColor
	{
		/// <summary>
		/// Černá figurka.
		/// </summary>
		Black,
		/// <summary>
		/// Bílá figurka.
		/// </summary>
		White
	}

	/// <summary>
	/// Druh hráče.
	/// </summary>
	public enum PlayerType
	{
		/// <summary>
		/// Člověk.
		/// </summary>
		Human,
		/// <summary>
		/// Počítač.
		/// </summary>
		Computer
	}

	/// <summary>
	/// Představuje argumenty pro událost s hráčem.
	/// </summary>
	public class PlayerEventArgs : EventArgs
	{
		private Player _Player;

		/// <summary>
		/// Hráč, jehož se událost týká.
		/// </summary>
		public Player Player
		{
			get { return _Player; }
		}

		/// <summary>
		/// Vytvoří nové argumenty pro událost s hráčem.
		/// </summary>
		/// <param name="piece">Hráč, jehož se událost týká.</param>
		public PlayerEventArgs(Player player)
		{
			_Player = player;
		}
	}

	/// <summary>
	/// Představuje hráče.
	/// </summary>
	public class Player : ICloneable
	{
		private PlayerType _Type;
		private PieceColor _Color;
		private int _ThinkingDepth = 2;

		/// <summary>
		/// Nastane, když se změní parametry hráče.
		/// </summary>
		public event EventHandler PlayerChanged;

		/// <summary>
		/// Druh hráče.
		/// </summary>
		public PlayerType Type
		{
			get { return _Type; }
			set
			{
				if(_Type != value)
				{
					// zmenime a spustime udalost
					_Type = value;
					OnPlayerChanged();
				}
			}
		}

		/// <summary>
		/// Barva, za kterou hráč hraje.
		/// </summary>
		public PieceColor Color
		{
			get { return _Color; }
		}

		/// <summary>
		/// Vrátí/nastaví hloubku přemýšlení.
		/// </summary>
		public int ThinkingDepth
		{
			get { return _ThinkingDepth; }
			set { _ThinkingDepth = value; }
		}

		protected void OnPlayerChanged()
		{
			if(PlayerChanged != null)
				PlayerChanged(this, EventArgs.Empty);
		}

		/// <summary>
		/// Vrátí opačnou barvu k zadané.
		/// </summary>
		/// <param name="color">Barva, ke které se vrátí opačná.</param>
		public static PieceColor ContraryColor(PieceColor color)
		{
			return color == PieceColor.Black ? PieceColor.White : PieceColor.Black;
		}

		/// <summary>
		/// Vytvoří nového hráče.
		/// </summary>
		/// <param name="type">Druh hráče.</param>
		/// <param name="color">Barva, za kterou hráč hraje.</param>
		public Player(PlayerType type, PieceColor color)
		{
			_Type = type;
			_Color = color;
		}

		/// <summary>
		/// Vrátí textové označení hráče.
		/// </summary>
		public override string ToString()
		{
			return (_Color == PieceColor.Black ? "Černý" : "Bílý") + " " + 
				(_Type == PlayerType.Human ? "lidský" : "počítačový") + " hráč";
		}

		#region ICloneable Members

		/// <summary>
		/// Vrátí novou kopii tohoto hráče.
		/// </summary>
		public object Clone()
		{
			Player novy = new Player(Type, Color);
			novy.ThinkingDepth = this.ThinkingDepth;
			return novy;
		}

		#endregion
	}
}
