﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Střelci
{
	/// <summary>
	/// Představuje okno nastavení nové hry.
	/// </summary>
	partial class NewGameForm : Form
	{
		private string _OldText = String.Empty;
		
		// pro vykreslovani nahledu
		private Painter _PreviewPainter;
		private Chessboard _PreviewChessboard;

		// sprava vestavenych rozmisteni
		Dictionary<string, string> _BuiltInLayouts;

		/// <summary>
		/// Představuje informace o nové hře.
		/// </summary>
		internal class NewGameInfo
		{
			private string m_LayoutFileName;
			private PieceColor m_Player1Color;
			private PieceColor m_Player2Color;
			private PlayerType m_Player1Type;
			private PlayerType m_Player2Type;
			private int _Player1Depth;
			private int _Player2Depth;
			
			/// <summary>
			/// Název souboru s rozmístěním figurek.
			/// </summary>
			public string LayoutFileName
			{
				get { return m_LayoutFileName; }
			}

			/// <summary>
			/// Barva prvního hráče.
			/// </summary>
			public PieceColor Player1Color
			{
				get { return m_Player1Color; }
			}

			/// <summary>
			/// Barva druhého hráče.
			/// </summary>
			public PieceColor Player2Color
			{
				get { return m_Player2Color; }
			}

			/// <summary>
			/// Typ prvního hráče.
			/// </summary>
			public PlayerType Player1Type
			{
				get { return m_Player1Type; }
			}

			/// <summary>
			/// Typ druhého hráče.
			/// </summary>
			public PlayerType Player2Type
			{
				get { return m_Player2Type; }
			}

			/// <summary>
			/// Vrátí hloubku přemýšlení prvního hráče.
			/// </summary>
			public int Player1Depth
			{
				get { return _Player1Depth; }
			}

			/// <summary>
			/// Vrátí hloubku přemýšlení druhého hráče.
			/// </summary>
			public int Player2Depth
			{
				get { return _Player2Depth; }
			}

			/// <summary>
			/// Vytvoří nové informace o hře.
			/// </summary>
			/// <param name="layoutFileName">Název souboru s rozmístěním figurek.</param>
			public NewGameInfo(string layoutFileName, PieceColor player1Color, PlayerType player1Type,
				PieceColor player2Color, PlayerType player2Type, int p1Depth, int p2Depth)
			{
				m_LayoutFileName = layoutFileName;
				m_Player1Color = player1Color;
				m_Player1Type = player1Type;
				m_Player2Color = player2Color;
				m_Player2Type = player2Type;
				_Player1Depth = p1Depth;
				_Player2Depth = p2Depth;
			}
		}

		/// <summary>
		/// Vytvoří nové okno nastavení nové hry.
		/// </summary>
		public NewGameForm()
		{
			InitializeComponent();

			// nacteme vestavena rozmisteni
			LoadBuiltInLayouts();
		}

		/// <summary>
		/// Přidá do seznamu vestavěnou koncovku.
		/// </summary>
		protected void AddBuiltInLayout(string fileName)
		{
			// ziskame nazev
			string nazev = Path.GetFileNameWithoutExtension(fileName);

			// pridame do seznamu
			builtInLayoutsListBox.Items.Add(nazev);

			// a do interniho seznamu
			_BuiltInLayouts.Add(nazev, fileName);
		}

		/// <summary>
		/// Načte do seznamu vestavěné koncovky ve formátu FEN.
		/// </summary>
		protected void LoadBuiltInLayouts()
		{
			// zjistime slozku ve ktere jsme
			string slozka = Path.GetDirectoryName(Application.ExecutablePath);
			slozka = Path.Combine(slozka, "Koncovky");
			// ziskame vsechny soubory FEN
			string[] soubory = Directory.GetFiles(slozka, "*.fen");

			// vytvorime interni seznam
			_BuiltInLayouts = new Dictionary<string, string>(soubory.Length);

			// postupne je pridame do seznamu
			foreach(string soubor in soubory)
				AddBuiltInLayout(soubor);

			// nakonec seznam seradime
			builtInLayoutsListBox.Sorted = true;

			// a vybereme prvni
			builtInLayoutsListBox.SelectedIndex = 0;
		}

		/// <summary>
		/// Zobrazí dialog nastavení nové hry.
		/// </summary>
		/// <param name="ownerForm">Formulář, který bude rodičem zobrazeného.</param>
		/// <returns>Informace o zvolené hře, nebo null.</returns>
		public NewGameInfo ShowNewGameDialog(Form ownerForm)
		{
			// zobrazime se
			DialogResult dr = this.ShowDialog(ownerForm);

			if(dr == DialogResult.OK)
			{
				string soubor;

				// vezmeme bud zadany soubor, nebo vybrane vestavene rozmisteni
				soubor = layoutSourceTabControl.SelectedIndex == 0 ?
					_BuiltInLayouts[builtInLayoutsListBox.SelectedItem.ToString()] : layoutFileTextBox.Text;
				
				// vratime info
				return new NewGameInfo(soubor,
					PieceColor.White,
					upTypeHumanRadioButton.Checked ? PlayerType.Human : PlayerType.Computer,
					PieceColor.Black,
					downTypeHumanRadioButton.Checked ? PlayerType.Human : PlayerType.Computer,
					whiteTrackBar.Value, blackTrackBar.Value);
			}

			// jinak nic
			return null;
		}

		/// <summary>
		/// Aktualizuje náhled šachovnice.
		/// </summary>
		protected void RefreshPreview(string fileName)
		{
			// pokud jsme dostali to same jak posledne, konec
			if(fileName == _OldText)
				return;

			// zapamatujem si predane
			_OldText = fileName;

			try
			{
				PieceColor barva;
				// nacteme figurky
				_PreviewChessboard = IO.LoadLayout(fileName, out barva);
			}
			catch
			{
				// doslo-li k chybe, budem delat, jakoby nic
				_PreviewChessboard = null;
				// skryjeme prvky
				previewLabel.Visible = false;
				previewPictureBox.Visible = false;
				return;
			}

			// pokud nemame malire,
			if(_PreviewPainter == null)
			{
				// vytvorime ho
				_PreviewPainter = new Painter2D(_PreviewChessboard, previewPictureBox);
				// nastavime kvalitu
				_PreviewPainter.HighQualityRendering = Properties.Settings.Default.HighQualityDrawing;
			}
			else
			{
				// jinak zmenime sachovnici malire
				_PreviewPainter.Chessboard = _PreviewChessboard;
			}

			// zobrazime prvky
			previewLabel.Visible = true;
			previewPictureBox.Visible = true;

			// nechame prekreslit
			_PreviewPainter.Refresh();
		}

		private void browseButton_Click(object sender, EventArgs e)
		{
			// zobrazime dialog
			layoutOpenFileDialog.ShowDialog();
		}

		private void layoutOpenFileDialog_FileOk(object sender, CancelEventArgs e)
		{
			// nastavime vybrany soubor
			layoutFileTextBox.Text = layoutOpenFileDialog.FileName;
			// jestli je to ok, nastavime fokus na potvrzeni
			if(startButton.Enabled)
				startButton.Focus();
		}

		private void startButton_Click(object sender, EventArgs e)
		{
			// mame-li sachovnici,
			if(_PreviewChessboard != null)
			{
				// bude to ok
				this.DialogResult = DialogResult.OK;
			}
			else
			{
				// jinak upozornime
				MessageBox.Show("Zvolte platný soubor s rozmístěním figurek.", ProductName,
					MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				layoutFileTextBox.Focus();
			}
		}

		private void builtInLayoutsListBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			// vykreslime nahled
			RefreshPreview(_BuiltInLayouts[builtInLayoutsListBox.SelectedItem.ToString()]);
		}

		private void layoutFileTextBox_TextChanged(object sender, EventArgs e)
		{
			// vykreslime nahled
			RefreshPreview(layoutFileTextBox.Text);
		}

		private void layoutSourceTabControl_SelectedIndexChanged(object sender, EventArgs e)
		{
			// podle vybrane karty aktualizujeme nahled
			if(layoutSourceTabControl.SelectedIndex == 0)
				builtInLayoutsListBox_SelectedIndexChanged(this, e);
			else
				layoutFileTextBox_TextChanged(this, e);
		}

		private void playerType_CheckedChanged(object sender, EventArgs e)
		{
			// povolime/zakazeme soupatka
			whiteTrackBar.Enabled = upTypeComputerRadioButton.Checked;
			blackTrackBar.Enabled = downTypeComputerRadioButton.Checked;
		}

		#region Technické detaily - úklid

		// uklidovy kod byl presunut do NewGameForm.Designer.cs

		//public void Dispose()
		//{
		//    // zabranime GC uvolnit nas podruhe
		//    GC.SuppressFinalize(this);
		//    // uklid
		//    Dispose(true);
		//}

		~NewGameForm()
		{
			// uklid
			Dispose(false);
		}



		//protected override void Dispose(bool disposeManaged)
		//{
		//    if(!Disposed)
		//    {
		//        try
		//        {
		//            // uklid rizenych prostredku
		//            if(disposeManaged)
		//            {
		//                // uklidime pripadneho malire
		//                if(m_PreviewPainter != null)
		//                {
		//                    m_PreviewPainter.Cleanup();
		//                    m_PreviewPainter = null;
		//                }
		//            }
		//        }
		//        finally
		//        {
		//            // uklid predka
		//            base.Dispose(disposeManaged);
		//        }
		//    }
		//    Disposed = true;
		//}

		#endregion
	}
}