﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Střelci
{
	/// <summary>
	/// Představuje obecného malíře, který vykresluje šachovnici.
	/// </summary>
	abstract class Painter
	{
		private Chessboard _Chessboard;
		private Control _OutputControl;
		private bool _HighQualityRendering = false;

		private Piece _SelectedPiece;
		private HighlightedTilesCollection _HighlightedTiles;

		/// <summary>
		/// Kolekce zvýrazněných políček.
		/// </summary>
		public class HighlightedTilesCollection : ICollection<Position>
		{
			private List<Position> _List = new List<Position>();
			private Painter _Painter;
			
			/// <summary>
			/// Vrátí malíře, jemuž kolekce přísluší.
			/// </summary>
			public Painter Painter
			{
				get { return _Painter; }
			}

			/// <summary>
			/// Vytvoří novou kolekci zvýrazněných políček.
			/// </summary>
			/// <param name="painter">Malíř, jemuž kolekce přísluší.</param>
			public HighlightedTilesCollection(Painter painter)
			{
				_Painter = painter;
			}

			#region ICollection<Position> Members

			/// <summary>
			/// Zvýrazní políčko.
			/// </summary>
			/// <param name="tile">Políčko, které má být zvýrazněno.</param>
			public void Add(Position tile)
			{
				_List.Add(tile);
				_Painter.Refresh();
			}

			/// <summary>
			/// Zruší zvýraznění všech políček.
			/// </summary>
			public void Clear()
			{
				_List.Clear();
				_Painter.Refresh();
			}

			/// <summary>
			/// Vrátí true, pokud kolekce obsahuje zadané políčko.
			/// </summary>
			/// <param name="tile">Políčko, které se hledá.</param>
			public bool Contains(Position tile)
			{
				return _List.Contains(tile);
			}

			/// <summary>
			/// Zkopíruje obsah kolekce do zadaného pole.
			/// </summary>
			/// <param name="array">Pole, do kterého se bude kopírovat.</param>
			/// <param name="arrayIndex">Počáteční index, kam se bude kopírovat.</param>
			public void CopyTo(Position[] array, int arrayIndex)
			{
				_List.CopyTo(array, arrayIndex);
			}

			/// <summary>
			/// Vrátí počet políček v kolekci.
			/// </summary>
			public int Count
			{
				get { return _List.Count; }
			}

			/// <summary>
			/// Vrátí true, pokud je kolekce jen pro čtení.
			/// </summary>
			public bool IsReadOnly
			{
				get { return false; }
			}

			/// <summary>
			/// Zruší zvýraznění daného políčka.
			/// </summary>
			/// <param name="tile">Políčko, jehož zvýraznění bude zrušeno.</param>
			/// <returns>True, pokud se podařilo odstranit, jinak false.</returns>
			public bool Remove(Position tile)
			{
				return _List.Remove(tile);
			}

			#endregion

			#region IEnumerable<Position> Members

			public IEnumerator<Position> GetEnumerator()
			{
				return _List.GetEnumerator();
			}

			#endregion

			#region IEnumerable Members

			System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
			{
				return _List.GetEnumerator();
			}

			#endregion
		}

		/// <summary>
		/// Vykreslovaná šachovnice.
		/// </summary>
		/// <remarks>
		/// Při nastavení šachovnice se vynuluje vybraná figurka.
		/// </remarks>
		public virtual Chessboard Chessboard
		{
			get { return _Chessboard; }
			set
			{
				_Chessboard = value;
				// vynulujeme vybranou (a prekreslime)
				SelectedPiece = null;
			}
		}

		/// <summary>
		/// Ovládací prvek, na který se bude partie vykreslovat.
		/// </summary>
		public virtual Control OutputControl
		{
			get { return _OutputControl; }
			set { _OutputControl = value; }
		}

		/// <summary>
		/// Vybraná figurka, nebo null.
		/// </summary>
		public virtual Piece SelectedPiece
		{
			get { return _SelectedPiece; }
			set
			{
				_SelectedPiece = value;
				// prekreslime
				Refresh();
			}
		}

		/// <summary>
		/// Vykreslovat ve vysoké kvalitě.
		/// </summary>
		public bool HighQualityRendering
		{
			get { return _HighQualityRendering; }
			set
			{
				_HighQualityRendering = value;
				// prekreslime
				Refresh();
			}
		}

		/// <summary>
		/// Vrátí kolekci zvýrazněných políček.
		/// </summary>
		public HighlightedTilesCollection HighlightedTiles
		{
			get { return _HighlightedTiles; }
		}

		/// <summary>
		/// Vytvoří nového malíře.
		/// </summary>
		protected Painter(Chessboard chessboard, Control outputControl)
		{
			// pokud nam nedali sachovnici, vyhodime vyjimku
			if(chessboard == null) throw new ArgumentNullException("chessboard");
			// stejne tak budeme chtit prvek
			if(outputControl == null) throw new ArgumentNullException("outputControl");

			// nastavime
			_Chessboard = chessboard;
			_OutputControl = outputControl;

			_HighlightedTiles = new HighlightedTilesCollection(this);
		}

		/// <summary>
		/// Převede souřadnici políčka X na souřadnici v pixelech.
		/// </summary>
		/// <param name="x">Souřadnice v políčkách.</param>
		public abstract int TileToX(int x);

		/// <summary>
		/// Převede souřadnici políčka Y na souřadnici v pixelech.
		/// </summary>
		/// <param name="y">Souřadnice v políčkách.</param>
		public abstract int TileToY(int y);

		/// <summary>
		/// Vrátí políčko pod zadanými souřadnicemi v pixelech, 
		/// nebo null, pokud tam žádné políčko není.
		/// </summary>
		/// <param name="x">Souřadnice X v pixelech.</param>
		/// <param name="y">Souřadnice Y v pixelech.</param>
		public abstract Position? GetTileFromPoint(int x, int y);

		/// <summary>
		/// Zajistí překreslení šachovnice.
		/// </summary>
		public abstract void Refresh();

		/// <summary>
		/// Vyčistí případné použité neřízené systémové zdroje.
		/// </summary>
		public abstract void Cleanup();
	}
}
