﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;

namespace Střelci
{
	/// <summary>
	/// Představuje malíře, který vykresluje šachovnici dvojrozměrně.
	/// </summary>
	class Painter2D : Painter, IDisposable
	{
		private int _ChessboardX, _ChessboardY;
		private int _ChessTileSize;

		/// <summary>
		/// Vykreslovaná šachovnice.
		/// </summary>
		/// <remarks>
		/// Při nastavení šachovnice vynuluje vybranou figurku.
		/// </remarks>
		public override Chessboard Chessboard
		{
			get { return base.Chessboard; }
			set
			{
				// nastavime vlastnost
				base.Chessboard = value;
				// aktualizujeme pozici
				RefreshChessboardPosition();
			}
		}

		/// <summary>
		/// Ovládací prvek, na který se bude partie vykreslovat.
		/// </summary>
		public override Control OutputControl
		{
			get { return base.OutputControl; }
			set
			{
				// pokud tu byl vystupni prvek,
				if(base.OutputControl != null)
				{
					// odpojime se od nej
					OutputControl.Paint -= new PaintEventHandler(m_OutputControl_Paint);
					OutputControl.Resize -= new EventHandler(m_OutputControl_Resize);
				}
				// nastavime
				base.OutputControl = value;

				// pokud mame zadany novy prvek,
				if(value != null)
				{
					// pripojime se na novy prvek
					OutputControl.Resize += new EventHandler(m_OutputControl_Resize);
					OutputControl.Paint += new PaintEventHandler(m_OutputControl_Paint);

					// aktualizujeme pozici sachovnice
					RefreshChessboardPosition();
				}
			}
		}

		/// <summary>
		/// Vytvoří nového dvojrozměrného malíře.
		/// </summary>
		/// <param name="chessboard">Šachovnice, kterou bude vykreslovat.</param>
		/// <param name="outputControl">Ovládací prvek, na který se bude šachovnice vykreslovat.</param>
		public Painter2D(Chessboard chessboard, Control outputControl) : base(chessboard, outputControl)
		{
			// ulozime hodnoty
			Chessboard = chessboard;
			OutputControl = outputControl;
		}

		void m_OutputControl_Paint(object sender, PaintEventArgs e)
		{
			// vykreslime
			Draw(e.Graphics);
		}

		protected void Draw(System.Drawing.Graphics graphics)
		{
			// stetce na policka
			Brush tmave = Brushes.MediumSeaGreen;
			Brush svetle = Brushes.LightGreen;
			Brush zluty_stetec = Brushes.Yellow;
			Brush bily_stetec = Brushes.White;
			Brush cerny_stetec = Brushes.Black;
			Brush zluty_pruhledny = new SolidBrush(Color.FromArgb(150, Color.Green));
			Pen cerne_pero = Pens.Black;
			Pen bile_pero = Pens.White;
			StringFormat sf = new StringFormat();

			try
			{
				// mame-li kreslit ve vysoke kvalite, nastavime parametry
				if(base.HighQualityRendering)
				{
					graphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.High;
				}
				else
				{
					graphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Default;
				}

				// vykreslime sachovnici
				for(int i = 0; i < Chessboard.Size; i++)
				{
					// souradnice x
					int x = TileToX(i);

					for(int j = 0; j < Chessboard.Size; j++)
					{
						// souradnice y
						int y = TileToY(j);
						// svetle policko?
						bool s = (i + j) % 2 == 0;

						// zvolime stetec
						Brush stetec;
						if(s)
							stetec = svetle;
						else
							stetec = tmave;

						// vykreslime policko
						graphics.FillRectangle(stetec, x, y, _ChessTileSize, _ChessTileSize);
					}
				}

				// projdeme zvyraznena pole
				foreach(Position pole in HighlightedTiles)
				{
					// a vykreslime je
					graphics.FillRectangle(zluty_pruhledny, TileToX(pole.X), TileToY(pole.Y), _ChessTileSize, _ChessTileSize);
				}

				// format pro zobrazeni popisku
				sf.Alignment = StringAlignment.Center;
				sf.LineAlignment = StringAlignment.Center;

				// zobrazime popisky sloupcu a radku
				for(int i = 0; i < Chessboard.Size; i++)
				{
					// sloupce nahore
					Rectangle rect = new Rectangle(TileToX(i),
												   TileToY(0) - _ChessTileSize / 2,
												   _ChessTileSize,
												   _ChessTileSize / 2);
					graphics.DrawString(Chessboard.ColumnLabels[i], OutputControl.Font, bily_stetec, rect, sf);
					// sloupce dole
					rect.Y = TileToY(Chessboard.Size);
					graphics.DrawString(Chessboard.ColumnLabels[i], OutputControl.Font, bily_stetec, rect, sf);

					// radky vlevo
					rect = new Rectangle(TileToX(0) - _ChessTileSize / 2,
										 TileToY(i),
										 _ChessTileSize / 2,
										 _ChessTileSize);
					graphics.DrawString(Chessboard.RowLabels[Chessboard.Size - i - 1], OutputControl.Font, bily_stetec, rect, sf);
					// radky vpravo
					rect.X = TileToX(Chessboard.Size);
					graphics.DrawString(Chessboard.RowLabels[Chessboard.Size - i - 1], OutputControl.Font, bily_stetec, rect, sf);
				}

				// projdeme figurky
				foreach(Piece figurka in Chessboard)
				{
					// policko
					Rectangle rect = TileRectangle(figurka.Position.X, figurka.Position.Y);

					// pokud je tato figurka vybrana, vykreslime ramecek
					if(SelectedPiece == figurka)
						graphics.FillRectangle(zluty_stetec, rect);

					// zmensime
					rect.Inflate(-_ChessTileSize / 10, -_ChessTileSize / 10);

					// vykreslime ji
					DrawPiece(graphics, rect, figurka);
				}
			}
			finally
			{
				// uklid
				sf.Dispose();
				sf = null;
				zluty_pruhledny.Dispose();
				zluty_pruhledny = null;
			}
		}

		/// <summary>
		/// Vykreslí zadanou figurku.
		/// </summary>
		protected static void DrawPiece(Graphics graphics, Rectangle rect, Piece piece)
		{
			Image obrazek;

			// zvolime podle typu
			if(piece is Pawn)
				obrazek = piece.Color == PieceColor.White ? Properties.Resources.bílý_pěšec : Properties.Resources.černý_pěšec;
			else if(piece is Bishop)
				obrazek = piece.Color == PieceColor.White ? Properties.Resources.bílý_střelec : Properties.Resources.černý_střelec;
			else if(piece is Queen)
				obrazek = piece.Color == PieceColor.White ? Properties.Resources.bílá_dáma : Properties.Resources.černá_dáma;
			else if(piece is King)
				obrazek = piece.Color == PieceColor.White ? Properties.Resources.bílý_král : Properties.Resources.černý_král;
			else
			{
				// tohle by se nemelo stat
				Debug.Assert(false, "Neznámá figurka " + piece.ToString() + " na šachovnici.");
				obrazek = null;
			}

			// vykreslime
			graphics.DrawImage(obrazek, rect);
		}

		void m_OutputControl_Resize(object sender, EventArgs e)
		{
			// obnovime pozici sachovnice
			RefreshChessboardPosition();

			// prekreslime se
			OutputControl.Invalidate();
		}

		/// <summary>
		/// Aktualizuje pozici šachovnice v závislosti na velikosti výstupního prvku.
		/// </summary>
		protected void RefreshChessboardPosition()
		{
			// pokud nemame vystupni prvek, nemuzeme nic udelat
			if(OutputControl == null)
				return;

			// velikost policka (+ nechame si misto na popisky sloupcu a rad)
			_ChessTileSize = Math.Min(OutputControl.Width, OutputControl.Height) / (Chessboard.Size + 1);
			// pozice sachovnice
			_ChessboardX = (OutputControl.Width - (_ChessTileSize * Chessboard.Size)) / 2;
			_ChessboardY = (OutputControl.Height - (_ChessTileSize * Chessboard.Size)) / 2;
		}

		/// <summary>
		/// Vrátí obdélník v pixelech ohraničující zadané políčko.
		/// </summary>
		/// <param name="x">Souřadnice X v políčkách.</param>
		/// <param name="y">Souřadnice Y v políčkách.</param>
		public Rectangle TileRectangle(int x, int y)
		{
			return new Rectangle(TileToX(x), TileToY(y), _ChessTileSize, _ChessTileSize);
		}

		/// <summary>
		/// Převede souřadnici políčka X na souřadnici v pixelech.
		/// </summary>
		/// <param name="x">Souřadnice v políčkách.</param>
		public override int TileToX(int x)
		{
			return _ChessboardX + x * _ChessTileSize;
		}

		/// <summary>
		/// Převede souřadnici políčka Y na souřadnici v pixelech.
		/// </summary>
		/// <param name="y">Souřadnice v políčkách.</param>
		public override int TileToY(int y)
		{
			return _ChessboardY + y * _ChessTileSize;
		}

		/// <summary>
		/// Vrátí políčko pod zadanými souřadnicemi v pixelech, 
		/// nebo null, pokud tam žádné políčko není.
		/// </summary>
		/// <param name="x">Souřadnice X v pixelech.</param>
		/// <param name="y">Souřadnice Y v pixelech.</param>
		public override Position? GetTileFromPoint(int x, int y)
		{
			// pokud jsme mimo,
			if(x < _ChessboardX || y < _ChessboardY ||
			   x > _ChessboardX + Chessboard.Size * _ChessTileSize ||
			   y > _ChessboardY + Chessboard.Size * _ChessTileSize)
			{
				// nic
				return null;
			}

			// prepocitame na policko
			Position pozice = new Position((x - _ChessboardX) / _ChessTileSize,
										   (y - _ChessboardY) / _ChessTileSize);

			return pozice;
		}

		/// <summary>
		/// Zajistí překreslení šachovnice.
		/// </summary>
		public override void Refresh()
		{
			OutputControl.Invalidate();
		}

		/// <summary>
		/// Vyčistí použité neřízené systémové zdroje.
		/// </summary>
		public override void Cleanup()
		{
			// uklidime
			Dispose();
		}

		#region Technické detaily - úklid

		private bool Disposed = false;

		public void Dispose()
		{
			// zabranime GC uvolnit nas podruhe
			GC.SuppressFinalize(this);
			// uklid
			Dispose(true);
		}

		~Painter2D()
		{
			// uklid
			Dispose(false);
		}

		protected virtual void Dispose(bool disposeManaged)
		{
			if(!Disposed)
			{
				// uklid rizenych prostredku
				if(disposeManaged)
				{
					// uklidime vystupni prvek
					OutputControl = null;
				}
			}
			Disposed = true;
		}

		#endregion
	}
}
